SystemOrganization addCategory: #'Squeak2JS-Cache'!
SystemOrganization addCategory: #'Squeak2JS-Server'!
SystemOrganization addCategory: #'Squeak2JS-Applications'!
SystemOrganization addCategory: #'Squeak2JS-Examples-ClassBrowser'!
SystemOrganization addCategory: #'Squeak2JS-Examples-Workspace'!
SystemOrganization addCategory: #'Squeak2JS-Widgets'!
SystemOrganization addCategory: #'Squeak2JS-Examples-BallGame'!
SystemOrganization addCategory: #'Squeak2JS-Stubs'!
SystemOrganization addCategory: #'Squeak2JS-Tests'!
SystemOrganization addCategory: #'Squeak2JS-Translator'!

OMeta2 subclass: #S2JOMeta2
	instanceVariableNames: 'currentMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

S2JOMeta2 subclass: #S2JAstReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

S2JAstReader subclass: #S2JAbstractPrinter
	instanceVariableNames: 'indentation'
	classVariableNames: 'ClassTextOutputStream JavascriptPrintStream TextOutputStream'
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JAbstractPrinter class methodsFor: 'converting' stamp: 'robi 1/17/2011 16:17'!
asJSString: aString
	"Use the squeak-library to encode the string.. if not found, use an easy escaping-algorithm."
	
	^ (aString respondsTo: #javascriptOn:)
		ifTrue: [
			JavascriptPrintStream ifNil: [ JavascriptPrintStream := TextStream on: '' ] ifNotNil: [ JavascriptPrintStream reset ].
			aString javascriptOn: JavascriptPrintStream.
			JavascriptPrintStream contents ]
		ifFalse: [ | tokens |
			tokens := aString findTokens: self jsEscapeChars keep: self jsEscapeChars.
			'"', (tokens inject: '' into: [ :all :each |
				all, 
				((self jsEscapeChars includes: each asCharacter) ifTrue: [ '\' ] ifFalse: [ '' ]), 
				each]), '"' ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/9/2011 20:23'!
convert: aBlock to: output
	
	aBlock value:
		output reset; yourself.
	^ output contents! !

!S2JAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/5/2011 21:22'!
convertClass: aClass
	
	^ self
		convertClass: aClass
		retrievingFrom: [ :methodSelector |
			self convertCompiledMethod: methodSelector ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/9/2011 17:50'!
convertClass: aClass retrievingFrom: aBlock
	"Return the converted version of the whole class. aBlock takes one parameter (a CompiledMethod) and returns
	the code for the method as string. A static (reused) output-stream is used to print the code."
	
	^ self
		convert: [ :output | self convertClass: aClass to: output retrievingFrom: aBlock ]
		to: (TextOutputStream ifNil: [ TextOutputStream := S2JTextStream for: self ])! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/18/2011 13:05'!
convertClass: aClass to: output retrievingFrom: aBlock
	
	| firstPrinted firstPrintedMemorizer |
	firstPrinted := false.
	firstPrintedMemorizer := [ :nextPartValid | nextPartValid and: [ firstPrinted or: [ firstPrinted := true. false ] ] ]. "Pretty complex. Remember, whether first line has been printed."
	output << 'Class(' << (self asJSString: (S2JSqueakAstToJsAst escapeReservedWord: aClass)) << ',' << self space << '{' << self newline.
	self
		printPart: 'superclass' with: (self printSuperclass: aClass) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceVariables' with: (self printVariables: aClass instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classInstanceVariables' with: (self printVariables: aClass class instVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classVariables' with: (self printVariables: aClass classVarNames) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'classMethods' with: (self printMethodsOf: aClass class retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer;
		printPart: 'instanceMethods' with: (self printMethodsOf: aClass retrievingFrom: aBlock) to: output guardedBy: firstPrintedMemorizer.
	output << self newline << '})'.! !

!S2JAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/5/2011 19:59'!
convertClasses: anArrayOfClasses
	
	^ self
		convertClasses: anArrayOfClasses
		retrievingFrom: [ :class |
			self convertClass: class ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/10/2011 10:20'!
convertClasses: anArrayOfClasses retrievingFrom: aBlock
	"Return the code for all the classes in anArrayOfClasses. aBlock takes on parameter (a class) and returns the code for the class."
	
	^ self
		convert: [ :output | output << (self
							printEach: [ :class | aBlock value: class ]
							of: anArrayOfClasses
							separatedBy: (';', self newline))]
		to: (ClassTextOutputStream ifNil: [ ClassTextOutputStream := S2JTextStream for: self ])! !

!S2JAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
indentation
	
	self subclassResponsibility! !

!S2JAbstractPrinter class methodsFor: 'converting' stamp: 'ag 1/5/2011 20:01'!
inputConverter

	^ S2JSqueakAstToJsAst! !

!S2JAbstractPrinter class methodsFor: 'converting' stamp: 'ag 1/17/2011 10:39'!
jsEscapeChars
	
	^ #($" $\)! !

!S2JAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
newline
	
	self subclassResponsibility! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 08:01'!
printEach: aBlock of: aCollection separatedBy: aString
	
	| result isFirst |
	isFirst := true.
	result := ''.
	aCollection do: [ :each | 
		isFirst
			ifTrue: [ isFirst := false ]
			ifFalse: [ result := result, aString ].
		result := result, (aBlock value: each)].
	^ result! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:16'!
printMethodsOf: aClass retrievingFrom: aBlock
	
	| methods |
	methods := aClass methodDict values.
	^ methods 
		ifEmpty: [ false ]
		ifNotEmpty: [
			'{', self newline,
			(self
				printEach: [ :eachmethod | | code |
					code := aBlock value: eachmethod.
					self indentation, self indentation, code key, ':', self space, code value ]
				of: methods
				separatedBy: ',', self newline),
			self newline, self indentation, '}' ].! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 10:43'!
printPart: aString with: aValue to: output guardedBy: aGuardBlock
	"Print a slot. If aValue is a String (should return false then), this is skipped.
	aGuardBlock is called with whether this part will be printed and returns, whether we have to prepend a colon + newline ('guards' whether we are the first line)."
	
	(aGuardBlock value: aValue isString)
		ifTrue: [ output << ',' << self newline ].
	aValue isString ifTrue: [
		output << self indentation << aString << ':' << self space << aValue ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 08:00'!
printSuperclass: aClass
	
	^ aClass superclass
		ifNil: [ false ]
		ifNotNil: [ S2JSqueakAstToJsAst escapeReservedWord: aClass superclass name ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes private' stamp: 'ag 1/10/2011 09:12'!
printVariables: aCollection
	"Print a collection containing all members of aCollection separated by colons"
	
	^ aCollection
		ifEmpty: [ false ]
		ifNotEmpty: [
			'[', self space,
			(self
				printEach: [ :each | '"', S2JSqueakAstToJsAst escapeCharForIdentifier, each, '"' ]
				of: aCollection
				separatedBy: ',', self space),
			self space, ']' ]! !

!S2JAbstractPrinter class methodsFor: 'converting classes' stamp: 'ag 1/10/2011 09:10'!
resetTextStreams
	"These helper-TextStreams are recreated lazily"
	
	ClassTextOutputStream := TextOutputStream := JavascriptPrintStream := nil.! !

!S2JAbstractPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:01'!
space
	
	self subclassResponsibility! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/8/2010 14:44'!
arguments =
	
	translate*:identifiers
	-> [ self functionParameters: identifiers ]! !

!S2JAbstractPrinter methodsFor: 'translate tokens' stamp: 'ag 1/25/2011 14:44'!
array = 
	
	translateWith(#expressionBase)*:contents
	-> [ '[', (contents separatedBy: ', '), ']' ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/9/2010 16:19'!
assignment =
	
	translate! !

!S2JAbstractPrinter methodsFor: 'pretty print' stamp: 'ag 1/10/2011 09:17'!
cSpace
	
	^ self class space! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/10/2011 09:49'!
expression =
	
	expressionBase:expr
	-> [ self indentation, expr ]! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'ag 1/25/2011 14:43'!
expression: anExpressionBodyString withAssignments: aStringCollection
	
	^ ((aStringCollection isNil or: [ aStringCollection isEmpty ])
			ifTrue: [ '' ]
			ifFalse: [
				(aStringCollection separatedBy: self cSpace, '=', self cSpace),
				self cSpace, '=', self cSpace ])
					, anExpressionBodyString! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/13/2011 12:02'!
expressionBase =
	
	translate:expressionBody translate*:assignments
	-> [ self expression: expressionBody withAssignments: assignments ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/24/2011 19:13'!
functionCall =
	// Can be either an invokation of a named function of an object OR an invokation of a anonymous function
	
	&anything:receiverNode (nil | translate):receiver (^string | translate):functionName translate:arguments
	-> [ self
			functionCall: functionName
			to: ((receiverNode notNil and: [ receiverNode name = 'functionDeclaration' ])
					ifTrue: [ '(', receiver, ')' ]
					ifFalse: [ receiver ] )
			withArguments: arguments ]! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'ag 1/10/2011 09:22'!
functionCall: aFunctionNameOrDeclaration to: aReceiverString withArguments: anArgumentsString
	
	| function |
	function := (aFunctionNameOrDeclaration name = 'functionDeclaration') 
		ifTrue: ['(', aFunctionNameOrDeclaration, ')'] 
		ifFalse: [aFunctionNameOrDeclaration].
	
	^ (aReceiverString
		ifNotNil: [ aReceiverString, '.' ]
		ifNil: [ '' ]), function, anArgumentsString! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'SW 1/25/2011 16:56'!
functionContent =
	// This is intentionally not named 'statements', to deny all nested statements-nodes
	// The ast-to-ast step must flatten all statements into one list of expressions here
	
	[ self indent ]
	translate*:expressions
	[ self unindent ]
	-> [ expressions separatedBy: ';', self newline ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/10/2011 09:38'!
functionDeclaration =
	
	translate:arguments
	translate:statements
	-> [ self functionDeclaration: arguments withBody: statements ]! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'ag 1/26/2011 10:05'!
functionDeclaration: aParameterString withBody: aStatementsString
	"aParameterString already contains the parantheses; #functionParameters: should be used."
	
	^ 'function', self cSpace, aParameterString, self cSpace, '{', self newline,
			(aStatementsString ifEmpty: [ '' ] ifNotEmpty: [ aStatementsString, ';' ]),
		self newline, self indentation, '}'! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'ag 1/25/2011 14:43'!
functionParameters: aStringCollection
	
	^ '(', (aStringCollection separatedBy: ', '), ')'! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'sec 1/4/2011 11:46'!
hasSelector: aSymbol
	"Adding the superclass method dictionary to the searcharea for matching methods is required because the printers don't implement all the methods on their own"
	
	^ ((super hasSelector: aSymbol) or: [ self class superclass includesSelector: aSymbol ])
		ifTrue: [ true ]
		ifFalse: [ self error: 'Unknown selector: ', aSymbol asString. false ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/13/2010 14:14'!
header =
	
	^string:functionName arguments:arguments
	-> [ functionName -> arguments ]! !

!S2JAbstractPrinter methodsFor: 'translate tokens' stamp: 'ag 12/2/2010 11:52'!
identifier =
	
	^string:identifier
	-> [ identifier ]! !

!S2JAbstractPrinter methodsFor: 'pretty print' stamp: 'ag 1/10/2011 09:17'!
indent
	
	self indentation: self indentation, self class indentation! !

!S2JAbstractPrinter methodsFor: 'accessing' stamp: 'lw 1/4/2011 17:25'!
indentation
	^ indentation! !

!S2JAbstractPrinter methodsFor: 'accessing' stamp: 'lw 1/4/2011 17:25'!
indentation: anObject
	indentation := anObject! !

!S2JAbstractPrinter methodsFor: 'initialization' stamp: 'lw 1/4/2011 17:25'!
initialize

	super initialize.
	self indentation: ''.! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/13/2010 19:37'!
method =
	// This does not return a string, but an association from the method-name to the method-declaration
	
	translate:functionNameAndArguments
	translate:statements
	-> [ functionNameAndArguments key -> 
		(self
			functionDeclaration: functionNameAndArguments value
			withBody: statements) ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/18/2011 10:02'!
newExpression =
	
	translate:functionCall
	-> [ 'new ', functionCall ]! !

!S2JAbstractPrinter methodsFor: 'pretty print' stamp: 'ag 1/10/2011 09:35'!
newline
	
	^ self class newline! !

!S2JAbstractPrinter methodsFor: 'translate tokens' stamp: 'ag 12/16/2010 16:32'!
number =
	
	^number:number
	-> [ number asString ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/10/2011 10:00'!
paranthesizedExpression =
	
	expressionBase:value
	-> [ '(', value, ')' ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/13/2010 17:37'!
reservedWord =
	
	^string! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 1/10/2011 09:49'!
returnExpression =
	
	expressionBase:expr
	-> [ self indentation, 'return ', expr ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/22/2010 16:36'!
slotAccess =
	
	(nil | translate):object ^symbol:slotName
	-> [ (object ifNotNil: [ object, '.' ] ifNil: [ '' ]), slotName asString ]! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'LT 1/10/2011 16:37:03.614'!
slotAssignment =
	
	(nil | translate):object translate:value
	-> [ (object ifNotNil: [ object, self cSpace, '=', self cSpace ] ifNil: [ '' ]), value ]! !

!S2JAbstractPrinter methodsFor: 'translate tokens' stamp: 'ag 1/9/2011 16:45'!
string =
	
	^string:string
	-> [ self class asJSString: string ]! !

!S2JAbstractPrinter methodsFor: 'pretty print' stamp: 'ag 1/10/2011 09:17'!
unindent
	
	self indentation: (self indentation allButLast: self class indentation size)! !

!S2JAbstractPrinter methodsFor: 'translate complex' stamp: 'ag 12/13/2010 19:40'!
variableDeclaration =
	
	translateNamed('identifier')+:variableNames translateNamed('assignment')?:assignment
	-> [ self variableDeclaration: variableNames withAssignment: assignment ]! !

!S2JAbstractPrinter methodsFor: 'helpers' stamp: 'ag 1/25/2011 14:43'!
variableDeclaration: variableNames withAssignment: assignment
	"Acts as an expression, not terminated by a semi colon"
	
	| oneVar firstLine |
	firstLine := self indentation, 'var ', ((oneVar := variableNames size = 1)
		ifTrue: [ variableNames first ]
		ifFalse: [ (variableNames separatedBy: ';', self cSpace, 'var ')]).
	^ assignment 
		ifNil: [ firstLine ]
		ifNotNil: [
			oneVar
				ifTrue: [ self expression: assignment withAssignments: { firstLine } ]
				ifFalse: [ 
					firstLine, ';', self newline, self indentation, (self expression: assignment withAssignments: variableNames) ]]! !

S2JAbstractPrinter subclass: #S2JPrettyPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ '    '! !

!S2JPrettyPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/10/2011 09:13'!
newline
	
	^ '
'! !

!S2JPrettyPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
space

	^ ' '! !

!S2JPrettyPrinter methodsFor: 'initialization' stamp: 'ag 1/10/2011 10:11'!
initialize
	"Do everything indented x2 from the start, to have json-like pretty-print for the whole js-file."
	
	super initialize.
	self
		indent;
		indent.! !

S2JAbstractPrinter subclass: #S2JPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:37'!
indentation

	^ ''! !

!S2JPrinter class methodsFor: 'pretty-print' stamp: 'ag 1/5/2011 20:02'!
newline
	
	^ ''! !

!S2JPrinter class methodsFor: 'pretty-print' stamp: 'lw 1/4/2011 17:30'!
space

	^ ''! !

!S2JAstReader class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:20'!
applyRule: aRule on: aMatcher
	
	^ super applyRule: #translate on: aMatcher! !

!S2JAstReader class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:13'!
createInputFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ { self inputConverter convert: aString as: aRule insideMethod: aCompiledMethod }! !

!S2JAstReader class methodsFor: 'converting private' stamp: 'ag 12/8/2010 22:13'!
inputConverter
	"An AstReader should know, which converter-class can provide the ast wants to read"
	
	self subclassResponsibility! !

!S2JAstReader methodsFor: 'helpers' stamp: 'ag 12/16/2010 23:37'!
hasSelector: aSymbol
	"Offers a hook to catch, if an unknown node occurs"
	
	^ self class includesSelector: aSymbol! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:36'!
translate =
	// Translate the next node using it's name as rule
	
	translateNamed(nil)! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:36'!
translateNamed :nodeName =
	// Assert, that the next node has the name nodeName and translate it using it's name as rule
	
	translateNamedWith(nodeName. nodeName)! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:36'!
translateNamedWith :nodeName :targetRule =
	// Assert, that the next node has the name nodeName and translate it with targetRule
	
	translateNamedWithArgument(nodeName. targetRule. nil)! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:35'!
translateNamedWithArgument :nodeName :targetRule :argument =
	// Assert, that the next node has the name nodeName, use targetRule to convert the node and pass argument when calling targetRule
	
	// Lookahead for one node
	&anything:node
	
	// Some assertions, e.g. about the nodeName
	?[ node class = S2JNode ]
	?[ nodeName isNil | (node name = nodeName )]
	[ targetRule ifNil: [ node name asSymbol ] ifNotNil: [ targetRule asSymbol ]]:appliedRule
	?[ self hasSelector: appliedRule ]
	
	// If an argument is given, call the targetRule with that argument. Else, call the targetRule without arguments
	(( ?[ argument notNil ] { apply(appliedRule. argument):result } )
	| { apply(appliedRule):result })
	
	// Return the parsed content of the node
	-> [ result ]! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:34'!
translateWith :aRule =
	// Translate the next node with aRule, regardless of it's name
	
	translateNamedWith(nil. aRule)! !

!S2JAstReader methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:47'!
translateWithArgument :aRule :argument =
	// Translate the next node with aRule, regardless of it's name, and pass aRule the argument
	
	translateNamedWithArgument(nil. aRule. argument)! !

S2JAstReader subclass: #S2JAstToAst
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JAstToAst methodsFor: 'helpers' stamp: 'ag 12/22/2010 13:45'!
passThroughAndTranslate =
	
	// TODO add all primitive values?..
	
	^symbol // use the primitive symbol rule, not the rule defined in this class
|	string
|	number
|	char
|	nil
	
|	({ translate*:children }:node
	-> [ self node: node name with: children ])! !

!S2JAstToAst methodsFor: 'helpers' stamp: 'ag 12/9/2010 12:37'!
renameTo :nodeName =
	
	translate*:contents
	-> [ self node: nodeName with: contents ]! !

!S2JAstToAst methodsFor: 'helpers' stamp: 'ag 12/9/2010 12:31'!
translate = 
	
	^translate
|	passThroughAndTranslate! !

S2JAstToAst subclass: #S2JJsAstOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JJsAstOptimizer class methodsFor: 'converting' stamp: 'ag 12/8/2010 21:30'!
inputConverter
	
	self error: 'The Js-Ast-Optimizer is not yet functional!!'.
	^ S2JSqueakToJsAst! !

!S2JJsAstOptimizer methodsFor: 'convert special cases' stamp: 'ag 12/8/2010 10:38'!
forLoop =
	// Special case of keywordExpression
	

	translate:receiver symbol:selector translate*:arguments
	?[ selector = #do: ]
	?[ arguments size = 1 ]
	[ arguments first ]:block
	?[ block size >= 1 ]
	?[ block first name = 'arguments' ]
	?[ block first size = 1 ]
	-> [ self forLoopOver: receiver do: block ]! !

!S2JJsAstOptimizer methodsFor: 'convert special cases' stamp: 'ag 12/8/2010 11:03'!
forLoopOver: anArrayNode do: aBlockNode

	| arrayName statements aStatementsNode aTemporariesNode loopVariableName |
	arrayName := 'arr$'.
	loopVariableName := aBlockNode first first.
	aStatementsNode := aBlockNode detect: [:each | each name = 'statements'] ifNone: [ nil ].
	aTemporariesNode := aBlockNode detect: [:each | each name = 'temporaries'] ifNone: [ nil ].
	aStatementsNode ifNotNil: [ statements := self apply: #translate withArgs: aStatementsNode].
	^self node: 'statements' with:
		{self node: 'variableDeclaration' with:
			{self node: 'identifier' with: { arrayName }.
			  self node: 'expression' with: anArrayNode}.
		 self node: 'forLoop' withoutNils:
			{self node: 'identifier' with: { arrayName }.
			loopVariableName.
			aTemporariesNode.
			statements}}! !

!S2JJsAstOptimizer methodsFor: 'convert complex' stamp: 'ag 12/8/2010 13:54'!
keywordExpression = 
	
	forLoop
|	messageSendExpression // ...not correct ...
! !

S2JAstToAst subclass: #S2JSqueakAstToJsAst
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/11/2011 17:35'!
arrayFunction
	
	^ 'array'! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
blockFunction
	
	^ 'block'! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
charFunction
	"Cannot name it char"
	
	^ 'character'! !

!S2JSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForIdentifier
	
	^ '$'! !

!S2JSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'robi 1/4/2011 15:21'!
escapeCharForMethod
	
	^ '_'! !

!S2JSqueakAstToJsAst class methodsFor: 'escaping' stamp: 'sec 1/5/2011 11:54'!
escapeReservedWord: aStringOrSymbol
	
	aStringOrSymbol ifNil: [ ^nil ].
	^ ((self jsReservedWords includes: aStringOrSymbol asString)
		ifTrue: [ self escapeCharForMethod ]
		ifFalse: [ '' ]), aStringOrSymbol asString! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'ag 1/17/2011 20:21'!
globalFunctionReceivers
	
	^ Dictionary newFrom: {
		JsGlobal name -> nil.
		JsDocument name -> (S2JNode named: 'identifier' with: { 'document' })
	}! !

!S2JSqueakAstToJsAst class methodsFor: 'converting' stamp: 'ag 2/1/2011 13:48'!
ignoredPrimitives
	"This returns an array of CompiledMethod. All primitive-declarations in these methods are ignored.
	This means, these methods must have a correct software-fallback-implementation."
	
	^ {
	self method: #@ of: Number.
	self method: #replaceFrom:to:with:startingAt: of: String.
	self method: #replaceFrom:to:with:startingAt: of: Symbol.
	self method: #replaceFrom:to:with:startingAt: of: Array.
	self method: #replaceFrom:to:with:startingAt: of: ByteString.
	self method: #atEnd of: PositionableStream.
	self method: #next of: ReadStream
	}! !

!S2JSqueakAstToJsAst class methodsFor: 'converting' stamp: 'ag 12/8/2010 18:14'!
inputConverter
	
	^ S2JParser! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
jsFunction
	
	^ 'js'! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'ag 1/5/2011 09:43'!
jsFunctionNameFor: aStringOrSymbol
	
	| result |
	result := ''.
	aStringOrSymbol asString do: [ :each | | jsIdentifier |
		jsIdentifier := self squeakCharacterToJsIdentifier at: each ifAbsent: [each asString].
		result := result, jsIdentifier].
	^ self escapeReservedWord: result! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'lw 1/13/2011 12:58'!
jsIdentifierFor: aString inMethod: aCompiledMethod
	"Check, if the identifier is a special global variable (JsGlobal, document, etc.) and escape reserved js-words"
	
	^ self globalFunctionReceivers
		at: aString
		ifAbsent: [
			(aCompiledMethod notNil and: [ aCompiledMethod methodClass allInstVarNames includes: aString ])
				ifTrue: [
					S2JNode named: 'slotAccess' with: { 
						S2JNode named: 'reservedWord' with: { 'this' }. 
						(self escapeCharForIdentifier asString, aString) asSymbol } ]
				ifFalse: [	
					S2JNode named: 'identifier' with: { self escapeReservedWord: aString } ]]! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'ag 2/7/2011 19:52'!
jsKeywordFor: aString
	"thisContext is not covered, just a stub leading to a not-defined-error.
	The returned association maps from the literal evaluating to the squeak-object to the literal evaluating to the underlying js-object."
	
	^ aString
		caseOf: {
			[ 'self' ] -> [ 'this' -> 'this' ].
			[ 'true' ] -> [ (self escapeReservedWord: 'true') -> 'true' ].
			[ 'false' ] -> [ (self escapeReservedWord: 'false') -> 'false' ].
			[ 'nil' ] -> [ 'nil' -> 'null' ].
			[ 'super' ] -> [ 'this' -> 'this' ]. "In Squeak, using super as identifier (e.g. a := super. ) without calling functions on it, seems to have the same effect as using self."
			[ 'thisContext' ] -> [ 'THIS_CONTEXT' -> 'THIS_CONTEXT' ].
		}
		otherwise: [ self error: 'Cannot translate keyword to JS: ', aString ]! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'ag 1/11/2011 17:34'!
jsReservedWords
	"http://javascript.about.com/library/blreserved.htm"
	
	^ #('abstract' 'as' 'boolean' 'break' 'byte' 'case' 'catch' 'char' 'class' 'continue' 'const' 'debugger' 'default' 'delete' 'do' 'double' 'else' 'enum' 'export' 'extends' 'false' 'final' 'finally' 'float' 'for' 'function' 'goto' 'if' 'implements' 'import' 'in' 'instanceof' 'int' 'interface' 'is' 'long' 'namespace' 'native' 'new' 'null' 'package' 'private' 'protected' 'public' 'return' 'short' 'static' 'super' 'switch' 'synchronized' 'this' 'throw' 'throws' 'transient' 'true' 'try' 'typeof' 'use' 'var' 'void' 'volatile' 'while' 'with' 'arguments'
	
	"Classes, that already exist in javascript, should not be overridden, our squeak-version of these classes will be escaped."
	'Object' 'Function' 'Number' 'Array'
	)! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
numberFunction
	
	^ 'number'! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
primitiveCallFunction
	
	^ 'primitiveDeclaration'! !

!S2JSqueakAstToJsAst class methodsFor: 'mapping-identifiers' stamp: 'ag 1/8/2011 14:30'!
squeakCharacterToJsIdentifier
	"Several characters can be used for selectors in squeak but
	are not allowed for JavaScript functionname"
	
	"TODO check, if one of these might be a reserved word (would cause conflicts with unary message named that way). Builds exception-rule."
	
	^ (Dictionary newFrom:
		{$: -> ''.  
		$+ -> 'plus'.
		$- -> 'minus'.
		$* -> 'times'.
		$/ ->  'slash'.
		$\ -> 'backslash'.
		$~ -> 'tilde'.
		$< ->  'less'.
		$> -> 'greater'.
		$= -> 'equals'.
		$@ -> 'at'.
		$% -> 'percent'.
		$& -> 'and'.
		$? -> 'question'.
		$!! -> 'exclamation'.
		$` -> 'backtick'.
		$, -> 'comma'. 
		$| -> 'pipe'})
			collect: [ :string | self escapeCharForMethod, string ]! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/10/2011 11:51'!
stringFunction
	
	^ 'string'! !

!S2JSqueakAstToJsAst class methodsFor: 'special functions' stamp: 'ag 1/24/2011 18:42'!
superFunction
	
	^ '_super'! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/10/2011 10:09'!
addReturnExpression: statements
	
	(statements isEmpty or: [(statements last name = 'returnExpression') not])
		ifTrue: [
			statements add: (self node: 'returnExpression' with: { self node: 'reservedWord' with: { 'this' } })].! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 12/9/2010 13:07'!
arguments =
	
	renameTo('arguments')! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/13/2011 12:01'!
array =
	// Pack inner nodes in expression-nodes
	
	translate*:elements
	arrayBase(elements collect: [ :each | self node: 'expression' with: { each }])! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 1/13/2011 12:01'!
arrayBase :collection =
	
	wrapAsPrimitive(
		self class arrayFunction.
		'array'.
		collection)! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 1/3/2011 14:18'!
binaryExpression =
	
	messageSendExpression! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 1/18/2011 15:22'!
block =
	
	translateNamed('arguments')?:arguments
	methodContent:statements
	
	// All return-statements in the block are nonlocal returns. After that, the last expression into a return-expression like in a method.
	// Nonlocal returns are function calls in JS: nonLocalReturn()
	[ self renameNonLocalReturns: statements ]
	renameLastExpression(statements)
	
	-> [self node: 'functionCall' with: 
		{ nil.
		self class blockFunction.
		self node: 'arguments' with: 
			{self node: 'functionDeclaration' withoutNils: 
				{( arguments ifNotNil: [ arguments ] ifNil: [ self node: 'arguments' ] ).
				statements }}}]! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 2/7/2011 21:28'!
braceExpression =
	// take the contents of the expression as it's js-arg-value. In a brace-expression, there is no assigment.
	
	translate*:expressions
	[ expressions do: [ :each | each first jsArgumentValue ifNotNil: [ each jsArgumentValue: each first jsArgumentValue ] ] ]
	arrayBase(expressions)! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 12/16/2010 23:39'!
byteArray =
	
	array! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/17/2011 20:17'!
cascadeReceiver
	
	^ self node: 'identifier' with: { 'cascadeReceiver', self class escapeCharForIdentifier }! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 12/16/2010 23:47'!
cascadedExpression =
	
	translate:receiver
	translateWithArgument(#messageSendExpression. self cascadeReceiver)*:functions
	-> [ self cascadedExpression: functions to: receiver ]! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/19/2011 16:00'!
cascadedExpression: functions to: receiver
	"The statements-node created here is later flattened into one flat list of expressions."
	
	| statements |
	statements := self node: 'functionDeclaration' with:
		{ self node: 'arguments'. self node: 'functionContent' with: { 
			self node: 'variableDeclaration' with: { self cascadeReceiver. 
				self node: 'assignment' with: { receiver }
				} } }.
	functions do: [ :each | statements last add: (self node: 'expression' with: { each })].
	statements last last name: 'returnExpression'.
	^ self
		functionCall: 'apply'
		to: statements
		withArguments: { self node: 'reservedWord' with: { 'this' } }! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/24/2011 10:29'!
character =
	
	char:char
	wrapAsPrimitive(self class charFunction. 'string'. char asString) // the 'string' here is the node inside the character()-function call. The Printer does not know about characters.
	! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/3/2011 16:53'!
checkSingleArgumentSelector: aSymbol
	"Answer, whether aSymbol is a selector for a keyword-message with one argument.
	aSymbol is already is keyword-selector.
	TODO Please implement a better way to do this."
	
	| tokens |
	tokens := aSymbol findTokens: $:.
	^ tokens size = 1 and: [ tokens first = (aSymbol asString subStrings: ':') first ]! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 2/7/2011 21:20'!
convertToJsArguments: arguments
	
	^ arguments collect: [ :argument |
			argument jsArgumentValue
				ifNil: [
					self
						functionCall: self class jsFunction
						to: argument
						withArguments: {} ]
				ifNotNil: [ argument jsArgumentValue ] ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 12/16/2010 23:12'!
doItSnippet =
	// No need to turn the last expression into a returnExpression, like in a method
	
	methodContent! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/24/2011 11:29'!
functionCall: selector to: receiver withArguments: arguments
	
	| functionName |
	functionName := selector isString ifTrue: [ self class jsFunctionNameFor: selector ] ifFalse: [ selector ].
	^ self node: 'functionCall' with: {
		receiver.
		functionName.
		self node: 'arguments' with: arguments }! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/8/2011 18:22'!
identifier =
	
	^string:name
	-> [ self class jsIdentifierFor: name inMethod: self currentMethod ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 2/7/2011 21:05'!
jsMethodArguments =
	
	(
	translateNamed('braceExpression'):arguments
	-> [ arguments last first ] // Unpack the array-contents
	
|	translate:singleArgument
	-> [ { singleArgument } asOrderedCollection ]
	):arguments
	
	-> [ self convertToJsArguments: arguments ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 2/7/2011 21:44'!
jsMethodCall :receiver =
	// Read a selector and the arguments for a js-method-call. Wrap everything into a functionCall-node
	// If a js-method-call is a parameter for another js-method-call, the js()-message will be skipped
	
	^symbol:selector
	?[ self checkSingleArgumentSelector: selector ]
	jsMethodArguments:args
	
	[ self
		functionCall: (selector subStrings: ':') first asString
		to: receiver
		withArguments: args ]:result
	-> [ result
			jsArgumentValue: result;
			yourself ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 2/7/2011 21:03'!
jsNewExpression =
	// An exception for a keywordExpression.
	// If the receiver of this message is a unary message with the selector #js, this results in a javascript native new-expression.
	// JsDocument js ctk js Graphics jsNew: { 1 }   -->   new ctk.Graphics(1)
	
	specialMessage('jsNew:'):function
	jsMethodArguments:arguments
	
	// Transform the slot-access to a method-call
	[ function
		name: 'functionCall';
		add: (self node: 'arguments' with: arguments )
	]
	-> [ self node: 'newExpression' with: { function }  ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 1/24/2011 10:44'!
keywordExpression = 
	
	jsNewExpression // JsDocument js ctk js Graphics jsNew: { 'a' }   -->   new document.ctk.Graphics(string("a"))
|	newExpression // #new Option: { 'a' }   -->   new Option(string("a"))
|	slotAssignment // a js b js: 'a'   -->   a.b = string("a")
|	slotMethodCall // a js abc: { 'a' }   -->   a.abc(string("a"))
|	messageSendExpression
! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 12/22/2010 14:47'!
messagePattern =
	
	^symbol:selector translate*:arguments
	-> [ self node: 'header' with: { self class jsFunctionNameFor: selector }, arguments ]! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 1/24/2011 19:06'!
messageSendExpression =
	
	superCall
	
|	(translate:receiver ^symbol:selector translate*:arguments
	-> [ self functionCall: selector to: receiver withArguments: arguments ])! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'sec 1/5/2011 10:03'!
method =
	
	translateWith(#messagePattern):header
	methodContent:statements
	[ self addReturnExpression: statements ]
	
	-> [ self node: 'method' with: { header. statements }]! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'srv 1/5/2011 11:48'!
methodContent  =
	// This can also be used in a place, where primitive-declarations are not allowed: the parser already handles that correctly
	// This flattens all expressions (cascaded expressions in the first place) into one flat collection of expressions
	
	translateNamed('temporaries')?:temporaries
	translateNamed('primitiveDeclaration')*:primitives
	translate?:statements
	-> [ self statements: statements withTemps: temporaries andPrimitives: primitives ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 1/18/2011 10:04'!
newExpression =
	// Exception for a keywordMessage. Translated to the Javascript-native 'new'-statements
	// #new Option: { 'abc' }   ->   new Option("abc")
	
	{ ^symbol:symbol }:symbolNode ?[ symbolNode name = 'symbol' & (symbol = 'new' asSymbol) ]
	jsMethodCall(nil):functionCall
	-> [ self node: 'newExpression' with: { functionCall } ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/9/2011 15:37'!
number =
	// Number-literals are packed into a 'primitive' number-function-call
	
	^number:number
	wrapAsPrimitive(self class numberFunction. 'number'. number)! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 2/7/2011 21:40'!
paranthesizedExpression =
	// I just found this method empty (without comment), don't really remember, why this works, but it does.
	
	translate:content
	-> [ content
		jsArgumentValue: content jsArgumentValue;
		yourself ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 2/7/2011 20:53'!
primitiveDeclaration =
	// Some primitive-declarations are just ignored, because the software implementation is perfectly fine.
	
	(?[ self class ignoredPrimitives includes: self currentMethod ]
	anything*
	-> [ nil ])
|	
	(anything // ignored for now. No need to remember the primitive-number, the js-implementation will know which method this is in
	-> [ self node: 'expression' with:
		{ self functionCall: self class primitiveCallFunction to: nil withArguments: {} } ])! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 1/10/2011 11:16'!
renameLastExpression :statements =
	
	[ statements ifNotEmpty: [ 
		(statements last name = 'expression'
			and: [statements last first name = 'functionCall']
				and: [statements last first second = 'nonLocalReturn'])
		 			ifFalse: [ statements last name: 'returnExpression' ] ] ]! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/10/2011 11:12'!
renameNonLocalReturns: statements
	
	^ statements doWithIndex: [ :st :index |
		st name = 'returnExpression' 
			ifTrue: [ | contents |
				st name: 'expression'.
				contents := st first.
				st removeAll; add: (self functionCall: 'nonLocalReturn' to: nil withArguments: { contents }) ]]! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 2/7/2011 19:49'!
reservedWord =
	
	^string:keyword
	[ self class jsKeywordFor: keyword ]:jsAndSqueakKeyword
	[ self
		node: 'reservedWord'
		with: { jsAndSqueakKeyword key } ]:result
	[ result jsArgumentValue:
		(self
			node: 'reservedWord'
			with: { jsAndSqueakKeyword value })]
	
	-> [ result ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 1/19/2011 19:24'!
serverBlock =
	
	&anything:block
	translateNamed('block') ^symbol:selector
	?[ selector = 'onServer' asSymbol ]
	-> [ self
		functionCall: 'serverBlock'
		to: nil
		withArguments: { self node: 'string' with: { block originalCode } } ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 1/24/2011 11:22'!
slotAccess =
	// An exception for a unaryMessage.
	// If the receiver of this message is a unary message with the selector #js
	// this is translated to a js-slot-access:   a js b   -->   a.b
	
	translateWithArgument('specialMessage'. 'js'):receiver
	^symbol:selector
	-> [ self node: 'slotAccess' with: { receiver. selector } ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 2/7/2011 21:06'!
slotAssignment =
	// An exception for a keywordExpression.
	// If the receiver of this message is a unary message with the selector #js
	// this is translated to a js-slot-assignment.
	
	specialMessage('js:'):receiver
	translate:argument
	-> [ self
		node: 'slotAssignment'
		with: { 
			receiver.
			(self convertToJsArguments: { argument }) first }  ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 1/24/2011 12:13'!
slotMethodCall =
	// An exception for a keywordMessage.
	// If the receiver of this message is a unary message with the selector #js
	// AND the single argument of this message is an array-literal,
	// this is translated to a method-call.
	
	translateWithArgument('specialMessage'. 'js'):receiver
	jsMethodCall(receiver)! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 1/24/2011 11:25'!
specialMessage :requiredSelector =
	// reads one message with the given selector  Returns the receiver.
	
	translate:receiver
	^symbol:selector
	?[ selector = requiredSelector asSymbol ]
	-> [ receiver ]! !

!S2JSqueakAstToJsAst methodsFor: 'helpers' stamp: 'ag 1/7/2011 23:06'!
statements: aNode withTemps: anElement andPrimitives: anElementCollection
	"All nils in the primitives-collection are removed (can occur, see #primitiveDeclaration)"
	
	| result |
	result := aNode ifNil: [ self node: 'statements' ] ifNotNil: [ aNode ].
	anElement ifNotNil: [ result addFirst: anElement; yourself ].
	anElementCollection ifNotNil: [
		result addAllFirst: (anElementCollection reject: [ :each | each isNil ]); yourself ].
	^ result name: 'functionContent'; yourself! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/9/2011 15:37'!
string =
	// String/Symbol-literals are packed into a 'primitive' string-function-call
	
	^string:string // Also covers symbols
	wrapAsPrimitive(self class stringFunction. 'string'. string asString)! !

!S2JSqueakAstToJsAst methodsFor: 'convert exceptions' stamp: 'ag 1/24/2011 19:09'!
superCall =
	// An exception to a message-send-expression: super-calls are translated in this way.
	
	{ ^string:superKeyword }:superNode ^symbol:selector translate*:arguments
	?[ superNode name = 'reservedWord' & superKeyword = 'super' ]
	-> [ 
		self
			functionCall: 
				(self
					functionCall: self class superFunction
					to: nil
					withArguments: { self node: 'string' with: { self class jsFunctionNameFor: selector asString } })
			to: nil
			withArguments: arguments ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert tokens' stamp: 'ag 1/8/2011 16:57'!
symbol =
	
	string! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 12/16/2010 12:31'!
temporaries =
	
	renameTo('variableDeclaration'):temporaries
	[ temporaries ifEmpty: [ nil ] ifNotEmpty: [ temporaries ] ]! !

!S2JSqueakAstToJsAst methodsFor: 'convert complex' stamp: 'ag 1/24/2011 19:06'!
unaryExpression =
	
	serverBlock // [ 'a + b' ] onServer value   -->   onServer("a + b").value()
|	slotAccess // a js b   -->   a.b
|	messageSendExpression! !

!S2JSqueakAstToJsAst methodsFor: 'ometa helpers' stamp: 'ag 2/7/2011 19:40'!
wrapAsPrimitive :functionName :nodeName :wrappedObject =
	// Wrap an object into a node, that will result in a function-call to functionName with the object as argument
	
	[ self
		node: nodeName
		with: ( wrappedObject isCollection & wrappedObject isString not
			ifTrue: [ wrappedObject ]
			ifFalse: [ { wrappedObject } ]) ]:primitiveObject
	[ self
		functionCall: functionName
		to: nil
		withArguments: { primitiveObject } ]:result
	
	-> [ result
		jsArgumentValue: primitiveObject;
		yourself ]! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:34'!
allMethodsDo: aBlock
	"Execute aBlock with each parsable method in the image and return the sum of visited methods and classes."
	
	| numberOfObjects |
	numberOfObjects := 0.
	Smalltalk allClasses do: [ :eachClass |
		eachClass methodDict notEmpty  ifTrue: [
			numberOfObjects := numberOfObjects + 1.
			eachClass methodDict do: [ :eachMethod |
				numberOfObjects := numberOfObjects + 1.
				aBlock value: eachMethod ]]].
	^ numberOfObjects! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/15/2010 16:25'!
allMethodsDoAndReport: aBlock
	"Execute aBlock for each method in the image, print which methods have failed
	and notify each 500 methods. Print status-report at the end."
	
	| converted succeeded total |
	converted := -1.
	succeeded := 0.
	total := Smalltalk allClasses inject: 0 into: [ :sum :each | sum + each methodDict size].
	Transcript cr; show: 'Total methods in image: ', total asString.
	[
		self allMethodsDo: 
			[ :eachMethod |
				[ converted := converted + 1. aBlock value: eachMethod. succeeded := succeeded + 1.
				(converted \\ 500 = 0) ifTrue: [ Transcript cr; show: 'Converted ', converted asString, ' methods.' ]
				]
					on: Exception do: [ :err | Transcript cr; show: '	', converted asString, ': ', eachMethod name, ' (', err, ')' ]].
	]
		ensure: [ Transcript cr; show: 'Finished!! ', succeeded asString, ' of ', converted asString, ' operations succeeded!!' ].! !

!S2JOMeta2 class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:20'!
applyRule: aRule on: aMatcher
	
	^ aMatcher apply: aRule! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/13/2010 14:34'!
classNamed: aClassOrClassname
	
	^ Smalltalk classNamed: aClassOrClassname asString! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 12/22/2010 16:06'!
convert: aString as: aRule
	
	^ self convert: aString as: aRule insideMethod: nil! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 12/22/2010 16:06'!
convert: aString as: aRule insideMethod: aCompiledMethod
	
	| matcherToResult |
	matcherToResult := self convertWithoutEnd: aString as: aRule insideMethod: aCompiledMethod.
	self finalizeMatcher: matcherToResult key.
	matcherToResult key apply: #end.
	^ matcherToResult value! !

!S2JOMeta2 class methodsFor: 'tests' stamp: 'ag 12/15/2010 16:26'!
convertAllMethods
	"Convert all methods in this image."
	
	self allMethodsDoAndReport: [ :eachMethod | 
		self convertCompiledMethod: eachMethod].! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 2/1/2011 10:33'!
convertAllMethodsOf: aClass
	
	^ aClass methodDict notEmpty
		ifTrue: [ | methods |
			methods := Dictionary new.
			aClass methodDict valuesDo:
				[ :method | | result |
					[ result := self convertCompiledMethod: method ]
						on: Exception do: [ :err | self error: 'Failed to convert ', method printString, ': ', err ].
					methods at: result key put: result value].
			methods]
		ifFalse: [ self error: 'Cannot convert class ', aClass name ]! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 2/1/2011 10:55'!
convertCompiledMethod: aCompiledMethod
	
	^ self
		convert: (
			(aCompiledMethod methodClass compilerClass == Compiler)
				ifTrue: [ self sourceOf: aCompiledMethod ]
				ifFalse: [ self decompiledSourceOf: aCompiledMethod ])
		as: #method
		insideMethod: aCompiledMethod! !

!S2JOMeta2 class methodsFor: 'converting debug' stamp: 'ag 12/22/2010 16:02'!
convertDebug: aString as: aRule
	
	| matcherToResult |
	matcherToResult := self matchStringWithoutEnd: aString with: aRule.
	[ matcherToResult key apply: #end ]
		on: OM2Fail do: [ :err | 
			matcherToResult value explore. 
			self error: err ].
	^ matcherToResult value! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 12/13/2010 16:06'!
convertDoIt: aString
	
	^ self convert: aString as: #doItSnippet! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 12/8/2010 18:19'!
convertMethod: aString
	
	^ self convert: aString as: #method! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 1/19/2011 15:57'!
convertMethod: aStringOrSymbol of: aClassOrClassname
	
	| method |
	method := self method: aStringOrSymbol of: aClassOrClassname.
	^ method
		ifNotNil: [ self convertCompiledMethod: method ]
		ifNil: [  ]! !

!S2JOMeta2 class methodsFor: 'tests' stamp: 'ag 12/13/2010 08:44'!
convertRandomMethods: aNumber
	"Match some random methods from the image. Return a String summarizing the result.
	Failed methods are printed to the Transcript."
	
	| runs successes |
	runs := successes := 0.
	aNumber timesRepeat: 
		[ | method |
			method := self randomMethod.
			method ifNotNil: [
				[ runs := runs + 1. S2JParser convertCompiledMethod: method. successes := successes + 1 ]
					on: Error do: [ :err | Transcript cr; cr; show: 'Failed to parse: ', method name, '. ', err ]]].
	^ runs asString, ' runs, ', successes asString, ' successes'! !

!S2JOMeta2 class methodsFor: 'converting' stamp: 'ag 12/8/2010 18:08'!
convertStatements: aString
	
	^ self convert: aString as: #statements! !

!S2JOMeta2 class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:18'!
convertWithoutEnd: aString as: aRule insideMethod: aCompiledMethod
	"Returns both the matcher and the result as an association."
	
	| matcher result |
	matcher := self matcherOn: (self createInputFrom: aString as: aRule insideMethod: aCompiledMethod) readStream.
	matcher
		currentMethod: aCompiledMethod;
		apply: #empty.
	result := self applyRule: aRule on: matcher.
	matcher apply: #spaces.
	^ matcher -> result! !

!S2JOMeta2 class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:12'!
createInputFrom: aString as: aRule insideMethod: aCompiledMethod
	
	self subclassResponsibility! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:31'!
decompiledSourceOf: aCompiledMethod
	
	^ aCompiledMethod decompile decompileString! !

!S2JOMeta2 class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:04'!
finalizeMatcher: anOMeta2
	"Do nothing, overwrite if needed to apply anything before applying #end"! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname
	
	^ self
		method: aStringOrSymbol
		of: aClassOrClassname 
		ifAbsent: [ self error: 'Method ', aStringOrSymbol asString, ' not found in class ', aClassOrClassname asString ]! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/19/2011 15:59'!
method: aStringOrSymbol of: aClassOrClassname ifAbsent: aBlock
	
	^ ((self classNamed: aClassOrClassname)
		lookupSelector: aStringOrSymbol asSymbol)
			ifNil: [ aBlock value ]! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 1/10/2011 08:58'!
print: aString to: aFilename
	"Create new file aFilename (or override) and write aString in it"
	
	^ aString squeakToUtf8 putOn: (FileStream forceNewFileNamed: aFilename)! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 2/1/2011 10:33'!
randomMethod
	
	| class result |
	result := nil.
	[ result isNil ] whileTrue: [
		class := Smalltalk allClasses atRandom.
		class methodDict notEmpty
			ifTrue: [ result := class methodDict atRandom ]].
	^ result! !

!S2JOMeta2 class methodsFor: 'helpers' stamp: 'ag 12/8/2010 18:18'!
sourceOf: aCompiledMethod
	
	^ aCompiledMethod getSource asString! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/8/2010 22:53'!
H =
	
	[ self halt ]! !

!S2JOMeta2 methodsFor: 'accessors' stamp: 'ag 12/22/2010 15:48'!
currentMethod
	
	^ currentMethod! !

!S2JOMeta2 methodsFor: 'accessors' stamp: 'ag 12/22/2010 15:48'!
currentMethod: aCompiledMethod
	
	currentMethod := aCompiledMethod! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/8/2010 14:35'!
fail =
	
	?[ false ]! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/13/2010 12:48'!
nil =
	
	anything:object ?[ object isNil ]
	-> [ object ]! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/9/2010 12:16'!
node: aString
	
	^ self node: aString with: {}! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/7/2010 22:56'!
node: aString with: anOrderedCollection
	
	^ S2JNode named: aString with: anOrderedCollection! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/7/2010 22:56'!
node: aString withoutNils: anOrderedCollection
	
	^ S2JNode named: aString withoutNils: anOrderedCollection! !

!S2JOMeta2 methodsFor: 'helpers' stamp: 'ag 12/7/2010 22:56'!
node: aString withoutNilsNotEmpty: anOrderedCollection
	
	^ S2JNode named: aString withoutNilsNotEmpty: anOrderedCollection! !

S2JOMeta2 subclass: #S2JParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JParser commentStamp: 'ag 12/7/2010 23:09' prior: 0!
tokens
	Rules, that read one single element in squeal source-code. They all don't return an S2JNode, all are parsed to the
	according Squeak-object (e.g. Number, Symbol, String).
	Spaces before and after the tokens in the source-code are NOT read!!
	This is because in some cases, tokens must be read exactly, without emitting any spaces.
	The things returned from here must not be converted anymore, especially the <> operator is unnecessary.
	
complex elements
	Bigger parts of squeak source-code. Most return an S2JNode (using #node:with:), which contains further complex elements or object read
	by rules in the tokens-protocol. As convention, the nodes (except for helper-rules like primary and messagePattern) should be named like the rules.
	Each rule is supposed to read spaces before applying itself!! This way, rules from this protocol can be called serially, without
	reading spaces inbetween. However, spaces must be read between single tokens.
	
helpers
	Squeak-methods, that are called from within ometa-rules (or helper-rules).
	OMeta2 return-parts of the rules (righthand of ->) cannot contain multiple expressions (?)...
!

!S2JParser class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:13'!
createInputFrom: aString as: aRule insideMethod: aCompiledMethod
	
	^ aString! !

!S2JParser class methodsFor: 'converting private' stamp: 'ag 12/22/2010 16:07'!
finalizeMatcher: anOMeta2
	
	anOMeta2 apply: #spaces.! !

!S2JParser class methodsFor: 'parsing' stamp: 'ag 12/1/2010 11:04'!
parseNumber: aString
	"Parse aString to an instance of Number. Return nil, if parsing failed.
	Return nil also, if a number could be read, but not the whole String was processed.
	(Number readFrom: aString does not fail in that case)."
	
	| stream result |
	stream := ReadStream on: aString.
	result := Number readFrom: stream ifFail: [ nil ].
	stream atEnd ifFalse: [ result := nil ].
	^ result! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 12/9/2010 13:45'!
array =
	
	$(
		(
			(spaces
				(((($# spaces)? array:array) -> [ array ])
				| nodeNamed(#symbolBaseForArray. 'symbol')
				| literalTokenBase
				):element
			) -> [ element ]
		)*:contents
	ch($))
	-> [ self node: 'array' with: contents ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:10'!
binaryExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	spaces:comment binaryExpressionMessage:message
	-> [ self node: 'binaryExpression' withoutNils:
		{ self extractCommentOf: comment. receiver }, message ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:09'!
binaryExpressionMessage =
	// Not returning a node, but a simple collection
	
	binarySelector:selector
	(unaryExpression | primary):argument
	-> [ { selector. argument } ]! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/1/2010 13:32'!
binarySelector =
	
	<$| | ((specialCharacter | $-) specialCharacter*)>:selector
	-> [ selector asSymbol ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 1/19/2011 19:52'!
block =
	
	<
	ch($[):comment1
		( (ch($:) spaces identifier:argumentName -> [ argumentName ])+:arguments ch($|)?:argumentSeparator )?
		temporaries?:temporaries
		statements?:statements
	ch($]):comment2
	>:originalCode
		
	// Forbidden (and not covered above): Arguments without separator bar, but with statements or temporaries
	?[ argumentSeparator notNil | arguments isNil | ((temporaries isNil or: [temporaries isEmpty]) & (statements isNil or: [statements isEmpty])) ]
	
	-> [ (self node: 'block' withoutNils: { self extractCommentOf: comment1.
			self node: 'arguments' withoutNilsNotEmpty: arguments.
			temporaries. statements.
			self extractCommentOf: comment2})
				originalCode: originalCode ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 21:32'!
braceExpression =
	
	ch(${):comment1
		(expression:expr ch($.) -> [ expr ] )*:expressions
		expression?:lastExpression
	ch($}):comment2
	
	-> [ self braceExpressionWith: expressions and: lastExpression ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 12/13/2010 21:20'!
braceExpressionWith: expressions and: lastExpression
	
	^ self node: 'braceExpression' withoutNils: 
		((expressions
			ifNotNil: [ expressions ]
			ifNil: [ {} ])
				asOrderedCollection
					add: lastExpression;
					yourself)! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 12/16/2010 16:46'!
byteArray =
	// Byte array can only contain number-objects, don't pack them into nodes
	
	$# ch($[)
		(
			(spaces number:number [ number first ]:byte
			?[ (byte isInteger) & (byte <= 255) & (byte >= 0)] 
			) -> [ byte ]
		)*:contents
	ch($])
	-> [ self node: 'byteArray' with: contents ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 21:19'!
cascadedExpression =
	// Constructs a node, whose first node is the receiver and all further nodes are messages sent to that receiver
	
	(keywordExpression | binaryExpression | unaryExpression):mainExpression
	(
		ch($;)
		(
		((spaces binarySelector:selector (unaryExpression | primary):argument) -> [ self node: 'binaryExpression' with: { selector. argument }])
		| (((spaces keyword:selector (binaryExpression | unaryExpression | primary):argument) -> 
				[ selector -> argument])+:selectorsAndArguments -> [ (self keywordMessage: selectorsAndArguments) name: 'keywordExpression' ])
		| (spaces unarySelector:selector -> [ self node: 'unaryExpression' with: { selector } ])
		):cascade
		-> [ cascade ]
	)+:cascades
	-> [ self cascadedExpression: mainExpression with: cascades ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 12/7/2010 13:16'!
cascadedExpression: mainExpression with: cascades
	"The receiver of the mainExpression is the receiver of the whole cascade.
	It is the first node, the other nodes are the messages sent to that receiver"
	
	^ self node: 'cascadedExpression' withoutNils: 
		((cascades
			ifNotNil: [ cascades ]
			ifNil: [ {} ])
				asOrderedCollection
					addFirst: mainExpression;
					addFirst: mainExpression removeFirst;
					yourself)! !

!S2JParser methodsFor: 'ometa helpers' stamp: 'ag 11/30/2010 11:01'!
ch :c =
	// This reads a given element and all spaces before it and return the element
	// ch was supposed to mean character, but this can be applied to any object
	// Called like this: ch($=)
	
	spaces:comment exactly(c)
	-> [ comment ]! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 12/7/2010 10:48'!
character =
	// The method in the super class is #char, don't confuse it!!
	
	( ($$ <anything>:value) | (seq('Character space') -> [ $  ]):value )
	-> [ self node: 'character' with: { value asCharacter } ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 1/25/2011 14:43'!
consumedBy: aBlock
	
	| result |
	result := super consumedBy: aBlock.
	result isString
		ifFalse: [ result := result separatedBy: '' ].
	^ result! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 16:34'!
doItSnippet =
	
	temporaries?:temps
	statements?:statements
	-> [ self node: 'doItSnippet' withoutNils: { temps. statements } ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/2/2010 09:41'!
expression =
	
	((spaces identifier:identifier (token(':=') | ch($_))) -> [ identifier ])*:variableNames 
	(cascadedExpression | keywordExpression | binaryExpression | unaryExpression | primary):expressionBody
	
	// First child is the expressionBody, further children are names of variables, that are assigned by this expression
	-> [ self node: 'expression' withoutNils: ({ expressionBody }, variableNames) ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 12/13/2010 20:28'!
extractCommentOf: anOrderedCollection
	
	| result |
	true ifTrue: [^ nil]. "TODO skip all comments for now..."
	result := ''.
	anOrderedCollection do: [:item |
		item name = 'comment' ifTrue: [
			result := result , item first
			]].
	^ result isEmpty
		ifTrue: [nil]
		ifFalse: [self node: 'comment' with:{result}]! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 2/1/2011 10:59'!
identifier =
	
	nodeNamed(#identifierBase. 'identifier')
|	(<$_ identifierBase>:decompiledIdentifier -> [ self node: 'identifier' with: { decompiledIdentifier } ])! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 11/30/2010 11:03'!
identifierBase =
	
	<letter letterOrDigit*>! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 11/22/2010 16:34'!
keyword =
	
	<identifierBase $:>:selector
	-> [ selector asSymbol ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:06'!
keywordExpression =
	
	(binaryExpression | unaryExpression | primary):receiver
	keywordExpressionMessage:message
	-> [self node: 'keywordExpression' with: { receiver }, message ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:06'!
keywordExpressionMessage =
	
	(((spaces keyword:selector ( binaryExpression | unaryExpression | primary ):argument) -> [ selector -> argument ])+:selectorAndArguments)
	-> [ self keywordMessage: selectorAndArguments ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 12/2/2010 10:31'!
keywordMessage: selectorAndArgumentNames
	"selectorAndArgumentNames contains associations, the part of the selector and the argument name.
	Resulting S2JNode will have the selector as symbol on the first position and the argumentNames at the other positions."
	
	| selector result |
	selector := ''.
	result := self node: 'keyword' with: {}.
	selectorAndArgumentNames do: [ :each |
		selector := selector, each key.
		result add: each value].
	^ result
		addFirst: selector asSymbol;
		yourself! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/16/2010 16:12'!
literalReservedWord =
	// Copy of reservedWord, with only the static values
	
	<(seq('true') | seq('false') | seq('nil'))>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ self node: 'reservedWord' with: { reservedWord }]! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 12/7/2010 15:23'!
literalToken =
	// Anything, that can be evaluated to an object without context
	
	literalTokenBase
|	(($# spaces array:array) -> [ array ])! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/16/2010 16:08'!
literalTokenBase =
	// Things, that can be evalutated to an object without any context (array-literals excluded)
	
	literalReservedWord
|	number
|	symbol
|	character
|	string
|	byteArray! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 13:56'!
messagePattern =
	
	((spaces keyword:selectorPart spaces identifier:argumentName) -> [ selectorPart -> argumentName ])+:selectorAndArgumentNames
	-> [ self keywordMessage: selectorAndArgumentNames ]
|	spaces binarySelector:selector spaces identifier:argumentName
	-> [ self node: 'binary' with: { selector. argumentName } ]	
|	spaces unarySelector:unarySelector
	-> [ self node: 'unary' with: { unarySelector } ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 09:11'!
method =
	// Only one temporaries-declaration allowed, but can be before or after primitive-declaration
	
	messagePattern:head
	temporaries?:temps
	primitiveDeclaration*:primitives
	temporaries?:secondTemps ( ?[secondTemps isNil]  | (?[temps isNil] [secondTemps]:temps) )
	statements?:statements
	
	-> [ self node: 'method' withoutNils: { head. temps }, primitives, { statements } ]! !

!S2JParser methodsFor: 'ometa helpers' stamp: 'ag 12/7/2010 11:10'!
nodeNamed :aRule :aNodeName =
	// Helper rule to encapsulate the result of a rule into a node
	
	apply(aRule):result
	-> [ self node: aNodeName asString with: { result } ]! !

!S2JParser methodsFor: 'parse easy' stamp: 'LT 12/6/2010 14:37:16.251'!
number =
	// Called that way because of super-class-method
	// The string is parsed once, assuring, that
	
	<
		($-? digit+ $r)?
		$-?
		(digit | upper)+
		($. (digit | upper)+)?
		($e ($- | $+)? (digit | upper)+)?
	>:result
	?[ (self class parseNumber: result) notNil ]
	-> [ self node: 'number' with: {self class parseNumber: result} ]
	
	// An alternative way to evaluate a number...
	| ($# <digit>:value) -> [ self node: 'number' with: {value asNumber} ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/16/2010 23:55'!
primary =
	// something, that evaluates to an object
	
	spaces (literalToken | reservedWord | identifier):result -> [ result ]
|	block
|	braceExpression
|	ch($() expression:expr ch($)) -> [ expr name: 'paranthesizedExpression'; yourself ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 13:40'!
primitiveDeclaration =
	// Reads generally all primitive declarations, there are many of them
	// <primitive: 'a'> <primitive: 2> <primitive: 'a' module: 'b'> <version: 'ab'> <command> etc. etc.
	
	ch($<)
		(
			primitiveDeclarationRule(#keywordExpressionMessage)
			| (space primitiveDeclarationRule(#binaryExpressionMessage))
			| primitiveDeclarationRule(#unaryExpressionMessage)
		):message
	ch($>)
	-> [ self node: 'primitiveDeclaration' with: { message } ]! !

!S2JParser methodsFor: 'ometa helpers' stamp: 'ag 12/16/2010 23:31'!
primitiveDeclarationRule :aRule =
	// Read everything, that is inside the <>-primitive-declaration; write it back on the input stream, followed
	// by input, that will not be matched. Apply the rule and read the fake-input. Return the result after applying the rule.
	
	// This allows things, that are forbidden, like <primive: a> (a is an identifier and has been accepted, because it's also a unarySelector
	
	<
		((
			<spaces (literalToken | unarySelector | binarySelector | keyword)>:input ?[ (input includesSubString: '>') not ]
		) -> [ input ])*
	>:primitiveInput
	[ self apply: aRule withArgs: primitiveInput, '^^' ]:result
	$^ $^
	-> [ result ]! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 2/7/2011 19:51'!
reservedWord =
	
	<(seq('true') | seq('false') | seq('nil') | seq('self') | seq('super') | seq('thisContext') )>:reservedWord
	&(binarySelector | space | end | $] | $. | $; | $} | $)) // Look ahead to ensure, that this token ends here, don't match e.g. self2, but match self/2
	-> [ self node: 'reservedWord' with: { reservedWord }]! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/13/2010 18:57'!
space =
	// A comment is equivalent to a space-character.
	// spaces are ignored (return nil), but comments result in a special node and are collected
	
	^space -> [ nil ]
	| (($" <(~$" anything)*>:result $") -> [ self node: 'comment' with: { result } ])! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 11/21/2010 13:46'!
specialCharacter =
	
	$+ | $* | $/ | $\ | $~ | $< | $> | $= | $@ | $% | $& | $? | $!! | $` | $,! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/13/2010 18:58'!
statements =
	
	(expression?:expr ch($.) -> [ expr ] )*:expressions
	(ch($^)?:returnSign expression:lastExpression ch($.)? ~expression)?
	-> [ self statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign ]! !

!S2JParser methodsFor: 'helpers' stamp: 'ag 12/15/2010 11:17'!
statementsWithExpressions: expressions lastExpression: lastExpression returnSign: returnSign
	"Build the contents of a statements-node from the information gathered while parsing 'statements'"
	
	"Rename the last expression, if it was a return-expression"
	lastExpression notNil & returnSign notNil ifTrue: [
		lastExpression name = 'expression'
			ifTrue: [ lastExpression name: 'returnExpression' ]
			ifFalse: [ self error: 'Unexpected node at return-sign: ', lastExpression printString ]].
	expressions
		add: lastExpression;
		reject: [ :each | each isNil ].
	^ self node: 'statements' withoutNilsNotEmpty: expressions! !

!S2JParser methodsFor: 'parse easy' stamp: 'mh 12/11/2010 00:38'!
string =
	// Called that way because of super-class-method
	
	$' ((~$' anything) | (($' $') -> [ $' ]))*:values $'
	-> [ self node: 'string' with: { values asOrderedCollection inject: '' into: [ :a :b | a, b asString ] } ]! !

!S2JParser methodsFor: 'parse easy' stamp: 'ag 12/7/2010 17:23'!
symbol =
	
	$# spaces (symbolBase | (string:stringSymbol -> [ stringSymbol first ])):value 
	-> [ self node: 'symbol' with: { value asSymbol } ]! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/13/2010 21:45'!
symbolBase =
	// This does not return a symbol, just consume input. Inside of array, symbols can be represented without
	// the #-character
	
	binarySelector
|	<keyword+ identifierBase>
|	<keyword+>
|	identifierBase
|	($: | $. | ($[ $]) | ${ | $} | $] | $^):result -> [ result asSymbol ]! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/16/2010 16:09'!
symbolBaseForArray =
	
	(~literalReservedWord
	(
		symbolBase
		| $[
	):result)
	-> [ result asSymbol ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 11/21/2010 19:10'!
temporaries =
	
	ch($|)
		(spaces identifier:identifier -> [ identifier ])*:variableNames
	ch($|)
	-> [ self node: 'temporaries' with: variableNames ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:07'!
unaryExpression =
	
	(unaryExpression | primary):receiver unaryExpressionMessage:message
	-> [ self node: 'unaryExpression' with: {receiver. message} ]! !

!S2JParser methodsFor: 'parse complex' stamp: 'ag 12/7/2010 12:07'!
unaryExpressionMessage =
	
	spaces unarySelector:message
	-> [ message ]! !

!S2JParser methodsFor: 'parse tokens' stamp: 'ag 12/2/2010 11:51'!
unarySelector =
	
	identifierBase:selector ~$:
	-> [ selector asSymbol ]! !

OrderedCollection subclass: #S2JNode
	instanceVariableNames: 'name originalCode jsArgumentValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JNode commentStamp: 'ag 11/18/2010 13:09' prior: 0!
This is only a named collection, that is supposed to hold further collections or strings to form a tree!

!S2JNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:08'!
named: aString with: anOrderedCollection
	
	^ self new
		name: aString;
		addAll: anOrderedCollection;
		yourself! !

!S2JNode class methodsFor: 'instance creation' stamp: 'ag 11/18/2010 13:58'!
named: aString withoutNils: anOrderedCollection
	
	anOrderedCollection ifNil: [ ^ nil ].
	^ self
		named: aString
		with: (anOrderedCollection reject: [ :each | each isNil])! !

!S2JNode class methodsFor: 'instance creation' stamp: 'ag 11/21/2010 19:15'!
named: aString withoutNilsNotEmpty: anOrderedCollection
	"Returns nil, if resulting node would be empty"
	
	| result |
	result := self named: aString withoutNils: anOrderedCollection.
	^ (result isNil or: [result isEmpty])
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!S2JNode methodsFor: 'comparing' stamp: 'ag 11/21/2010 19:19'!
= anObject
	
	^ super = anObject and: 
		[ name = anObject name ]! !

!S2JNode methodsFor: 'testing' stamp: 'ag 12/16/2010 15:58'!
includesChildNamed: aString
	
	^ self name = aString
		or: [ self anySatisfy: [ :node | 
			(node isKindOf: S2JNode) and: [
			node includesChildNamed: aString ] ] ]! !

!S2JNode methodsFor: 'testing' stamp: 'LT 12/1/2010 14:06:25.154'!
isArrayNode

	^ name = 'array' or: [name = 'byteArray' or: [name = 'braceExpression']]! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 2/7/2011 19:20'!
jsArgumentValue
	^ jsArgumentValue! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 2/7/2011 19:25'!
jsArgumentValue: anS2JNode
	"The ast-to-ast-step stores an additional info here: The value of this node, if it would act as a argument to a js-function-call.
	E.g. a number-literal is stored directly as a number-node here, without wrapping it as squeak-object and calling js() on it.
	This is important, as for boolean-value the js()-mechanism has been found to be non-functional!!"
	
	jsArgumentValue := anS2JNode! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name
	
	^ name! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 1/25/2011 14:45'!
name: aString
	
	name := aString! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 1/19/2011 19:16'!
originalCode
	^ originalCode! !

!S2JNode methodsFor: 'accessing' stamp: 'ag 1/19/2011 19:17'!
originalCode: anObject
	"This is only set on 'block'-nodes. Never mind. :-)"
	
	originalCode := anObject! !

!S2JNode methodsFor: 'printing' stamp: 'ag 11/19/2010 00:39'!
printString
	
	| result first |
	first := true.
	result := name, ' ('.
	self do: [ :each |
		first ifTrue: [first := false] ifFalse: [result := result, ', '].
		result := result,  each printString].
	^ result, ')'! !

!HTTPConnection methodsFor: '*Squeak2JS' stamp: 'LT 1/26/2011 14:39:01.244'!
interact
	"We refine this to bring up the debugger when an error occurs inside the Swazoo."



	"longer description is below method"
	| interactionBlock |
	interactionBlock := 
		[[	[ [true] whileTrue: 
				[self getAndDispatchMessages.
				Processor yield] 
			]
			on: Error
			do: [:ex | 
				
				((Smalltalk at: #S2JDebugSwazooInternalErrors ifAbsent: [nil]) == true)
					ifTrue: [ (ex class = SwazooStreamNoDataError) | (ex class = SpSocketError) ifFalse: [ self halt ] ]
				] ]
		ifCurtailed: [ 
			(Delay forMilliseconds: 50) wait. "to finish sending, if any"
			self isOpen ifTrue: [self close]] ].
	self server isMultiThreading 
		ifTrue: [self loop: (interactionBlock forkAt: Processor userBackgroundPriority)]
		ifFalse: [interactionBlock value].
	^self

"I represent a specifc connection with an HTTP client (a browser, probably) over which will come an HTTP request.  Here, I fork the handling of the request so that the current thread (which is most likely the HTTP server main loop) can carry on with the next request.  This means that more than one request may being handled in the image at a time, and that means that the application developer must worry about thread safety - e.g the problem of a given business object being updated by more than one HTTP request thread.
For a GemStone implementation of Swazoo, one may want only one request is handled at a time, multi-threadedness being handled by having multiple gems.  This is a nice option because the application developer does not have to worry about thread safety in this case - GemStone handles the hard stuff.
*And* the thing called a loop that was in this method was no such thing.  In all circumstances, >>getAndDispatchMessages handles exactly one requst and then closes the socket!! (very non-HTTP1.1).  Anyway, for now I'm just going to make that explicit.  This needs to be re-visited to support HTTP 1.1."! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:05'!
classesWithDirtyCategories
	
	^ self ourClasses select: [ :class |
		(class name endsWith: 'Test') not & (class name endsWith: 'Tests') not &
		(class organization categories includes: 'as yet unclassified' asSymbol) ]! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:02'!
classesWithEmptyCategories
	
	^ (self ourClasses select: [ :class |
			class organization categories anySatisfy: [ :category | class organization isEmptyCategoryNamed: category ] ]),
		self ourEmptyClassCategories! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:02'!
countCategoriesOf: aClass
	
	^ (aClass organization categories
		reject: [ :cat | cat == 'no messages' asSymbol ])
			size! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:02'!
debugSwazooErrors: aBoolean
	
	^ Smalltalk
		at: #S2JDebugSwazooInternalErrors
		put: aBoolean! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:02'!
grep: aString
	"Search the source-code of our packages and return a collection of all CompiledMethods, that contain aString in their source-code."
	
	| methods |
	methods := OrderedCollection new.
	self ourClasses do: [ :class |
		class methodDict values do: [ :compiledMethod |
			(compiledMethod getSource asString includesSubString: aString)
					ifTrue: [ methods add: compiledMethod ]]].
	^ methods! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:03'!
ourClasses
	
	^ self ourPackage classesAndMetaClasses! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:03'!
ourEmptyClassCategories
	
	^ self ourPackage systemCategories select: [ :cat |
		SystemOrganization isEmptyCategoryNamed: cat]! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:03'!
ourPackage
	
	^ PackageOrganizer default
		packageNamed: 'Squeak2JS' asSymbol
		ifAbsent: [ self error: 'Our package Squeak2JS could not be found...' ]! !

!Utilities class methodsFor: '*Squeak2JS' stamp: 'ag 2/3/2011 12:03'!
removeEmptyCategories
	"Removes method-categories and class-categories. Return, how many categories have been removed."
	
	| removedCategories |
	removedCategories := 0.
	self ourClasses do: [ :class | | categoriesBefore |
		categoriesBefore := self countCategoriesOf: class.
		class organization removeEmptyCategories.
		removedCategories := removedCategories + categoriesBefore - (self countCategoriesOf: class) ].
	self ourEmptyClassCategories do: [ :cat |
		SystemOrganization removeCategory: cat.
		removedCategories := removedCategories + 1].
	^ removedCategories! !

!MIMEDocument class methodsFor: '*Squeak2JS' stamp: 'hk 1/26/2011 10:40'!
extendedMIMEdatabase
	| d |
	(d := self defaultMIMEdatabase)
		 at: 'js' put: 'text/javascript';
		 at: 'hqx' put: 'application/mac-binhex40';
		 at: 'cpt' put: 'application/mac-compactpro';
		 at: 'pdf' put: 'application/pdf';
		 at: 'ps' put: 'application/postscript';
		 at: 'ai' put: 'application/postscript';
		 at: 'eps' put: 'application/postscript';
		 at: 'rtf' put: 'text/rtf';
		 at: 'ics' put: 'text/calendar';
		 at: 'bin' put: 'application/octet-stream';
		 at: 'dms' put: 'application/octet-stream';
		 at: 'lha' put: 'application/octet-stream';
		 at: 'lzh' put: 'application/octet-stream';
		 at: 'exe' put: 'application/octet-stream';
		 at: 'class' put: 'application/octet-stream';
		 at: 'zip' put: 'application/zip';
		 at: 'gtar' put: 'application/x-gtar';
		 at: 'swf' put: 'application/x-shockwave-flash';
		 at: 'sit' put: 'application/x-stuffit';
		 at: 'tar' put: 'application/x-tar';
		 at: 'au' put: 'audio/basic';
		 at: 'snd' put: 'audio/basic';
		 at: 'mid' put: 'audio/midi';
		 at: 'midi' put: 'audio/midi';
		 at: 'mpga' put: 'audio/mpeg';
		 at: 'mp2' put: 'audio/mpeg';
		 at: 'mp3' put: 'audio/mpeg';
		 at: 'aiff' put: 'audio/x-aiff';
		 at: 'aif' put: 'audio/x-aiff';
		 at: 'aifc' put: 'audio/x-aiff';
		 at: 'rm' put: 'audio/x-pn-realaudio';
		 at: 'ram' put: 'audio/x-pn-realaudio';
		 at: 'rpm' put: 'audio/x-pn-realaudio-plugin';
		 at: 'ra' put: 'audio/x-realaudio';
		 at: 'wav' put: 'audio/x-wav';
		 at: 'css' put: 'text/css';
		 at: 'mpeg' put: 'video/mpeg';
		 at: 'mpg' put: 'video/mpeg';
		 at: 'mpe' put: 'video/mpeg';
		 at: 'qt' put: 'video/quicktime';
		 at: 'mov' put: 'video/quicktime';
		 at: 'avi' put: 'video/x-msvideo';
		 at: 'movie' put: 'video/x-sgi-movie'.
	^ d! !

SwazooResource subclass: #S2JResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

S2JResource subclass: #S2JBasicApp
	instanceVariableNames: 'clients repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

S2JBasicApp class
	instanceVariableNames: 'instance'!

S2JBasicApp class
	instanceVariableNames: 'instance'!

S2JBasicApp subclass: #S2JApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'ag 1/25/2011 15:49'!
allRequiredClasses
	"Return the requiredClasses together with all their superclasses. Only for this class, not the superclasses."
	
	| allClasses |
	allClasses := OrderedCollection new.
	self requiredClasses do: [ :eachClass |
		eachClass allSuperclasses reverseDo: [ :superClass |
			(allClasses includes: superClass) ifFalse: [ allClasses add: superClass ]].
		(allClasses includes: eachClass) ifFalse: [ allClasses add: eachClass ]].
	^ allClasses! !

!S2JApp class methodsFor: 'rendering-configuration' stamp: 'srv 1/24/2011 14:05'!
codeFactory
	
	^ S2JPrettyCodeFactory.! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'SW 1/31/2011 08:35'!
codeForAllClasses

	^ self codeFactory codeForClasses: self collectRequiredClasses! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'ag 1/24/2011 17:00'!
collectPostloadScripts
	"Collect #requiredClasses from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass postloadScripts ]
		upTo: S2JApp! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'ag 1/24/2011 17:01'!
collectRequiredClasses
	"Collect #requiredClasses from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass allRequiredClasses ]
		upTo: S2JApp! !

!S2JApp class methodsFor: 'rendering-javascript' stamp: 'srv 1/24/2011 14:03'!
postloadScripts
	
	^ #( 'bootstrap.js' 'kernel_primitives.js' )! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:04'!
recompileRequiredClasses
	"This causes all required classes of THIS web-app (none of the super-classes) to be invalidated in the code-factory-cache."
	
	^ self allRequiredClasses do: [ :each | each compileAll ]! !

!S2JApp class methodsFor: 'rendering-javascript' stamp: 'srv 1/24/2011 14:06'!
requiredClasses
	"Return any array of classes, that must be compiled to JS-code to enable this application.
	Classes added by subclasses must also return the classes in this implementation, as these clases are required by the js-bootstrapping-library."
	
	^ { BlockClosure. Float. True. False. Error. ByteString. OrderedCollection. Character. Array. UndefinedObject. Point. WriteStream }! !

!S2JApp class methodsFor: 'rendering-javascript' stamp: 'srv 1/24/2011 14:14'!
scripts
	"= preload scripts"
	
	^ #( 'squeaky.js' )! !

!S2JApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 16:35'!
startupClass
	"Return something other than nil to call the static method #start on the class returned here."
	
	^ nil! !

!S2JApp class methodsFor: 'rendering-css' stamp: 'ag 1/26/2011 15:06'!
styles
	
	^ #( 'default.css' )! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:05'!
writeClassesTo: aDirectoryName
	"Write all classes required by this WebApp (together with their superclasses) to files named '<classname>.txt' in the given directory."
	
	self codeForAllClasses.
	self collectRequiredClasses withIndexDo: [ :cl :index |
		self codeFactory
			printCodeFor: cl
			to: aDirectoryName, index asString, '-', cl name, '.txt' ].! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:05'!
writeClassesToFile: aFileName
	"Write all classes required by this WebApp (together with their superclasses) to files named '<classname>.txt' in the given directory."
	
	^ self codeFactory
		printCodeFor: self collectRequiredClasses
		to: aFileName! !

!S2JApp class methodsFor: 'rendering-helpers' stamp: 'srv 1/24/2011 14:06'!
writeSiteTo: aLocalFilename
	"Write the website, that would be sent to a client, to a new file named aLocalFilename (overwrites existing)."
	
	^ self codeFactory converterClass
		print: self new render
		to: aLocalFilename! !

!S2JApp methodsFor: 'hooks' stamp: 'ag 1/24/2011 20:43'!
onloadCode
	
	^ super onloadCode, 'app=startupS2J();'! !

!S2JApp methodsFor: 'rendering' stamp: 'hk 2/1/2011 11:58'!
renderJavascript
	"Render the external javascript resources and the link to the compiled classes code."
	
	^
	super renderJavascript,
	(self renderScriptTag: S2JClassesHandler pathSuffix),
	(self renderScriptTags: self class collectPostloadScripts),
	self renderStartupCall, String cr! !

!S2JApp methodsFor: 'rendering' stamp: 'ag 1/24/2011 20:43'!
renderStartupCall
	
	^ 
	'<script type="text/javascript">',
	'var startupS2J = function() {', (self class startupClass
		ifNotNil: [ 'return ', (self class codeFactory codeFor: (self class startupClass name, ' start')) ]
		ifNil: [ '' ]),
			'}',
	'</script>'! !

S2JApp subclass: #S2JBallGameApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-BallGame'!

!S2JBallGameApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/25/2011 16:37'!
postloadScripts
	
	^ #( 'misc/canvastoolkit.js' 'misc/additional_functions.js' )! !

!S2JBallGameApp class methodsFor: 'rendering-javascript' stamp: 'lw 1/13/2011 11:34'!
requiredClasses
	
	^ { Rectangle. S2JBall. S2JBallGame }! !

!S2JBallGameApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 16:36'!
startupClass
	
	^ S2JBallGame! !

!S2JBallGameApp class methodsFor: 'configuration' stamp: 'lw 1/12/2011 18:39'!
title

	^ 'Bouncing Balls'! !

!S2JBallGameApp class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath 
	
	^ 'balls/'! !

!S2JBallGameApp methodsFor: 'rendering' stamp: 'LT 1/19/2011 15:22:01.602'!
renderContent
	
	^ '<canvas id="ballCanvas" width="400" height="300" style="background-color: #ffffff;  position: relative; border: solid 1px #000000;"></canvas>'! !

S2JApp subclass: #S2JConsoleApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JConsoleApp class methodsFor: 'configuration' stamp: 'HK 1/11/2011 11:52'!
title 

	^ 'Console'.! !

!S2JConsoleApp class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath 

	^ 'console/'.! !

!S2JConsoleApp methodsFor: 'rendering' stamp: 'SW 1/17/2011 20:36'!
renderContent

^ '
<h3>Push your code</h3>
	<p><textarea id="input"></textarea></p>
	<p><input type="button" value="Send data" onclick="CONNECTION.sendCodeInput();"></p>
	<h3>Log</h3>
	<p>
	<input type="button" value="Connect" onclick="CONNECTION.connect();">
	<input type="button" value="Disconnect" onclick="CONNECTION.disconnect();">
	<input type="button" value="Clear log" onclick="CONNECTION.clearLog();">
	</p>
	<table>
	<thead>
	<tr>
	<th>Status</th>
	<th>Time</th>
	<th>Log</th>
	<th>Eval</th>
	</tr>
	</thead>
	<tbody>
	</tbody>
	</table>
	'.! !

S2JApp subclass: #S2JDemoApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JDemoApp class methodsFor: 'configuration' stamp: 'ag 1/24/2011 17:21'!
requiredClasses
	
	^ { S2JText. S2JContainer }! !

!S2JDemoApp class methodsFor: 'configuration' stamp: 'HK 1/11/2011 11:16'!
title

	^ 'DemoApp'.! !

!S2JDemoApp class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath 

	^ 'demo/'! !

S2JApp subclass: #S2JIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JIndex class methodsFor: 'configuration' stamp: 'ag 1/12/2011 18:21'!
title

	^ 'Index'! !

!S2JIndex class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath

	^ 'index/'! !

!S2JIndex methodsFor: 'hooks' stamp: 'hk 2/1/2011 11:55'!
linkFor:application
	^ '<a href="/', application uniquePath , '" title="', application title ,'">', application title , '</a>'.
	    
	
	! !

!S2JIndex methodsFor: 'hooks' stamp: 'hk 1/25/2011 16:57'!
renderContent
	
	^ '<h2>Installed applications: </h2>', self renderLinks! !

!S2JIndex methodsFor: 'hooks' stamp: 'ag 1/25/2011 11:51'!
renderLinks

	| result |
	result := '<ul>'.
	S2JBasicApp allSubclassesDo: [ :application | 
		(application = self class or: [ application uniquePath isNil]) ifFalse: [
			result := result , '<li>', (self linkFor:application) , '</li>'.		
			].
		].
	result := result , '</ul>'.
	^ result
	    
	
	! !

S2JApp subclass: #S2JWidgetApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

S2JWidgetApp subclass: #S2JBrowserApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JBrowserApp class methodsFor: 'rendering-javascript' stamp: 'SW 1/31/2011 14:05'!
requiredClasses
	
	^ { S2JClassBrowser . S2JSideSelector . S2JBrowserPane . S2JClassPane . S2JMethodPane . S2JPackagePane . S2JProtocolPane . S2JCodeBox . S2JSaveButton  . S2JLoader }! !

!S2JBrowserApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 16:36'!
startupClass
	
	^ S2JClassBrowser! !

!S2JBrowserApp class methodsFor: 'configuration' stamp: 'ag 1/17/2011 18:27'!
title
	
	^ 'Class Browser'! !

!S2JBrowserApp class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:49'!
uniquePath
	
	^ 'browser/'! !

!S2JWidgetApp class methodsFor: 'rendering-javascript' stamp: 'hk 2/1/2011 15:43'!
requiredClasses
	
	^ { S2JList. S2JText. Association. S2JListOption . S2JButton. S2JHorizontalContainer. S2JVerticalContainer . S2JImage . S2JLoader . S2JParagraph }! !

!S2JWidgetApp class methodsFor: 'rendering-css' stamp: 'ag 1/26/2011 15:06'!
styles
	
	^ #( 'widgets.css' )! !

!S2JWidgetApp methodsFor: 'rendering' stamp: 'ag 1/17/2011 18:14'!
renderContent
	
	^ '<div id="container"></div>'! !

S2JWidgetApp subclass: #S2JWidgetPlayground
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JWidgetPlayground class methodsFor: 'rendering-javascript' stamp: 'hk 2/1/2011 14:32'!
requiredClasses
	
	^ { S2JRadioGroup . S2JRadioButton . S2JLabel }! !

!S2JWidgetPlayground class methodsFor: 'rendering-javascript' stamp: 'hk 1/26/2011 11:01'!
startupClass 

	^ S2JContainer! !

!S2JWidgetPlayground class methodsFor: 'configuration' stamp: 'hk 1/24/2011 17:20'!
title

	^ 'Let''s play'! !

!S2JWidgetPlayground class methodsFor: 'configuration' stamp: 'hk 2/1/2011 11:48'!
uniquePath

	^ 'play/'! !

S2JWidgetApp subclass: #S2JWorkspaceApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-Workspace'!

!S2JWorkspaceApp class methodsFor: 'rendering-javascript' stamp: 'ag 2/1/2011 12:00'!
requiredClasses
	
	^ { S2JWorkspace. S2JNode. S2JTextStream. S2JPrettyPrinter. S2JParser. S2JSqueakAstToJsAst. 
	
	"Classes to get OMeta2 to work (other than super-classes of our translator-classes)"
	ReadStream. OM2LazyStream. OM2EndOfStream. IdentityDictionary }! !

!S2JWorkspaceApp class methodsFor: 'rendering-javascript' stamp: 'robi 1/26/2011 11:36'!
startupClass

	^ S2JWorkspace! !

!S2JWorkspaceApp class methodsFor: 'configuration' stamp: 'robi 1/26/2011 11:24'!
title
	^ 'Workspace'! !

!S2JWorkspaceApp class methodsFor: 'configuration' stamp: 'hk 2/1/2011 12:02'!
uniquePath
	^ 'workspace/'! !

!S2JBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/24/2011 17:00'!
collect: aBlock upTo: topSuperClass
	"Evaluate aBlock with all relevant superclasses as single argument and collect the resulting collections."
	
	| class result superClasses |
	result := OrderedCollection new.
	superClasses := topSuperClass allSuperclasses.
	class := self.
	[ superClasses includes: class ]
		whileFalse: [
			result addAllFirst: (aBlock value: class).
			class := class superclass ].
	^ result
		inject: OrderedCollection new
		into: [ :all :cl | (all includes: cl) ifFalse: [ all add: cl ]. all ]! !

!S2JBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/26/2011 14:52'!
collectScripts
	"Collect #scripts from all superclasses"
	
	^ self
		collect: [ :eachClass | eachClass scripts ]
		upTo: S2JBasicApp! !

!S2JBasicApp class methodsFor: 'rendering-css' stamp: 'ag 1/26/2011 15:09'!
collectStyles
	"Collect #styles from all superclasses"
	
	^ (self
		collect: [ :eachClass | eachClass styles ]
		upTo: S2JBasicApp)
			collect: [ :scriptName | 'styles/', scriptName ]! !

!S2JBasicApp class methodsFor: 'singleton' stamp: 'ag 1/12/2011 17:59'!
doesNotUnderstand: aMessage
	"removes need to call instance every time the singleton is accessed"
	
	^ self instance perform: aMessage selector withArguments: aMessage arguments! !

!S2JBasicApp class methodsFor: 'configuration' stamp: 'HK 1/10/2011 16:30'!
htmlDocumentType

	^ '<!!DOCTYPE html>'.! !

!S2JBasicApp class methodsFor: 'singleton' stamp: 'HK 1/12/2011 10:43'!
instance
	
	instance ifNil: [ instance := self new ].
	^ instance.! !

!S2JBasicApp class methodsFor: 'rendering-javascript' stamp: 'ag 1/31/2011 14:06'!
scripts
	
	^ #( 'console.js' 'bidirectional_communication.js' 'server.js' )! !

!S2JBasicApp class methodsFor: 'rendering-css' stamp: 'hk 1/26/2011 12:29'!
styles

	^ #()! !

!S2JBasicApp class methodsFor: 'configuration' stamp: 'ag 1/24/2011 17:19'!
title
	"Must be implemented by non-abstract subclasses"
	
	^ nil! !

!S2JBasicApp class methodsFor: 'configuration' stamp: 'ag 1/24/2011 17:20'!
uniquePath
	"Must be implemented by non-abstract subclasses"
	
	^ nil! !

!S2JBasicApp methodsFor: 'dispatching' stamp: 'ag 1/25/2011 13:41'!
addClient: aClient

	self clients add: aClient.
	aClient application: self.! !

!S2JBasicApp methodsFor: 'dispatching' stamp: 'ag 1/25/2011 13:41'!
answerNewContentTo: aClient

	aClient increaseRevision.
	^ self
		answerWithStatusCode: 200
		andContent: (self repository contentAtRevision: aClient revision)! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'SW 1/18/2011 11:26'!
answerNoContent

	^ self answerWithStatusCode: 204! !

!S2JBasicApp methodsFor: 'dispatching' stamp: 'ag 1/25/2011 16:20'!
answerTo: aRequest
	
	| handlerClass |
	handlerClass := S2JClientHandler subclasses
		detect: [ :eachClass | eachClass canHandle: aRequest ]
		ifNone: [ nil ].
	^ handlerClass
		ifNotNil: [ (handlerClass newFor: self) answerTo: aRequest ]
		ifNil: [ self answerContent: self render withType: 'text/html' ]! !

!S2JBasicApp methodsFor: 'accessing' stamp: 'HK 1/12/2011 10:49'!
clients
	^ clients! !

!S2JBasicApp methodsFor: 'accessing' stamp: 'HK 1/12/2011 10:49'!
clients: anObject
	clients := anObject! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'hk 2/1/2011 12:00'!
fullUrlFor: aUrl
	^ (aUrl startsWith: 'http')
		ifTrue: [ aUrl ]
		ifFalse: [ S2JFileRequestHandler pathSuffix , '/' , aUrl ]! !

!S2JBasicApp methodsFor: 'initialization' stamp: 'ag 1/13/2011 20:17'!
initialize

	super initialize.
	self
		clients: OrderedCollection new;
		repository: S2JContentRepository new.! !

!S2JBasicApp methodsFor: 'hooks' stamp: 'ag 1/24/2011 16:45'!
onloadCode
	
	^ 'S2JConnection.connect();'! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'HK 1/12/2011 15:13'!
removeAllClients

	self clients
		select: [ :each | each isWebSocketClient ]
		thenDo: [ :each | each close ].
	self clients: OrderedCollection new.! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 11:47'!
removeClient: aClient
	
	self clients remove: aClient ifAbsent: [ ].! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/13/2011 20:18'!
render
	
	^ self class htmlDocumentType,
		'<html>',
			self renderHeader, 
			self cr, 
			self renderBody,
		'</html>'! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/24/2011 16:45'!
renderBody

	^'<body onload="', self onloadCode, '" onunload="">',
		self renderContent,
	   '</body>'.! !

!S2JBasicApp methodsFor: 'hooks' stamp: 'ag 1/24/2011 14:31'!
renderContent 
	"Subclasses return custom html-content"
	
	^ ''! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'hk 1/26/2011 12:20'!
renderHeader

	^ '<head>',
		self renderTitle,
		self renderJavascript,
		self renderStyles,
	'</head>'.! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'srv 1/24/2011 14:08'!
renderJavascript
	"Render the external javascript resources."
	
	^ self renderScriptTags: self class collectScripts! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 09:56'!
renderScriptTag: aFilename
	
	^ '<script type="text/javascript" src="', aFilename, '"></script>', String cr! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 14:56'!
renderScriptTags: aCollectionOfFilenames
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames of the js-files. The path (url) will be prepended."
	
	^ self
		renderTags: aCollectionOfFilenames
		collectingFrom: [ :url | self renderScriptTag: url]! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'hk 1/26/2011 12:27'!
renderStyleTag: aFilename
	
	^ '<link href="', aFilename, '" rel="stylesheet" type="text/css" />', String cr! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 15:07'!
renderStyles
	"Render the external javascript resources."
	
	^ self
		renderTags: self class collectStyles
		collectingFrom: [ :url | self renderStyleTag: url]! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 14:55'!
renderTags: aCollectionOfFilenames collectingFrom: aBlock
	"aCollectionOfFilenames does not contain absolute Urls, but only the filenames. The path (url) will be prepended."
	
	^ aCollectionOfFilenames
		inject: ''
		into: [ :string :eachUrl | 
			string, (aBlock value: (self fullUrlFor: eachUrl)) ]! !

!S2JBasicApp methodsFor: 'rendering' stamp: 'ag 1/26/2011 09:57'!
renderTitle
	
	^'<title>',
	 self class title,
	'</title>',
	String cr! !

!S2JBasicApp methodsFor: 'accessing' stamp: 'HK 1/12/2011 10:49'!
repository
	^ repository! !

!S2JBasicApp methodsFor: 'accessing' stamp: 'HK 1/12/2011 10:49'!
repository: anObject
	repository := anObject! !

!S2JBasicApp methodsFor: 'hooks' stamp: 'SW 1/25/2011 11:59'!
reset 
	
	self
		clients: OrderedCollection new;
		repository: S2JContentRepository new.! !

!S2JBasicApp methodsFor: 'sending content' stamp: 'ag 2/3/2011 12:21'!
sendCode: code
	
	| jsCode |
	jsCode := self translateSqueakCode: code.
	self updateRepositoryWith: jsCode.
	self sendCodeToWebSocketClients: jsCode.! !

!S2JBasicApp methodsFor: 'sending content' stamp: 'SW 1/31/2011 09:25'!
sendCodeToWebSocketClients:jsCode

	self webSocketClients do: [ :each | each receive: jsCode ]
! !

!S2JBasicApp methodsFor: 'sending content' stamp: 'SW 1/31/2011 09:22'!
translateSqueakCode: code

	| jsCode |
	[ jsCode := S2JPrinter convertDoIt: code ]
		on: Exception
		do: [ :err | jsCode := 'alert("Could not parse squeak-code: ', Character cr asString, code,
			Character cr asString, 'Error was: ', err asString, '");' ].
	^ jsCode! !

!S2JBasicApp methodsFor: 'sending content' stamp: 'ag 1/26/2011 11:47'!
updateRepositoryWith: someContent
	
	self repository addContent: someContent.! !

!S2JBasicApp methodsFor: 'sending content' stamp: 'ag 1/25/2011 13:33'!
updateRepositoryWithSqueakCode: code

	| jsCode |
	[ jsCode := S2JPrinter convertDoIt: code ]
		on: Exception
		do: [ :err | jsCode := 'alert("Could not parse squeak-code: ', Character cr asString, code,
			Character cr asString, 'Error was: ', err asString, '");' ].
	self updateRepositoryWith: jsCode.! !

!S2JBasicApp methodsFor: 'dispatching' stamp: 'ag 1/25/2011 14:04'!
webSocketClients
	
	^ self clients select: [ :each | each isWebSocketClient ]! !

S2JBasicApp subclass: #S2JEmptyTestApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JEmptyTestApp class methodsFor: 'rendering-javascript' stamp: 'ag 2/3/2011 12:47'!
codeForAllClasses
	"This interface is implemented by S2JApp. We 'steal' the implementation from S2JApp, but we don't want to inherit from it!!"
	
	^ S2JApp codeForAllClasses! !

!S2JEmptyTestApp class methodsFor: 'rendering-javascript' stamp: 'ag 2/3/2011 12:47'!
scripts
	
	^ #( 'squeaky.js' 'test/tests.js' )! !

!S2JEmptyTestApp class methodsFor: 'rendering-css' stamp: 'ag 2/3/2011 12:47'!
styles
	
	^ #( 'test.css' )! !

!S2JEmptyTestApp class methodsFor: 'configuration' stamp: 'ag 2/3/2011 12:47'!
title

	^ 'test'! !

!S2JEmptyTestApp class methodsFor: 'configuration' stamp: 'ag 2/3/2011 12:47'!
uniquePath

	^ 'emptyTest/'! !

S2JEmptyTestApp subclass: #S2JTestApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Applications'!

!S2JTestApp class methodsFor: 'configuration' stamp: 'ag 2/3/2011 12:47'!
uniquePath

	^ 'test/'! !

!S2JTestApp methodsFor: 'hooks' stamp: 'ag 2/3/2011 12:41'!
onloadCode
	
	^ super onloadCode, 'S2JTests.runTests();'! !

S2JResource subclass: #S2JClientHandler
	instanceVariableNames: 'application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

S2JClientHandler subclass: #S2JClassesHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JClassesHandler class methodsFor: 'request handling' stamp: 'ag 1/26/2011 09:13'!
pathSuffix
	
	^ 'classes'! !

!S2JClassesHandler methodsFor: 'request handling' stamp: 'ag 1/26/2011 09:19'!
answerTo: aRequest
	
	^ self answerCode: self application class codeForAllClasses! !

!S2JClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 16:29'!
canHandle: aRequest
	
	| segments |
	segments := self pathSegments: aRequest.
	^ segments size >= 2 and: [ segments second = self pathSuffix ]! !

!S2JClientHandler class methodsFor: 'instance creation' stamp: 'HK 1/11/2011 16:33'!
newFor: anApplication

	^ self new application: anApplication.! !

!S2JClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:31'!
pathSegments: aRequest
	
	^ (aRequest requestLine requestURI identifier) findTokens: '/'! !

!S2JClientHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:07'!
pathSuffix
	
	self subclassResponsibility! !

!S2JClientHandler methodsFor: 'request handling' stamp: 'ag 1/25/2011 14:08'!
answerTo: aRequest
	
	self subclassResponsibility! !

!S2JClientHandler methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:33'!
application
	^ application! !

!S2JClientHandler methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:33'!
application: anObject
	application := anObject! !

S2JClientHandler subclass: #S2JCometHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JCometHandler class methodsFor: 'configuration' stamp: 'HK 12/15/2010 15:39'!
identifierRange

	^ 10e10.! !

!S2JCometHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix
	
	^ 'xhr'! !

!S2JCometHandler methodsFor: 'responding' stamp: 'HK 1/3/2011 20:30'!
answerNewIdentifier: anIdentifier

	^ self
		answerWithStatusCode: 202
		andContent: anIdentifier.! !

!S2JCometHandler methodsFor: 'request handling' stamp: 'ag 1/25/2011 13:45'!
answerTo: aRequest
	
	| client |
	client := self findClient: aRequest.
	^ client
		ifNil: [ self registerNewClient ]
		ifNotNil: [ 
			aRequest isGet ifTrue: [ client update ] ifFalse: [
			aRequest isPost ifTrue: [ client sent: aRequest ] ]]! !

!S2JCometHandler methodsFor: 'identification' stamp: 'SW 1/19/2011 10:12'!
extractIdentifierFrom: aRequest

	(self includesIdentifier: aRequest) ifFalse: [ ^ nil ].
	^ (aRequest requestLine requestURI asString subStrings: '=') at: 2.! !

!S2JCometHandler methodsFor: 'request handling' stamp: 'HK 1/12/2011 15:32'!
findClient: aRequest

	| identifier |
	identifier := self extractIdentifierFrom: aRequest.
	identifier ifNil: [ ^ nil ].
	^ self findClientById: identifier.! !

!S2JCometHandler methodsFor: 'identification' stamp: 'HK 1/12/2011 10:54'!
findClientById: anIdentifier

	^ self application clients
		detect: [ :uniqueClient |
			uniqueClient isCometClient and:
				[ uniqueClient identifier = anIdentifier ]]
		ifNone: [ nil ].! !

!S2JCometHandler methodsFor: 'identification' stamp: 'HK 1/3/2011 12:14'!
generateIdentifier

	| randomId |
	[ randomId := self class identifierRange atRandom ] doWhileFalse:
		[ self isIdentifierAvailable: randomId ].
	
	^ randomId asString.! !

!S2JCometHandler methodsFor: 'identification' stamp: 'HK 1/3/2011 13:29'!
includesIdentifier: aRequest

	^ aRequest requestLine requestURI asString includesSubString: '?id='.! !

!S2JCometHandler methodsFor: 'identification' stamp: 'HK 1/12/2011 10:40'!
isIdentifierAvailable: anIdentifier

	| allocatedIdentifiers |
	allocatedIdentifiers := self application class clients
		select: [ :each | each isCometClient ]
		thenCollect: [ :each | each identifier ].
		
	 ^ (allocatedIdentifiers includes: anIdentifier) not.! !

!S2JCometHandler methodsFor: 'convenience' stamp: 'SW 1/18/2011 15:36'!
registerNewClient

	| identifier client |
	
	identifier := self generateIdentifier.
	client := (S2JCometClient new identifier: identifier).
	client revision: self application repository revision.
	self application addClient: client.
	^ self answerNewIdentifier: identifier.! !

S2JClientHandler subclass: #S2JFileRequestHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix
	
	^ 'file'! !

!S2JFileRequestHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:41'!
rootDirectory: aDirectoryName
	"Configure the root-directory, this mini-server is serving. Only required once."
	
	| dir |
	dir := FileDirectory on: aDirectoryName.
	dir exists ifFalse: [ self error: 'Directory does not exist: ', aDirectoryName ].
	^ Smalltalk
		at: #S2JFileRootDirectory
		put: dir! !

!S2JFileRequestHandler methodsFor: 'request handling' stamp: 'hk 1/31/2011 15:42'!
answerTo: aRequest
	
	| file |
	file := self extractFile: aRequest.
	^ file
		ifNotNil: [
			file binary.
			"images require binary mode. plain text works regardless of encoding"
			self application
				answerCode: file contents
				withType: file mimeType ]
		ifNil: [
			self application
				answerWithStatusCode: 404
				andContent: 'Requested file not found' ]! !

!S2JFileRequestHandler methodsFor: 'request handling' stamp: 'ag 1/25/2011 15:44'!
extractFile: aRequest
	
	| segments fileRoot |
	segments := self class pathSegments: aRequest.
	segments removeFirst. "The app-name"
	segments removeFirst. "The /file -suffix"
	fileRoot := Smalltalk
		at: #S2JFileRootDirectory
		ifAbsent: [ self error: 'The root-directory name has not been configurd!! Execute: S2JFileRequestHandler rootDirectory: "/absolute/path/to/root/"' ].
	^ fileRoot oldFileOrNoneNamed: (segments separatedBy: FileDirectory pathNameDelimiter asString)! !

S2JClientHandler subclass: #S2JMethodInvocationHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JMethodInvocationHandler class methodsFor: 'configuration' stamp: 'ag 1/25/2011 15:17'!
pathSuffix 
	
	^ 'mi'! !

!S2JMethodInvocationHandler methodsFor: 'request handling' stamp: 'ag 1/26/2011 17:13'!
answerTo: aRequest
	
	| code arguments currentArg result |
	aRequest ensureFullRead.
	code := (aRequest postDataAt: 'code') value.
	arguments := OrderedCollection new.
	currentArg := 0.
	[ currentArg isNil ] whileFalse: [ | key arg |
		key := 'arg', currentArg asString.
		arg := (aRequest postDataAt: key) value.
		arg
			ifNil: [ currentArg := nil ]
			ifNotNil: [
				arguments add: arg.
				currentArg := currentArg + 1 ] ].
	result := (Compiler evaluate: code) valueWithArguments: arguments asArray.
	^ self application answerContent: (result serializeToJavaScript) withType: 'text/javascript'! !

S2JClientHandler subclass: #S2JWebSocketHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JWebSocketHandler class methodsFor: 'request handling' stamp: 'ag 1/25/2011 16:29'!
canHandle: aRequest
	"This class has an own implementation of this. That's why the abstract method pathSuffix is not implemented."
	
	^ aRequest isWebSocketRequest! !

!S2JWebSocketHandler class methodsFor: 'configuration' stamp: 'HK 1/4/2011 12:17'!
millisecondsToWaitForHandshake

	^ 200.! !

!S2JWebSocketHandler class methodsFor: 'client handling' stamp: 'SW 1/19/2011 09:45'!
updateAllClients

	self application webSocketClients do: [ :each |
		each update ].! !

!S2JWebSocketHandler class methodsFor: 'client handling' stamp: 'SW 1/19/2011 09:45'!
updateAllClientsExcept: aClient
	
	aClient increaseRevision.
	self application webSocketClients
		reject: [ :uniqueClient | uniqueClient = aClient ]
		thenDo: [ :each | each update ].! !

!S2JWebSocketHandler methodsFor: 'request handling' stamp: 'SW 1/17/2011 12:23'!
answerTo: aRequest

	| webSocket |
	webSocket := aRequest webSocket.
		
	(self isNew: webSocket ) ifTrue: 
		[ self setupClientWith: webSocket ].
	^ webSocket answerTo: aRequest.! !

!S2JWebSocketHandler methodsFor: 'convenience' stamp: 'HK 1/12/2011 16:25'!
isNew: aWebSocket

	| registeredSockets |
	registeredSockets := self application webSocketClients collect:
		[ :each | each socket ].
	
	^ (registeredSockets includes: aWebSocket) not.! !

!S2JWebSocketHandler methodsFor: 'convenience' stamp: 'SW 1/31/2011 10:11'!
setupClientWith: aWebSocket

	| client |
	client := (S2JWebSocketClient newWithSocket: aWebSocket).
	self application addClient: client.
	"self updateDelayed: client."! !

!S2JWebSocketHandler methodsFor: 'convenience' stamp: 'SW 1/18/2011 10:54'!
updateDelayed: aClient

	[ (Delay forMilliseconds: self class millisecondsToWaitForHandshake) wait. 
	aClient update] fork.! !

S2JResource subclass: #S2JDispatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 13:57'!
defaultPort

	^ 8887! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/18/2011 10:47'!
forcedRestart
	"Why is this even necessary?" "Brought to you by UGLYFIX!! :-)"
	
	SwazooServer initSingleton.
	SwazooServer allInstancesDo: [ :swa | 
		swa stop; initSites].
	HTTPServer allInstancesDo: [ :http |
		http stop; initialize].
	HTTPConnection allInstancesDo: [ :conn |
		conn close].
	SwazooServer singleton start.
	Smalltalk garbageCollect.
	self start.! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
install
	
	^ self installOn: self defaultPort! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 14:16'!
installOn: aPort
	
	^ (SwazooSite
		newNamed: self name)
		host: '*'
		ip: '*'
		port: aPort;
		addResource: (self uriPattern: '/');
		yourself! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
isInstalled
	
	^ self site notNil! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/25/2011 14:16'!
site
	
	^ SwazooServer singleton siteNamed: self name! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:12'!
start
	
	self isInstalled ifFalse: [ self install ].
	self site start.! !

!S2JDispatcher class methodsFor: 'setup' stamp: 'ag 1/13/2011 20:13'!
uninstall
	
	self isInstalled ifTrue: [
		self removeAllClients.
		self resetContent.
		SwazooServer singleton removeSite: self site ].! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 12:27'!
answerTo: aRequest 
	
	| application |
	(self hasToBeForwarded: aRequest ) 
		ifTrue: [ ^ self forward: aRequest ].		
	application := self findApplicationNamed: (self identifierOf: aRequest).
	application
		ifNil: [  ^ self redirectTo: '/', S2JIndex uniquePath ]
		ifNotNil: [ ^ application instance answerTo: aRequest ].! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'hk 1/25/2011 13:51'!
findApplicationNamed: aName

	aName ifNil: [ ^ nil ].
	^ S2JBasicApp allSubclasses
		detect: [ :each | each uniquePath = aName ]
		ifNone: [ nil ]. ! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 12:27'!
forward: aRequest

	| url |
	url:= self identifierOf: aRequest.
	^ self redirectTo: '/', url! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 12:19'!
hasToBeForwarded: aRequest
	
	| url |
	url := aRequest requestLine requestURI identifier.
	^ ( url findTokens: '/') size = 1 and: [ (url endsWith: '/') not ]
	
	
! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'hk 2/1/2011 11:54'!
identifierOf: aRequest
	
	((aRequest requestLine requestURI identifier) findTokens: '/') size = 0 
		ifTrue: [ ^ nil  ]
		ifFalse: [ ^ ((aRequest requestLine requestURI identifier) findTokens: '/') first, '/' ].
	
	
! !

!S2JDispatcher methodsFor: 'dispatching' stamp: 'HK 1/4/2011 15:45'!
redirectTo: aPath

	^HTTPResponse new
		statusCode: 302;
		entity: '';
		location: aPath;
		yourself.! !

!S2JResource methodsFor: 'responding' stamp: 'hk 1/26/2011 10:51'!
answerCode: code
	
	^ self
		answerCode: code
		withType: 'text/javascript'! !

!S2JResource methodsFor: 'responding' stamp: 'hk 1/26/2011 10:27'!
answerCode: code withType: aString
	
	^ self
		answerContent: code
		withType: aString! !

!S2JResource methodsFor: 'responding' stamp: 'HK 12/17/2010 13:57'!
answerContent: data withType: aString

	| response |
	response := HTTPResponse ok.
	^ response
		contentType: aString;
		entity: data;
		yourself.! !

!S2JResource methodsFor: 'responding' stamp: 'HK 12/15/2010 17:15'!
answerCreated

	^ self answerWithStatusCode: 201.! !

!S2JResource methodsFor: 'responding' stamp: 'HK 1/10/2011 13:53'!
answerNotModified

	^ self answerWithStatusCode: 304.! !

!S2JResource methodsFor: 'responding' stamp: 'HK 12/15/2010 17:16'!
answerWithStatusCode: aNumber

	^ self answerWithStatusCode: aNumber andContent: ''.! !

!S2JResource methodsFor: 'responding' stamp: 'HK 12/15/2010 15:47'!
answerWithStatusCode: aNumber andContent: content

	^ HTTPResponse ok
		statusCode: aNumber;
		entity: content.! !

!S2JResource methodsFor: 'printing-helpers' stamp: 'ag 1/13/2011 20:19'!
cr

	^ String cr! !

TestCase subclass: #S2JTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

S2JTestCase subclass: #S2JAbstractTest
	instanceVariableNames: 'currentRule nilResultAllowed checkResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

S2JAbstractTest subclass: #S2JAbstractPrinterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JAbstractPrinterTest methodsFor: 'as yet unclassified' stamp: 'srv 1/10/2011 15:02'!
testExampleMethods
	
	super testExampleMethods.
	
	self class isOnCIServer ifTrue: [
		{ Object. Number. ProtoObject. OrderedCollection }
			do: [ :class | self class converterClass convertClass: class ]].! !

S2JAbstractPrinterTest subclass: #S2JPrettyPrinterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JPrettyPrinterTest class methodsFor: 'testing' stamp: 'lw 1/4/2011 17:46'!
converterClass
	
	^ S2JPrettyPrinter! !

!S2JPrettyPrinterTest class methodsFor: 'testing' stamp: 'lw 1/4/2011 11:23'!
defaultCurrentRule
	
	^ #doItSnippet! !

!S2JPrettyPrinterTest methodsFor: 'test printing' stamp: 'ag 1/18/2011 15:27'!
testIndention

	self
		assertMatches: '[:e | JsGlobal js alert: e. [ :f | f + 1 ]. [ :f | f := f + 2. ^ [JsGlobal js alert: f ] ]]'
		andProduces:  
 '            block(function (e) {
                alert(e.js());
                block(function (f) {
                    return f._plus(number(1));
                });
                return block(function (f) {
                    f = f._plus(number(2));
                    nonLocalReturn(block(function () {
                        return alert(f.js());
                    }));
                });
            })'! !

!S2JPrettyPrinterTest methodsFor: 'test printing' stamp: 'ag 1/18/2011 15:28'!
testLoop

	self
		assertMatches: '{1. 2. 3} do: [:elem | JsGlobal js alert: elem]'
		andProduces: 
 '            array([number(1), number(2), number(3)]).do_(block(function (elem) {
                return alert(elem.js());
            }))'! !

!S2JPrettyPrinterTest methodsFor: 'as yet unclassified' stamp: 'lw 1/12/2011 21:18'!
testSlotAssignment

	self 
		assertMatches: 'a js slotti js: b'
		andProduces: '            a.slotti = b.js()'.! !

!S2JPrettyPrinterTest methodsFor: 'test printing' stamp: 'ag 2/7/2011 21:07'!
testTrueFalse

	self 
		assertMatches: '10 < 5 ifTrue: [ JsGlobal js alert: ''small'' ] ifFalse: [ JsGlobal js alert: ''big'' ]'
		andProduces: 
 '            number(10)._less(number(5)).ifTrue_ifFalse_(block(function () {
                return alert("small");
            }), block(function () {
                return alert("big");
            }))'! !

!S2JPrettyPrinterTest methodsFor: 'test printing' stamp: 'ag 1/13/2011 11:49'!
testVarDeclaration
	
	self 
		assertMatches: '|a b c|'
		andProduces: 
 '            var a; var b; var c'! !

S2JAbstractPrinterTest subclass: #S2JPrinterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JPrinterTest class methodsFor: 'testing' stamp: 'lw 1/4/2011 11:23'!
converterClass
	
	^ S2JPrinter! !

!S2JPrinterTest class methodsFor: 'testing' stamp: 'ag 12/22/2010 13:52'!
defaultCurrentRule
	
	^ #doItSnippet! !

!S2JPrinterTest methodsFor: 'testing' stamp: 'sec 1/4/2011 12:13'!
expectedFailures
	
	^ #( testClientEvaluation )! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'ag 1/10/2011 10:49'!
testAddition
	
	self
		assertMatches: '3+4'
		andProduces: 'number(3)._plus(number(4))'.! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'ag 2/7/2011 21:09'!
testCascadedExpression
	
	self
		currentRule: #cascadedExpression;
		
		assertMatches: 'a b; c'
		andProduces: '(function(){var cascadeReceiver$=a;cascadeReceiver$.b();return cascadeReceiver$.c();}).apply(this)';
		
		assertMatches: 'a b: (a b; d); c'
		andProduces: '(function(){var cascadeReceiver$=a;cascadeReceiver$.b_((function(){var cascadeReceiver$=a;cascadeReceiver$.b();return cascadeReceiver$.d();}).apply(this));return cascadeReceiver$.c();}).apply(this)'! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'HK 11/29/2010 16:31'!
testClientEvaluation

	self fail.
	"clientEval: [ JsGlobal alert: 3+4]"! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'srv 1/18/2011 14:58'!
testEscapingIdentifiers
	
	self
		currentRule: #doItSnippet;
		
		assertMatches: 'Object new'
		andProduces: '_Object._new()';
		
		assertMatches: 'a := true'
		andProduces: 'a=_true'.
	
	self class isOnCIServer ifTrue: [
		self assert: ((S2JCodeFactory new codeForClass: Object) beginsWith: 'Class("_Object",').
		self assert: ((S2JCodeFactory new codeForClass: Point) beginsWith: 'Class("Point",{superclass:_Object')].! !

!S2JPrinterTest methodsFor: 'test general' stamp: 'srv 1/10/2011 15:02'!
testExampleMethods
	
	super testExampleMethods.! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'ag 2/7/2011 21:42'!
testExceptionalSyntax
	
	self
		assertMatches: 'X js Y' "Slot access"
		andProduces: 'X.Y';
		
		assertMatches: 'X js Y js Z'
		andProduces: 'X.Y.Z';
		
		assertMatches: 'X js Y js: Z' "Slot assignment"
		andProduces: 'X.Y=Z.js()';
		
		assertMatches: 'X js Y js: {Z. A}'
		andProduces: 'X.Y=[Z, A]';
		
		assertMatches: 'X js Y: Z' "Slot method call"
		andProduces: 'X.Y(Z.js())';
		
		assertMatches: 'X js Y: 1'
		andProduces: 'X.Y(1)';
		
		assertMatches: 'a js a: (a js a: 1)'
		andProduces: 'a.a(a.a(1))';
		
		assertMatches: 'X js Y: {Z. A}'
		andProduces: 'X.Y(Z.js(), A.js())';
		
		assertMatches: 'X js Y: {1. true}'
		andProduces: 'X.Y(1, true)';
		
		assertMatches: 'X js Y jsNew: Z' "new-expression variant 1"
		andProduces: 'new X.Y(Z.js())';
		
		assertMatches: 'X js Y jsNew: { Z. A }'
		andProduces: 'new X.Y(Z.js(), A.js())';
		
		assertMatches: '#new X: { Z. A }' "new-expression variant 2"
		andProduces: 'new X(Z.js(), A.js())';
		
		assertMatches: '#new X: { 1. false }'
		andProduces: 'new X(1, false)';
		
		assertMatches: '#new X: Z'
		andProduces: 'new X(Z.js())'.! !

!S2JPrinterTest methodsFor: 'test general' stamp: 'robi 1/17/2011 18:01'!
testJsStringPrinting
	
	self
		assert: [ '"\\\"''"' = (S2JAbstractPrinter asJSString: '\"''')];
		assert: [ '"abc def ()/&%"' = (S2JAbstractPrinter asJSString: 'abc def ()/&%')];
		assert: [ '"\r"' = (S2JAbstractPrinter asJSString: String cr)].! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'ag 1/18/2011 15:28'!
testLoop
	
	self
		assertMatches: '{1. 2. 3} do: [:elem | JsGlobal js alert: elem]'
		andProduces: 'array([number(1), number(2), number(3)]).do_(block(function(elem){return alert(elem.js());}))';
		
		assertMatches: 'arr do: [:elem | JsGlobal js alert: elem ]'
		andProduces: 'arr.do_(block(function(elem){return alert(elem.js());}))';
		
		assertMatches: 'arr do: block'
		andProduces: 'arr.do_(block)'.! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'ag 2/7/2011 21:30'!
testNonLocalReturn
	
	self
		assertMatches: '
			[ JsGlobal js alert: ''outer''.
			[JsGlobal js alert: ''inner''.
			 ^self].
			JsGlobal js alert: ''end'' ]'
		andProduces: 
			'block(function(){alert("outer");block(function(){alert("inner");nonLocalReturn(this);});return alert("end");})'! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'ag 1/19/2011 19:53'!
testPerformOnServer
	
	self
		assertMatches: '[1] onServer'
		andProduces: 'serverBlock("[1]")';
		
		assertMatches: '[:a|1] onServer'
		andProduces: 'serverBlock("[:a|1]")';
		
		assertMatches: '[ [ 1 ] onServer ] onServer'
		andProduces: 'serverBlock("[ [ 1 ] onServer ]")'.! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'ag 1/13/2011 11:42'!
testPrinting
	
	self
		assert: '1 +/ 2' matches: #binaryExpression andProduces: 'number(1)._plus_slash(number(2))';
		assert: '{ 1. a b: c. ''s'' }' matches: #braceExpression andProduces: 'array([number(1), a.b_(c), string("s")])';
		assert: 'a :=   12 a: 12' matches: #expression andProduces: 'a=number(12).a_(number(12))';
		assert: '1 a: 2 b: 3' matches: #keywordExpression andProduces: 'number(1).a_b_(number(2), number(3))';
		assert: '1 a:a b:B. 2 s.' matches: #doItSnippet andProduces: 'number(1).a_b_(a, B);number(2).s()';
		assert: '| a b v |' matches: #temporaries andProduces: 'var a;var b;var v';
		assert: '1 bcd' matches: #unaryExpression andProduces: 'number(1).bcd()';
		assert: '(1 2 3 ''d  '')' matches: #array andProduces: 'array([number(1), number(2), number(3), string("d  ")])';
		assert: '#[ 1 2 3 255]' matches: #byteArray andProduces: 'array([number(1), number(2), number(3), number(255)])';
		assert: '12.3' matches: #number andProduces: 'number(12.3)';
		assert: 'self' matches: #reservedWord andProduces: 'this';
		assert: '#a12' matches: #symbol andProduces: 'string("a12")'.! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'LT 1/4/2011 20:12:13.487'!
testSelectorTranslation
	
	| assertBlock |
	assertBlock := [ :squeakSelector :jsFunctionname  | 
		self assert: jsFunctionname equals: (S2JSqueakAstToJsAst jsFunctionNameFor: squeakSelector).].
	
	assertBlock
		value: 'do:forDuration:' value: 'do_forDuration_';
		value: 'do:' value: 'do_';
		value: 'do' value: '_do';
		value: '*+' value: '_times_plus';
		value: '++' value: '_plus_plus'.! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'lw 1/12/2011 21:19'!
testSlotAssignment

	self 
		assertMatches: 'a js slotti js: b'
		andProduces: 'a.slotti=b.js()'.! !

!S2JPrinterTest methodsFor: 'test printing' stamp: 'ag 1/24/2011 19:16'!
testSuper
	
	self
		assertMatches: 'super a'
		andProduces: '_super("a")()';
		
		assertMatches: 'super a: 1 b: d'
		andProduces: '_super("a_b_")(number(1), d)';
		
		assertMatches: 'a := super'
		andProduces: 'a=this';
		
		assertMatches: '^super'
		andProduces: 'return this'.! !

!S2JPrinterTest methodsFor: 'test stories' stamp: 'ag 2/7/2011 21:30'!
testTrueFalse

	self 
		assertMatches: '10 < 5 ifTrue: [ JsGlobal js alert: ''small'' ] ifFalse: [ JsGlobal js alert: ''big'' ]'
		andProduces:
			'number(10)._less(number(5)).ifTrue_ifFalse_(block(function(){return alert("small");}), block(function(){return alert("big");}))'! !

!S2JAbstractTest class methodsFor: 'testing' stamp: 'ag 12/9/2010 13:15'!
converterClass
	
	self subclassResponsibility! !

!S2JAbstractTest class methodsFor: 'testing' stamp: 'ag 12/13/2010 09:23'!
defaultCheckResults
	
	^ true! !

!S2JAbstractTest class methodsFor: 'testing' stamp: 'ag 12/13/2010 09:24'!
defaultCurrentRule
	
	^ nil! !

!S2JAbstractTest class methodsFor: 'testing' stamp: 'ag 12/13/2010 09:23'!
defaultNilResultsAllowed
	
	^ false! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:29'!
assert: aString matches: aRule
	"In most cases, throws an error when failing (hard fail)"
	
	| result |
	result := self class converterClass convert: aString as: aRule.
	self nilResultAllowed ifFalse: [ self assert: result notNil ].
	^ result! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:17'!
assert: aString matches: aRule andProduces: anObject
	"In most cases, throws an error when failing (hard fail). #= Checks collections deeply."
	
	| result |
	result := self assert: aString matches: aRule.
	self checkResults ifTrue: [ self assert: result = anObject ].
	^ result! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/13/2010 08:46'!
assert: aString matches: aRule andProducesNode: aCollection
	"Matches the string to the rule and compares the result to an S2JNode containing the given collection and named like the rule.
	(Follows a convention)."
	
	| result |
	result := self assert: aString matches: aRule.
	self checkResults ifTrue: [ self assert: result = (self node: aRule asString with: aCollection) ].
	^ result! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 15:57'!
assertConverting: anInputString containsNodeNamed: aNodeName
	
	^ self assert: 
		(( self assertMatches: anInputString )
			includesChildNamed: aNodeName).! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
assertFails: aString
	"Checks against the currentRule"
	
	self
		assertFails: aString
		toMatch: self currentRule.! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
assertFails: aString toMatch: aRule
	
	self
		should: [ self assert: aString matches: aRule ]
		raise: OM2Fail
		description: 'Matching should have failed but didn''t!!'.! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 15:57'!
assertMatches: aString
	"Check against the currentRule"
	
	^ self assert: aString matches: self currentRule! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
assertMatches: aString andProduces: anObject
	"Check against the currentRule"
	
	^ self
		assert: aString
		matches: self currentRule
		andProduces: anObject! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
assertMatches: aString andProducesNode: aCollection
	"Check against the currentRule"
	
	^ self
		assert: aString
		matches: self currentRule
		andProducesNode: aCollection! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:18'!
checkResults
	^ checkResults! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:18'!
checkResults: aBoolean
	checkResults := aBoolean! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:16'!
currentRule
	^ currentRule! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:16'!
currentRule: aSymbol
	currentRule := aSymbol! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 15:56'!
denyConverting: anInputString containsNodeNamed: aNodeName
	
	self deny: 
		(( self assertMatches: anInputString )
			includesChildNamed: aNodeName).! !

!S2JAbstractTest methodsFor: 'test general' stamp: 'ag 12/13/2010 09:18'!
exampleMethod1
	
	"Comment"
	self class.
	^ [ JsGlobal alter: 'example'].! !

!S2JAbstractTest methodsFor: 'test general' stamp: 'ag 12/13/2010 09:18'!
exampleMethod2
	
	| temporaries |
	< primitiveDecl: 1 >
	"Comment"
	self class.
	self
		a: 1;
		b: 2;
		c.
	^ [ JsGlobal alter: 'example'].! !

!S2JAbstractTest methodsFor: 'test general' stamp: 'ag 12/13/2010 09:22'!
""exampleMethod3""
	""
	""<"" primitiveDecl: ""1 "">""
	""
	""| ""temporariesAfterPrimitiveDecl"" |""
	"Comment"
	""self"" class"".""
	self""
		""a:"" 1"";""
		""b: ""2"";""
		""c"".""
	""^"" [""JsGlobal ""alter:"" 'example'""]"".""! !

!S2JAbstractTest methodsFor: 'initialization' stamp: 'ag 12/13/2010 09:24'!
initialize
	
	super initialize.
	self
		nilResultAllowed: self class defaultNilResultsAllowed;
		checkResults: self class defaultCheckResults;
		currentRule: self class defaultCurrentRule.! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:16'!
nilResultAllowed
	^ nilResultAllowed! !

!S2JAbstractTest methodsFor: 'accessing' stamp: 'ag 12/9/2010 13:18'!
nilResultAllowed: aBoolean
	nilResultAllowed := aBoolean! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
node: aString
	
	^ S2JNode named: aString with: {}! !

!S2JAbstractTest methodsFor: 'helpers' stamp: 'ag 12/9/2010 13:14'!
node: aString with: aCollection
	
	^ S2JNode named: aString with: aCollection! !

!S2JAbstractTest methodsFor: 'private' stamp: 'ag 1/11/2011 11:42'!
performTest
	
	self class = S2JAbstractTest | (self class = S2JTestCase) | (self class = S2JAbstractPrinterTest)
		ifTrue: [ "Just ignore the abstract tests" ]
		ifFalse: [ super performTest ].! !

!S2JAbstractTest methodsFor: 'test general' stamp: 'srv 1/10/2011 15:00'!
testExampleMethods
	
	self class converterClass
		convertMethod: #exampleMethod1 of: self class;
		convertMethod: #exampleMethod2 of: self class;
		convertMethod: #exampleMethod3 of: self class.! !

S2JAbstractTest subclass: #S2JParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JParserTest class methodsFor: 'testing' stamp: 'ag 12/9/2010 13:24'!
converterClass
	
	^ S2JParser! !

!S2JParserTest methodsFor: 'testing' stamp: 'ag 12/16/2010 17:03'!
expectedFailures
	
	^ #( #testComments )! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/7/2010 17:16'!
testArray
	
	self
		currentRule: #array;
		
		assertMatches: '()' andProducesNode: {};
		assertMatches: '(:)' andProducesNode: { self node: 'symbol' with: { #: } };
		assertMatches: '(  )  ' andProducesNode: {};
		assertMatches: '( 2. 1) ';
		assertMatches: '( 2. 1 [ ] ^ 23 { } ) ';
		assertMatches: '(  1 2   3) ' andProducesNode: 
			{self node: 'number' with:{1}. self node: 'number' with:{2}. self node: 'number' with:{3}};
		assertMatches: '(''aa'' 1 ''b1'' 32)' andProducesNode: 
			{self node: 'string' with: { 'aa' }. self node: 'number' with: {1}. self node: 'string' with: { 'b1' }. self node: 'number' with: {32}};
		assertMatches: '(#abc     11    ''a''   )' andProducesNode: 
			{self node: 'symbol' with: {#abc}. self node: 'number' with: {11}. self node: 'string' with: {'a'}};
		assertMatches: '(  ( 1 2 3    ''2'')   #a  ''c''   )' andProducesNode: 
			{self node: 'array' with: {self node: 'number' with: {1}. self node: 'number' with: {2}. self node: 'number' with: {3}. self node: 'string' with: { '2' }}.
			self node: 'symbol' with: {#a}. self node: 'string' with: {'c'}};
		assertMatches: '($a     11    ''a''   #a2   #(1 2   $2) )  ' andProducesNode: { self node: 'character' with: { $a }. self node: 'number' with: {11}. self node: 'string' with: {'a'}. self node: 'symbol' with: {#a2}. self node: 'array' with: {self node: 'number' with: {1}. self node: 'number' with: {2}. self node: 'character' with: { $2 }}};
		assertMatches: '( a )' andProducesNode: { self node: 'symbol' with: {'a'}};
		assertMatches: '( a b )' andProducesNode: {self node: 'symbol' with: {'a'}. self node: 'symbol' with: {'b'}} ;
		assertMatches: '(d ((b ((a))) (c)))' andProducesNode: {self node: 'symbol' with: {'d'}. self node: 'array' with: {self node: 'array' with: {self node: 'symbol' with: {'b'}. 
			self node: 'array' with: {self node: 'array' with: {self node: 'symbol' with: {'a'}}}}. self node: 'array' with: {self node: 'symbol' with: {'c'}}}};
		assertMatches: '( self a:bc == |? ?!!!! )  ' andProducesNode: {self node: 'symbol' with: {'self'}. self node: 'symbol' with: {'a:bc'}. 
			self node: 'symbol' with: {'=='}. self node: 'symbol' with: {'|'}. self node: 'symbol' with: {'?'}. self node: 'symbol' with: {'?!!!!'}};
		assertMatches: '( self #( aa 1 ) ( ''a'' #1 #ab: ab:))  ' andProducesNode: { self node: 'symbol' with: {'self'}. self node: 'array' with: {self node: 'symbol' with: {'aa'}. self node: 'number' with: {1}. }. self node: 'array' with: { self node: 'string' with: { 'a' }. self node: 'number' with: {1}. self node: 'symbol' with: {'ab:'}. self node: 'symbol' with: {'ab:'} }} ;
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '(';
		assertFails: '#()';
		assertFails: '#(';
		assertFails: '{1}';
		assertFails: ' ( 1 2  )'. "Should not read prepended spaces"! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/16/2010 17:01'!
testBinaryExpression
	
	self
		currentRule: #binaryExpression;
		
		assertMatches: 'a+b' andProducesNode:  (self node:'binaryExpression' with: { self node: 'identifier' with: {'a'} . #+. self node: 'identifier' with: {'b'}} );
		assertMatches: '1 * b' andProducesNode: (self node:'binaryExpression' with: 
			{self node: 'number' with: {1} .#* . self node: 'identifier' with: {'b'}});
		assertMatches: '1 \ b / c' andProducesNode: 
			(self node: 'binaryExpression' with: 
				{ self node: 'binaryExpression' with: {  self node: 'number' with: {1} .  #\  . 
				self node: 'identifier' with: {'b'}} . #/ . self node: 'identifier' with: {'c'}} );
		
		assertFails: 'a 2';
		assertFails: 'a: 2';
		assertFails: '+ 2'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 11/21/2010 20:02'!
testBinarySelector
	"Would be cool to find all the binary messages in the image and test them..."
	
	self
		currentRule: #binarySelector;
		
		"special cases: single pipe and single minus"
		assertMatches: '|' andProduces: #|;
		assertMatches: '-' andProduces: #-;
		
		"names starting with a minus"
		assertMatches: '-*' andProduces: #-*;
		assertMatches: '-***' andProduces: #-***;
		assertMatches: '-*%&' andProduces: #-*%&;
		assertMatches: '-*/\' andProduces: #-*/\;
		assertMatches: '-!!?' andProduces: #-!!?;
		assertMatches: '-==' andProduces: #-==;
		
		"mixed characters, check each one"
		assertMatches: '*+' andProduces: #*+;
		assertMatches: '/=\' andProduces: #/=\;
		assertMatches: '~,,,<' andProduces: #~,,,<;
		assertMatches: '>@' andProduces: #>@;
		assertMatches: '%&' andProduces: #%&;
		assertMatches: '?!!' andProduces: #?!!;
		assertMatches: '``' andProduces: #``;
		assertMatches: '+??=' andProduces: #+??=;
		assertMatches: ',!!!!,' andProduces: #,!!!!,;
		
		"Not allowed things"
		assertFails: '';
		assertFails: ' ';
		assertFails: 'w';
		assertFails: '2';
		assertFails: '--';
		assertFails: '||';
		assertFails: '|--';
		assertFails: '%--';
		assertFails: '%||';
		assertFails: '&-&-';
		assertFails: '- %';
		assertFails: ' -';
		assertFails: '& &';
		assertFails: 'w a';
		assertFails: '- -';
		assertFails: '#a a';
		assertFails: '#aa 2'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/16/2010 17:01'!
testBlock
	
	self
		currentRule: #block;
		
		assertMatches: '[]' andProducesNode: { };
		assertMatches: '[  ]' andProducesNode: { };
		assertMatches: '[||]' andProducesNode: {self node: 'temporaries'};
		assertMatches: '[:a|||]' andProducesNode: 
			{ self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. self node: 'temporaries'};
		assertMatches: '[||a]' andProducesNode: {self node: 'temporaries'. self node: 'statements' with: {self node: 'expression' with: {self node: 'identifier' with: {'a'}}}};
		assertMatches: '[:a|||a]' andProducesNode:
			{ self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. self node: 'temporaries'. self node: 'statements' with: { self node: 'expression' with: {self node: 'identifier' with: {'a'}}}};
		assertMatches: '[:a||a|a]' andProducesNode: 
			{ self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. self node: 'temporaries' with: { self node: 'identifier' with: {'a'}}. 
			self node: 'statements' with: { self node: 'expression' with: { self node: 'identifier' with: {'a'}}}};
		assertMatches: '[:a|a|a|a]' andProducesNode:
			{ self node: 'arguments' with: { self node: 'identifier' with: {'a'} }. self node: 'statements' with:
				{ self node: 'expression' with: { self node: 'binaryExpression' with: { self node: 'binaryExpression' with: 
					{ self node: 'identifier' with: {'a'}. #|. self node: 'identifier' with: {'a'} }. #| . 
					self node: 'identifier' with: {'a'}.  }}}};
		assertMatches: '[ :a ]  ' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'a'}}};
		assertMatches: '   [ :a ]  ' andProducesNode: {self node: 'arguments' with: { self node: 'identifier' with: {'a'}}};
		assertMatches: '[ :a :b ]  ' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'a'}. self node: 'identifier' with: {'b'}}};
		assertMatches: '[ :a  :b | ]' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'a'}. self node: 'identifier' with: {'b'}}};
		assertMatches: '  [   :a |   |  a  |   ]  ' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. 
			self node: 'temporaries' with: {self node: 'identifier' with: {'a'}}};
		assertMatches: '[  :a  | a ]  ' andProducesNode:
			{self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. 
			self node: 'statements' with: {self node: 'expression' with: {self node: 'identifier' with: {'a'}}}};
		assertMatches: '[ :a  | a. ^ 1 ]' andProducesNode:
			{self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. self node: 'statements' with:
				{self node: 'expression' with: {self node: 'identifier' with: {'a'}}. self node: 'returnExpression' with: {self node: 'number' with: {1}}}};
		assertMatches: '[  :a  |  1  ]' andProducesNode:
			{self node: 'arguments' with: {self node: 'identifier' with: {'a'}}. self node: 'statements' with: {self node: 'expression' with: {self node: 'number' with: {1}}}};
		assertMatches: '[  :b  |   []  ]' andProducesNode:
			{self node: 'arguments' with: {self node: 'identifier' with: {'b'}}. self node: 'statements' with: {self node: 'expression' with: { self node: 'block'}}};
		
		"Can also leave spaces between : and the argument-name !!"
		assertMatches: '[ :  i | ]' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'i'}}};
		assertMatches: '[ :  i :  d | ]' andProducesNode: {self node: 'arguments' with: {self node: 'identifier' with: {'i'}. self node: 'identifier' with: {'d'}}};
		
		assertFails: '[]]';
		assertFails: '[[]';
		assertFails: '[::a]';
		assertFails: '[ :a | b';
		assertFails: '[ :a |||| b]';
		assertFails: '[ | :a | b]';
		assertFails: '[ || :a | b]';
		assertFails: '[ :a || b]';
		assertFails: '[ :a | :b  ]';
		assertFails: '[ :a || :b|]';
		assertFails: '[ :a | b| :a]';
		assertFails: '[ :a :1 | b]';
		assertFails: '[ aaaa"] ';
		assertFails: '[ "aaa"a"] ';
		assertFails: '[ "aaaa] '.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/16/2010 17:02'!
testBraceExpression
	
	self
		currentRule: #braceExpression;
		
		assertMatches: '{}' andProducesNode: { };
		assertMatches: '  {  }   ' andProducesNode: { };
		assertMatches: '{1}' andProducesNode: {self node: 'expression' with: {self node: 'number' with: {1}}};
		assertMatches: '  {  11 }  ' andProducesNode: {self node: 'expression' with: {self node: 'number' with: {11}}};
		assertMatches: '  {  11. }  ' andProducesNode: {self node: 'expression' with: { self node: 'number' with: {11}}};
		assertMatches: '{ 1.   2}' andProducesNode: {self node: 'expression' with: 
			{self node: 'number' with: {1}}. self node: 'expression' with: { self node: 'number' with: {2}}};
		assertMatches: '{1.2}' andProducesNode: {self node: 'expression' with: {self node: 'number' with: {1.2}}};
		assertMatches: '{1. 2}' andProducesNode: { self node: 'expression' with: {self node: 'number' with: {1}}. 
			self node: 'expression' with: {self node: 'number' with: {2}}};
		assertMatches: '{1. ''.'' .}' andProducesNode: { self node: 'expression' with: {self node: 'number' with: {1}}. 
			self node: 'expression' with: {self node: 'string' with: { '.'}}};
		assertMatches: '{1  .  2  .}' andProducesNode: { self node: 'expression' with: {self node: 'number' with: {1}}. 
			self node: 'expression' with: {self node: 'number' with: {2}}};		
		
		assertFails: '.{}.';
		assertFails: '{1 .. 2}';
		assertFails: '{ 1  . 2 ..}';
		assertFails: '{.1.2}';
		assertFails: '{..1.}';
 		assertFails: '{ "bb}';
 		assertFails: '{ "b "   .  b"}'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/9/2010 13:40'!
testByteArray
	
	self
		currentRule: #byteArray;
		
		assertMatches: '#[1]' andProducesNode: {1};
		assertMatches: '#  [  1  ]  ' andProducesNode: {1};
		assertMatches: '#[]' andProducesNode: {};
		assertMatches: '#   [1 2 3 ]' andProducesNode: {1. 2. 3.};
		assertMatches: '#[255 000 000 123 255]' andProducesNode: {255. 0. 0. 123. 255};
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '#[ 1';
		assertFails: '  #[ 1 ]';
		assertFails: '# [ 1.2 ]';
		assertFails: '#[ 256 1 00]';
		assertFails: '#[ -1 ]';
		assertFails: '#[ +1 ]';
		assertFails: '#[ #[ 1 ] ]  ';
		assertFails: '#( 255 )'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/7/2010 10:09'!
testCascadedExpression
	
	self
		currentRule: #cascadedExpression;
		assertMatches: 'Transcript  show:2;cr' andProducesNode: (self node: 'cascadedExpression' with:
				{self node: 'identifier' with:{'Transcript'}.  self node: 'keywordExpression' with: {#show: . self node: 'number' with: {2}} .self node:'unaryExpression' with:{#cr}});
		"assertMatches: 'Transcript  show:2 ""Comment"";cr' andProducesNode: (self node: 'cascadedExpression' with:
				{self node: 'identifier:' with:{'Transcript'}. self node: 'keywordExpression' with: {#show: . self node: 'number' with: {2}} .self node:'unarayExpression' with:{self node:'comment' with:{'Comment'} . #cr} } );" "comments are not yet stored correctly"

		assertFails: 'Transcript  show:2 "Comment;cr';				
		assertFails: 'Transcript show:1'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/7/2010 11:14'!
testCharacter
	
	self
		currentRule: #character;
		
		assertMatches: '$c' andProducesNode: { $c };
		assertMatches: '$&' andProducesNode: { $& };
		assertMatches: '$ ' andProducesNode: { $ };
		assertMatches: '$`' andProducesNode: { $` };
		assertMatches: 'Character space' andProducesNode: { $  };
		
		assertFails: '';
		assertFails: ' ';
		assertFails: 'a';
		assertFails: '`';
		assertFails: ' $a';
		assertFails: '$$$';
		assertFails: '$  a';
		assertFails: '$av';
		assertFails: '$ a'.! !

!S2JParserTest methodsFor: 'test general' stamp: 'ag 12/16/2010 17:03'!
testComments
	
	self
		currentRule: #binaryExpression;
		assertMatches: '1 "Comment "* b' andProducesNode: (self node:'binaryExpression' with: 
			{ self node: 'comment' with:{ 'Comment ' } . self node: 'number' with: {1}. #*. 
			self node: 'identifier' with: {'b'}});
		
		currentRule: #block;
		assertMatches: '"aaaaa" "aaa"[]'  andProducesNode: { self node:'comment' with: {'aaaaaaaaa' } };
		assertMatches: '"aaaaa" ["a aa"]'  andProducesNode: { self node:'comment' with: {'aaaaa' } . self node:'comment' with: {'a aaa' }};
		assertMatches: '[ "a aaa"] '  andProducesNode: { self node:'comment' with: {'a aaa' } };
		assertMatches: '[ "aaaa"] '  andProducesNode: { self node:'comment' with: {'aaaa' } };
		
		assertMatches: '"aaa"{}' andProducesNode: { self node: 'comment' with: {'aaa' } };
		assertMatches: '"aaa"{ "bbb"}' andProducesNode: { self node: 'comment' with: {'aaabbb' } };
		assertMatches: '{ "bbb"}' andProducesNode: { self node: 'comment' with: {'bbb' } }.! !

!S2JParserTest methodsFor: 'test general' stamp: 'ag 12/13/2010 09:15'!
testExampleMethods
	
	super testExampleMethods! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/7/2010 16:54'!
testExpressionAssignment
	
	self
		currentRule: #expression;
		
		assertMatches: 'a' andProducesNode: {self node: 'identifier' with: {'a'}};
		assertMatches: 'a := b' andProducesNode: {self node: 'identifier' with: {'b'}. 
			self node: 'identifier' with: {'a'}};
		assertMatches: '  a   :=   b  ' andProducesNode: {self node: 'identifier' with: {'b'}. 
			self node: 'identifier' with: {'a'}};
		assertMatches: 'a := b := c := 1' andProducesNode: {self node: 'number' with: {1}. 
			self node: 'identifier' with: {'a'}. self node: 'identifier' with: {'b'}. self node: 'identifier' with: {'c'}};
		assertMatches: '  a    :=  b  :=  c :=   1' andProducesNode: {self node: 'number' with: {1}. 
			self node: 'identifier' with: {'a'}. self node: 'identifier' with: {'b'}. self node: 'identifier' with: {'c'}};
		assertMatches: 'a _ b' andProducesNode: {self node: 'identifier' with: {'b'}. 
			self node: 'identifier' with: {'a'}};
		assertMatches: 'a := 1';
		assertMatches: 'a := b := 1';
		assertMatches: 'a := b := c := d';
		assertMatches: '   a     :=      a     ';
		assertMatches: '   a     _   1 ';
		assertMatches: 'a:=1';
		assertMatches: 'a:=1.0';
		assertMatches: 'a:=b:=1.0';
		assertMatches: 'a_v_1';
		
		assertFails: 'a : = b';
		assertFails: ' := b';
		assertFails: 'a := ';
		assertFails: 'a := b :=';
		assertFails: 'a := b := 1 := a';
		assertFails: 'a := := b   ';
		assertFails: 'a _ _ b';
		assertFails: 'a __ b';
		assertFails: 'a :=:= b';
		assertFails: 'a := b v := a';
		assertFails: 'a  b := a ';
		assertFails: 'a := 1 := b';
		assertFails: 'a: := b';
		assertFails: 'a: _ b';
		assertFails: 'a :_ b';
		assertFails: 'a := 1 := 2:=f';
		assertFails: 'a __ b';
		assertFails: 'a : = b'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'LT 12/6/2010 11:22:53.039'!
testIdentifier
	
	self
		currentRule: #identifier;
		
		assertMatches: 'abc' andProducesNode: {'abc'};
		assertMatches: 'a1' andProducesNode: {'a1'};
		assertMatches: '1' andProducesNode: {'1'};
		assertMatches: 'aaaaaaaa1233333' andProducesNode: {'aaaaaaaa1233333'};
		assertMatches: '' andProducesNode: {''};
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '|';
		assertFails: ':';
		assertFails: '#abc';
		assertFails: '1abc';
		assertFails: '123';
		assertFails: 'abc';
		assertFails: 'abc:';
		assertFails: 'abc<<'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 11/22/2010 11:30'!
testKeywordSelector
	
	self
		currentRule: #keyword;
		
		assertMatches: 'abc:' andProduces: #abc:;
		assertMatches: 'a1:' andProduces: #a1:;
		assertMatches: '1:' andProduces: #1:;
		assertMatches: 'aaaaaaaa1233333:' andProduces: #aaaaaaaa1233333:;
		assertMatches: ':' andProduces: #:;
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '#abc';
		assertFails: '1abc';
		assertFails: '123';
		assertFails: 'abc';
		assertFails: 'abc';
		assertFails: 'abc<<'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/13/2010 09:27'!
testMethod
	
	self
		currentRule: #method;
		
		assertFails: 'a |a| <p> |a|'. "Don't allow two temporaries-declarations"! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'LT 12/6/2010 11:22:08.233'!
testNumber
	
	self
		currentRule: #number;
		
		assertMatches: '1' andProducesNode: {1};
		assertMatches: '0' andProducesNode: {0};
		assertMatches: '123' andProducesNode: {123};
		assertMatches: '112345678' andProducesNode: {112345678};
		assertMatches: '000' andProducesNode: {0};
		assertMatches: '-000' andProducesNode: {0};
		assertMatches: '-1' andProducesNode: {-1};
		assertMatches: '-1.22' andProducesNode: {-1.22};
		assertMatches: '1.22e23' andProducesNode: {1.22e23};
		assertMatches: '122e23' andProducesNode: {12200000000000000000000000};
		assertMatches: '35r35' andProducesNode: {35r35};
		assertMatches: '-35r35' andProducesNode: {-35r35};
		assertMatches: '22r12.12' andProducesNode: {22r12.12};
		assertMatches: '22r12.12e12' andProducesNode: {22r12.12e12};
		assertMatches: '-22r-12.12e12' andProducesNode: {-22r-12.12e12};
		assertMatches: '22r-12.12e12' andProducesNode: {22r-12.12e12};
		assertMatches: '-22r12.12e12' andProducesNode: {-22r12.12e12};
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '+1';
		assertFails: '+1.2';
		assertFails: '+23r23.3';
		assertFails: '+23r+233.3';
		assertFails: '#abc';
		assertFails: '1abc';
		assertFails: 'abc';
		assertFails: '<<'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/7/2010 15:35'!
testPrimitiveDeclaration
	
	self
		currentRule: #primitiveDeclaration;
		
		assertMatches: '<a:1b:1>';
		assertMatches: '<a>';
		assertMatches: '<  a>';
		assertMatches: '  <  a  >';
		assertMatches: '< | 1>';
		assertMatches: '< == 2>';
		assertMatches: '<  var: ''anc'' >';
		assertMatches: '< & ''add''>';
		assertMatches: '< abc: 22 dfe: 23>';
		
		assertFails: '<1>';
		assertFails: '< 1 2 >';
		assertFails: '<a: 1 1>';
		"assertFails: '<v: d>';" "These should not match, but match currently. Never mind, is not allowed by the compiler"
		"assertFails: '<d: 2 d: e>';"
		"assertFails: '< | d >';"
		assertFails: '<''2''>';
		assertFails: '<| 1>';
		assertFails: '<< 2>';
		assertFails: '< > 2>';
		assertFails: '<> 2>';
		assertFails: '< a: 2 ';
		assertFails: '< a ';
		assertFails: '< | 2 '.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/16/2010 16:01'!
testReservedWord
	
	self
		currentRule: #statements;
		
		assertConverting: '1 a; b: self; c' containsNodeNamed: 'reservedWord';
		assertConverting: '{1. self}' containsNodeNamed: 'reservedWord';
		assertConverting: '{1. nil}' containsNodeNamed: 'reservedWord';
		assertConverting: '{1. true.nil,true}' containsNodeNamed: 'reservedWord';
		assertConverting: '{1. true.nil,true}' containsNodeNamed: 'reservedWord';
		assertConverting: '{true|nil,true}' containsNodeNamed: 'reservedWord';
		assertConverting: '{1. true.nil.{true}}' containsNodeNamed: 'reservedWord';
		denyConverting: '{1. self2. true12}' containsNodeNamed: 'reservedWord';
		denyConverting: '{1. aNself}' containsNodeNamed: 'reservedWord';
		denyConverting: '#(1 self super)' containsNodeNamed: 'reservedWord';
		assertConverting: '#(true)' containsNodeNamed: 'reservedWord';
		
		currentRule: #primary;
		
		assertConverting: '(self)' containsNodeNamed: 'reservedWord';
		assertConverting: '(self/1)' containsNodeNamed: 'reservedWord';
		assertConverting: '([self])' containsNodeNamed: 'reservedWord';
		assertConverting: '([self.])' containsNodeNamed: 'reservedWord';
		assertMatches: '(self2)' andProduces: (self node: 'paranthesizedExpression' with: { self node: 'identifier' with: { 'self2' }});
		denyConverting: '([self2])' containsNodeNamed: 'reservedWord'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'ag 12/7/2010 15:53'!
testStatements

	"TODO Not complete test !! Write more !! " 
	
	self
		currentRule: #statements;
		nilResultAllowed: true;
		
		assertMatches: '...';
		assertMatches: 'a. b. ^ c';
		assertMatches: 'a.b.c';
		assertMatches: '1 a. b d: d d: 1. 2. ^ a d: d f: 2';
		assertMatches: '  a  .  b   .. . . . .. . . . .   c.  ^ 1.';
		
		assertFails: ' a. ^ 1...';
		assertFails: ' 1. a ^ 1';
		assertFails: '^ d d. 1';
		assertFails: '^ .';
		assertFails: '^....';
		assertFails: '^^'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/1/2010 17:35'!
testString
	
	self
		currentRule: #string;
		
		assertMatches: '''a''' andProduces: (self node: 'string' with: { 'a' });
		assertMatches: '''''' andProduces: (self node: 'string' with: { '' });
		assertMatches: '''123''' andProduces: (self node: 'string' with: { '123' });
		assertMatches: ''' '''' '''' ''' andProduces: (self node: 'string' with: { ' '' '' ' });
		assertMatches: '''''''123''' andProduces: (self node: 'string' with: { '''123' });
		assertMatches: '''    ''' andProduces: (self node: 'string' with: { '    ' });
		assertMatches: '''"$%&''' andProduces: (self node: 'string' with: { '"$%&' });
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '#abc';
		assertFails: ''' ''a';
		assertFails: ' ''''';
		assertFails: 'a''''';
		assertFails: '''as';
		assertFails: 'as''';
		assertFails: ''' '' ''';
		assertFails: '''a '' a''';
		assertFails: '''/'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/7/2010 17:32'!
testSymbol
	
	"TODO extend test, if stringS2J gets extended to support '' inside of a string"
	
	self
		currentRule: #symbol;
		
		assertMatches: '#abc' andProducesNode: {#abc};
		assertMatches: '# ''''' andProducesNode: {#''};
		assertMatches: '#   abc' andProducesNode: {#  abc};
		assertMatches: '#abc1' andProducesNode: {#abc1};
		assertMatches: '#a' andProducesNode: {#a};
		assertMatches: '#''a''' andProducesNode: {#a};
		assertMatches: '#''a b c''' andProducesNode:  {#'a b c'};
		assertMatches: '#''''' andProducesNode: {#''};
		assertMatches: '#''123''' andProducesNode:  {#'123'};
		assertMatches: '#''    ''' andProducesNode: {#'    '};
		assertMatches: '#''"$%&''' andProducesNode: {#'"$%&'};
		assertMatches: '#abc1:' andProducesNode: {#abc1:};
		assertMatches: '#|' andProducesNode: {#|};
		assertMatches: '#-&&' andProducesNode: {#-&&};
		assertMatches: '#-' andProducesNode: {#-};
		assertMatches: '#a:b:c1' andProducesNode: {#a:b:c1};
		assertMatches: '#:' andProducesNode: {#:};
		
		assertFails: '';
		assertFails: ' ';
		assertFails: 'abc';
		assertFails: '#1abc';
		assertFails: '#""';
		assertFails: '#'' ''a';
		assertFails: '#a''''';
		assertFails: '#''as';
		assertFails: '#as''';
		assertFails: '#'' '' ''';
		assertFails: '#''a '' a''';
		assertFails: '#123';
		assertFails: '###';
		assertFails: '#''/'.! !

!S2JParserTest methodsFor: 'test complex' stamp: 'LT 12/6/2010 11:38:46.1'!
testUnaryExpression
	
	self
		currentRule: #unaryExpression;
		
		assertMatches: 'a b' andProducesNode: { self node: 'identifier' with: {'a'}. #b };
		assertMatches: '1 b' andProducesNode: { self node: 'number' with: {1}. #b };
		assertMatches: 'a b c' andProducesNode: { self node: 'unaryExpression' 
			with: { self node: 'identifier' with: {'a'}. #b }. #c };
		assertMatches: ' a  b c d ' andProducesNode: { self node: 'unaryExpression' with: { self node: 'unaryExpression' with: { self node: 'identifier' with: {'a'}. #b }. #c }. #d };
		assertMatches: '   ''abc''    b1     ' andProducesNode: { self node: 'string' with: { 'abc' }. #b1 };
		
		assertFails: '';
		assertFails: '  ';
		assertFails: 'a';
		assertFails: '  a  ';
		assertFails: 'a  1';
		assertFails: 'a b 1';
		assertFails: 'a || b';
		assertFails: 'a % b';
		assertFails: 'a -|';
		assertFails: 'a b > a';
		assertFails: 'a unary>'.! !

!S2JParserTest methodsFor: 'test tokens' stamp: 'ag 12/7/2010 10:08'!
testUnarySelector
	"Would be cool to find all the unary messages in the image and test them..."
	
	self
		currentRule: #unarySelector;
		
		assertMatches: 'abc' andProduces: #abc;
		assertMatches: 'a1' andProduces: #a1;
		assertMatches: '1' andProduces: #1;
		assertMatches: 'aaaaaaaa1233333' andProduces: #aaaaaaaa1233333;
		assertMatches: '' andProduces: #;
		
		assertFails: '';
		assertFails: ' ';
		assertFails: '|';
		assertFails: ':';
		assertFails: '#abc';
		assertFails: '1abc';
		assertFails: '123';
		assertFails: 'abc';
		assertFails: 'abc:';
		assertFails: 'abc<<'.! !

S2JAbstractTest subclass: #S2JSqueakAstToJsAstTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JSqueakAstToJsAstTest class methodsFor: 'testing' stamp: 'ag 12/9/2010 13:19'!
converterClass
	
	^ S2JSqueakAstToJsAst! !

!S2JSqueakAstToJsAstTest methodsFor: 'test stories' stamp: 'ag 1/18/2011 15:29'!
testBlockArgument
	
	self
		currentRule: #block;
		
		assertMatches: '[]' andProduces: (self node: 'functionCall' with: 
			{nil. 'block'. self node: 'arguments' with: 
				{self node: 'functionDeclaration' with: {self node: 'arguments'. self node: 'functionContent'}}});
		assertMatches: '[1]' andProduces: (self node: 'functionCall' with:  
			{nil. 'block'. self node: 'arguments' with: 
				{self node: 'functionDeclaration' with: 
					{ self node: 'arguments'. 
					self node: 'functionContent' with: 
						{ self node: 'returnExpression' with: 
							{ self node: 'functionCall' with: 
								{ nil. 'number'. 
								self node: 'arguments' with: 
								{ self node: 'number' with: { 1 }}}}}}}});
		assertMatches: '[^''1'']' andProduces: (self node: 'functionCall' with:  
			{nil. 'block'. self node: 'arguments' with: 
				{self node: 'functionDeclaration' with: 
					{ self node: 'arguments'. 
					self node: 'functionContent' with: 
						{ self node: 'expression' with: 
							{ self node: 'functionCall' with: 
							{ nil.
							'nonLocalReturn'.
							self node: 'arguments' with: 
								{ self node: 'functionCall' with: 
								{ nil.
								'string'. self node: 'arguments' with: 
								{self node: 'string' with: { '1' }}}}}}}}}});
		assertMatches: '[:a]' andProduces: 
		(self node: 'functionCall' with: 
			{nil. 'block'. self node: 'arguments' with: 
				{self node: 'functionDeclaration' with: {self node: 'arguments'  with: { self node: 'identifier' with: { 'a' }}. self node: 'functionContent'}}}).! !

!S2JSqueakAstToJsAstTest methodsFor: 'test general' stamp: 'ag 12/13/2010 09:15'!
testExampleMethods
	
	super testExampleMethods! !

!S2JSqueakAstToJsAstTest methodsFor: 'test stories' stamp: 'ag 1/18/2011 15:29'!
testNonLocalReturn
	
	self
		currentRule: #block;
		
		assertMatches: 
			'[ [^ ''returned nonlocally''] value ]'
		 andProduces: (self node: 'functionCall' with: 
			{nil.
			'block'.
			self node: 'arguments' with: 
				{self node: 'functionDeclaration' with: 
					{ self node: 'arguments' with: {}.
					self node: 'functionContent' with: 
						{self node: 'returnExpression' with: 
						{self node: 'functionCall' with: 
							{ self node: 'functionCall' with: 
								{nil. 
								'block'.
								self node: 'arguments' with: 
									{self node: 'functionDeclaration' with: 
										{self node: 'arguments' with: {}.
										self node: 'functionContent' with:
											{ self node: 'expression' with: 
												{ self node: 'functionCall' with: 
													{nil. 'nonLocalReturn'. self node: 'arguments' with: 
														{ self node: 'functionCall' with: { nil. 'string'. self 
															node: 'arguments' with: {self node: 'string' with: {'returned nonlocally'}}}}}}}}. 
														}}.
							'value'.
			 				self node: 'arguments' with: {}}}}}}})
		
		
	"expected JavaScript: block(function(){return block(function(){nonLocalReturn('returned nonlocally')}).value() }, this)"! !

!S2JSqueakAstToJsAstTest methodsFor: 'as yet unclassified' stamp: 'lw 1/12/2011 21:16'!
testSlotAssignment

	self
		currentRule: #expression;
		
		assertMatches: 'a js slotti js: b' 
		andProduces: (self node: 'expression' with: 
			{self node: 'slotAssignment' with: 
				{self node: 'slotAccess' with: 
					{self node: 'identifier' with: {'a'}. #slotti}. 
				self node: 'functionCall' with: 
					{self node: 'identifier' with: {'b'}. 
					'js'. 
					self node: 'arguments' with: {}}}})
	 ! !

S2JTestCase subclass: #S2JCodeCleanTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JCodeCleanTest methodsFor: 'tests' stamp: 'ag 2/3/2011 12:05'!
testNoDirtyCategories
	
	self assert: (Utilities classesWithDirtyCategories isEmpty)! !

!S2JCodeCleanTest methodsFor: 'tests' stamp: 'ag 2/3/2011 12:04'!
testNoEmptyCategories
	
	self assert: (Utilities classesWithEmptyCategories isEmpty)! !

S2JTestCase subclass: #S2JCodeFactoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JCodeFactoryTest methodsFor: 'test methods' stamp: 'ag 12/16/2010 11:21'!
aTestMethod
	
	"Empty method for tests"! !

!S2JCodeFactoryTest methodsFor: 'test methods' stamp: 'ag 12/16/2010 11:23'!
aTestMethod2
	
	"Empty method for tests"! !

!S2JCodeFactoryTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 11:15'!
codeFactory
	"Skip the singleton-pattern"
	
	^ S2JCodeFactory new! !

!S2JCodeFactoryTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 11:21'!
getTestMethod
	
	^ S2JOMeta2 method: #aTestMethod of: self class! !

!S2JCodeFactoryTest methodsFor: 'helpers' stamp: 'ag 12/16/2010 11:23'!
getTestMethod2
	
	^ S2JOMeta2 method: #aTestMethod2 of: self class! !

!S2JCodeFactoryTest methodsFor: 'helpers' stamp: 'ag 1/5/2011 22:57'!
methodNumIn: aClass
	
	^ aClass methodDict size + aClass class methodDict size! !

!S2JCodeFactoryTest methodsFor: 'test cases' stamp: 'ag 1/6/2011 11:06'!
testCaching
	
	| factory |
	factory := self codeFactory.
	self assert: factory codeCache size == 0.
	factory codeForMethod: #aTestMethod of: self class.
	self assert: factory codeCache size == 1.
	factory codeForMethod: #aTestMethod of: self class.
	self assert: factory codeCache size == 1.
	factory codeForMethod: #aTestMethod2 of: self class.
	self assert: factory codeCache size == 2.
	factory codeForClass: self class.
	factory codeFor: self class. "If already cached, this works, too"
	self assert: factory codeCache size == ((self methodNumIn: self class) + 1).
	
	factory := self codeFactory.
	factory codeFor: '1 a'.
	self assert: factory codeCache size == 1.
	factory codeFor: '1 a'.
	factory codeFor: '1 a'.
	self assert: factory codeCache size == 1.
	factory codeFor: '1 a; b'.
	factory codeFor: '1 a; b'.
	self assert: factory codeCache size == 2.! !

!S2JCodeFactoryTest methodsFor: 'test cases' stamp: 'ag 1/5/2011 22:58'!
testCachingClasses
	
	| factory |
	factory := self codeFactory.
	factory codeForClass: self class.
	self assert: factory codeCache size = (self class methodDict size + 1).
	factory codeForClasses: { self class. S2JNode }.
	self assert: 
		factory codeCache size = 
		((self methodNumIn: self class) + (self methodNumIn: S2JNode) + 3). "Methods + 2 classes + 1 class-array"! !

!S2JCodeFactoryTest methodsFor: 'test cases' stamp: 'ag 12/16/2010 21:20'!
testCachingWithRecompile
	
	| factory size |
	factory := self codeFactory.
	factory codeForClass: self class.
	size := factory codeCache size.
	self class compileAll.
	factory codeForClass: self class.
	self assert: size == factory codeCache size.! !

!S2JCodeFactoryTest methodsFor: 'test cases' stamp: 'ag 12/16/2010 21:14'!
testEqualsBlock
	
	| factory block antiBlock method1 method2 |
	factory := self codeFactory.
	block := [ :o1 :o2 | self assert: (factory compareCacheKey: o1 to: o2)].
	antiBlock := [ :o1 :o2 | self deny: (factory compareCacheKey: o1 to: o2)].
	
	block
		value: 'abc' value: 'abc';
		value: self getTestMethod value: self getTestMethod;
		value: self class value: self class.
	antiBlock
		value: 'abc1' value: 'abc2';
		value: self class value: Object;
		value: 1 value: 2;
		value: 1 value: '1';
		value: self getTestMethod value: (S2JOMeta2 method: #asString of: Object);
		value: self getTestMethod value: self getTestMethod2.
	
	method1 := self getTestMethod.
	self class compileAll.
	method2 := self getTestMethod.
	self deny: method1 == method2. "Should be new objects after recompiling!!"
	block value: method1 value: method2.! !

S2JTestCase subclass: #S2JJavascriptTest
	instanceVariableNames: 'semaphore clientTestResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

S2JJavascriptTest class
	instanceVariableNames: 'currentTestCase'!

!S2JJavascriptTest commentStamp: 'ag 1/26/2011 15:16' prior: 0!
This Test starts the browser with the url of the S2JTestApp.
Then it waits for the test-results from the browser.!

S2JJavascriptTest class
	instanceVariableNames: 'currentTestCase'!

!S2JJavascriptTest class methodsFor: 'accessing' stamp: 'ag 1/31/2011 12:31'!
currentTestCase
	^ currentTestCase! !

!S2JJavascriptTest class methodsFor: 'accessing' stamp: 'ag 1/31/2011 12:31'!
currentTestCase: anObject
	currentTestCase := anObject! !

!S2JJavascriptTest class methodsFor: 'waiting' stamp: 'srv 1/31/2011 22:35'!
reportJSResults: aString
	"If we currently are executing a test-case, it gets notified."
	
	self currentTestCase notNil & self isOnCIServer ifTrue: [ self currentTestCase clientResponded: aString ].
	^ ''! !

!S2JJavascriptTest class methodsFor: 'accessing' stamp: 'ag 1/31/2011 12:30'!
retryTime
	" in seconds "
	
	^ 120! !

!S2JJavascriptTest methodsFor: 'helpers' stamp: 'ag 2/3/2011 13:00'!
clientResponded: aString
	
	self clientTestResult: aString.
	self semaphore signal.! !

!S2JJavascriptTest methodsFor: 'accessing' stamp: 'ag 2/3/2011 13:00'!
clientTestResult
	^ clientTestResult! !

!S2JJavascriptTest methodsFor: 'accessing' stamp: 'ag 2/3/2011 13:00'!
clientTestResult: anObject
	clientTestResult := anObject! !

!S2JJavascriptTest methodsFor: 'helpers' stamp: 'srv 1/27/2011 16:48'!
killFireFox
	
	CommandShell command: 'pkill firefox;exit'! !

!S2JJavascriptTest methodsFor: 'testing' stamp: 'ag 2/3/2011 12:57'!
performTest: url tryAgain: tryAgain
	
	^ self performTest: url tryAgain: tryAgain doing: []! !

!S2JJavascriptTest methodsFor: 'testing' stamp: 'ag 2/3/2011 16:00'!
performTest: url tryAgain: tryAgain doing: aBlock
	
	self runFireFox: url.
	(Delay forSeconds: 3) wait.
	aBlock value.
	(self semaphore waitTimeoutSeconds: self class retryTime) 
		ifTrue: [
			self killFireFox.
			tryAgain ifTrue: [ self performTest: url tryAgain: false doing: aBlock] ] "have a second try, because sometimes the bodys onload is not evaluated"
		ifFalse: [ ^ self clientTestResult ].
	self error: 'No answer from the Javascript-side'.! !

!S2JJavascriptTest methodsFor: 'helpers' stamp: 'ag 2/3/2011 12:48'!
runFireFox: url

	CommandShell command: 'firefox -P JSTests ', url, ';exit'! !

!S2JJavascriptTest methodsFor: 'accessing' stamp: 'lw 1/27/2011 05:07'!
semaphore
	^ semaphore! !

!S2JJavascriptTest methodsFor: 'accessing' stamp: 'lw 1/27/2011 05:07'!
semaphore: anObject
	semaphore := anObject! !

!S2JJavascriptTest methodsFor: 'running' stamp: 'ag 2/3/2011 13:09'!
setUp
	
	self class isOnCIServer ifTrue: [
		S2JDispatcher forcedRestart.
		self clientTestResult: nil.
		self semaphore: Semaphore new.
		self class currentTestCase: self ].! !

!S2JJavascriptTest methodsFor: 'running' stamp: 'ag 1/31/2011 12:32'!
tearDown
	
	self class isOnCIServer ifTrue: [
		self killFireFox.
		self class currentTestCase: nil ].! !

!S2JJavascriptTest methodsFor: 'testing' stamp: 'ag 2/3/2011 13:23'!
testClientside
	
	self class isOnCIServer ifTrue: [
		self assert: 
			0 =
			(self performTest: 'http://localhost:8887/test' tryAgain: true ) asInteger ].! !

!S2JJavascriptTest methodsFor: 'testing' stamp: 'srv 2/15/2011 11:51'!
testOnServerBlock
	
	"TODO test commented out, because communication required bootstrap.js and classes, which are not loaded yet"
	
	false & (self class isOnCIServer) ifTrue: [
		self assert: 
			'abc123' =
			(self
				performTest: 'http://localhost:8887/emptyTest'
				tryAgain: true
				doing: [ S2JEmptyTestApp sendCode: '[ S2JJavascriptTest js reportJSResults: S2JmockTestResult ] onServer value' ] ) ].! !

!S2JJavascriptTest methodsFor: 'testing' stamp: 'srv 2/15/2011 11:50'!
testSendCode
	
	"TODO test commented out, because communication required bootstrap.js and classes, which are not loaded yet"
	
	false & (self class isOnCIServer) ifTrue: [
		self assert: 
			'abc123' =
			(self
				performTest: 'http://localhost:8887/emptyTest'
				tryAgain: true
				doing: [ S2JEmptyTestApp sendCode: 'S2JServer js performOnServer: '' S2JJavascriptTest js reportJSResults: S2JmockTestResult ''' ] ) ].! !

S2JTestCase subclass: #S2JSwazooTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

S2JSwazooTests subclass: #S2JCometHandlerTests
	instanceVariableNames: 'client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JCometHandlerTests methodsFor: 'accessing' stamp: 'HK 1/5/2011 16:57'!
client
	^ client! !

!S2JCometHandlerTests methodsFor: 'accessing' stamp: 'HK 1/5/2011 16:57'!
client: anObject
	client := anObject! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 11:16'!
killClient

	self client timeStampOfLatestRequest: TimeStamp now - S2JCometClient defaultMaximumAge! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 12:04'!
setUp

	self client: S2JCometClient new.
	self client application: (S2JTestApp instance reset );
			    identifier: (S2JCometHandler newFor: self client application ) generateIdentifier.
	self client application addClient: self client.
		
! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 11:38'!
testClientMatchingAtCometHandler

	client := (S2JCometHandler newFor: (S2JTestApp instance)) findClientById: self client identifier.
	self assert: ( client = self client).
	! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/26/2011 09:22'!
testCreateNewId
	| oldCount newCount response path |
	
	oldCount := self client application clients size.
	path := S2JTestApp uniquePath, '/', S2JCometHandler pathSuffix.	
	response := S2JDispatcher new answerTo: (self basicGetFor: path).
	self assert: response notNil.
	self assert: (response statusCode = 202).
	newCount := self client application clients size.
	self assert: newCount = (oldCount + 1)! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 11:23'!
testDeathAfterMaximumAge 

	self killClient.
	self deny: self client isAlive ! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/26/2011 09:23'!
testDispatchApplicationAndCreateCometClient
	
	| clientId response path |

	path := S2JTestApp uniquePath,'/', S2JCometHandler pathSuffix.
	response := S2JDispatcher new answerTo: (self basicGetFor: path).
	clientId := response body asString.
	
	self client: ((S2JCometHandler newFor: S2JTestApp instance ) findClientById: clientId).
	self assert: (self client notNil).
	self assert: (self client isAlive).
! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 14:50'!
testForwardingCometHandler

	| request path response |
	
	path := S2JTestApp uniquePath, S2JCometHandler pathSuffix, '?id=', self client identifier.
	request :=  self basicGetFor: path. 
	
	self client application repository addContent: 'test'.
	response := (S2JCometHandler newFor: (S2JTestApp instance)) answerTo: request. 

	self assert: (response body asString = 'test').
! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/19/2011 10:17'!
testIsInitiallyAlive


	self assert: self client isAlive! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 14:50'!
testQueriesKeepAlive
	| path response |
	
	self client timeStampOfLatestRequest: (TimeStamp now - (Duration seconds: 2)).
	path := S2JTestApp uniquePath, S2JCometHandler pathSuffix , '?id=', self client identifier.

	[ response := S2JDispatcher new answerTo: (self basicGetFor: path) ] fork.
	(Delay forSeconds: 2) wait.
	self assert: self client isAlive.
	self killClient.
	! !

!S2JCometHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/26/2011 09:23'!
testRemoveDeadClient

	| clientId response path |
	
	path := S2JTestApp uniquePath, '/', S2JCometHandler pathSuffix , '?id=', self client identifier.
	[ (Delay forSeconds: 1) wait.
		 self killClient.
	] fork.
	response := S2JDispatcher new answerTo: (self basicGetFor: path).
	self assert: (response statusCode = S2JCometClient HTTPStatusCodeTimeOut).
	self assert: ((S2JCometHandler newFor: (S2JTestApp instance)) findClientById: clientId) isNil.! !

S2JSwazooTests subclass: #S2JDispatcherTests
	instanceVariableNames: 'client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JDispatcherTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 12:53'!
expectedFailures
	^{ #testDispatchVirginWebSocketRequest}! !

!S2JDispatcherTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 14:50'!
testDispatchVirginCometRequest

	| response path httpRequest |
	
	path := S2JTestApp uniquePath, S2JCometHandler pathSuffix.
	httpRequest :=	(self basicGetFor: path).
	response := S2JDispatcher new answerTo: httpRequest.
	
	self assert: (response body asString isAllDigits ).! !

!S2JDispatcherTests methodsFor: 'as yet unclassified' stamp: 'SW 1/25/2011 12:46'!
testDispatchVirginWebSocketRequest

	| response path httpRequest |
	
	path := S2JTestApp uniquePath, S2JWebSocketHandler defaultPath.
	httpRequest :=	(self basicWSFor: path).
	response := S2JDispatcher new answerTo: httpRequest.
	
	self assert: (response body asString isAllDigits ).! !

!S2JSwazooTests methodsFor: 'as yet unclassified' stamp: 'SW 1/31/2011 08:52'!
arrayFromHexString: aString
	| array stream lookup |
	lookup := '123456789ABCDEF'.
	array := ByteArray new: aString size // 2.
	stream := aString readStream.
	1 to: array size do: [ :i || h l |
		h := stream next.
		h := h = $0 ifTrue: [0] ifFalse: [lookup indexOf: h].
		l := stream next.
		l := l = $0 ifTrue: [0] ifFalse: [lookup indexOf: l].
		array at: i put: ((h bitShift: 4) bitOr: l) ].
	^array! !

!S2JSwazooTests methodsFor: 'as yet unclassified' stamp: 'SW 1/19/2011 10:34'!
basicGetFor: aPath

	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET ', aPath, ' HTTP/1.1';
		nextPutLine: 'Host: localhost';
		nextPutLine: 'Connection: Keep-Alive';
		crlf.
	^ HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).! !

!S2JSwazooTests methodsFor: 'as yet unclassified' stamp: 'SW 1/31/2011 09:08'!
basicWSFor: aPath

	| requestStream request|
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET ', aPath, ' HTTP/1.1';
		nextPutLine: 'Host: example.com';
		nextPutLine: 'Connection: Upgrade';
		nextPutLine: 'Sec-WebSocket-Key2: 8 1 8 6    7 ]   8476';
		nextPutLine: 'Sec-WebSocket-Protocol: sample';
		nextPutLine: 'Upgrade: WebSocket';
		nextPutLine: 'Sec-WebSocket-Key1: 2  __4 ''79  #  9"; +K 67  -655';
		nextPutLine: 'Origin: http://example.com';
		nextPutLine: 'Content-Length: 8';
		crlf;
		nextPutLine: (self arrayFromHexString: '598F6AEF77E67CBE') asString.
		
	request := HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).
	request task: (SwazooTask newOn: request).
	request task connection: HTTPConnection new.
	^ request! !

!S2JSwazooTests methodsFor: 'as yet unclassified' stamp: 'HK 1/12/2011 17:17'!
upgradeGetFor: aPath

	| requestStream |
	requestStream := SwazooStream on: String new.
	requestStream
		nextPutLine: 'GET ', aPath, ' HTTP/1.1';
		nextPutLine: 'Host: localhost';
		nextPutLine: 'Upgrade: WebSocket';
		crlf.
	^ HTTPRequest readFrom: (SwazooStream on: requestStream writeBuffer contents).! !

S2JSwazooTests subclass: #S2JWebSocketHandlerTests
	instanceVariableNames: 'client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Tests'!

!S2JWebSocketHandlerTests methodsFor: 'as yet unclassified' stamp: 'SW 1/31/2011 11:52'!
testOpenConnection

	| oldCount response request |

	oldCount := S2JTestApp instance clients size.

	request := self basicWSFor: S2JTestApp uniquePath, '/ws'..
	response := (S2JWebSocketHandler newFor: (S2JTestApp instance)) answerTo: request.	
	self assert: response isWebSocketHandshake.
	
	self assert: (S2JTestApp instance clients size = (oldCount +1)).! !

!S2JTestCase class methodsFor: 'testing' stamp: 'srv 1/5/2011 11:08'!
isOnCIServer
	
	^ true == (Smalltalk at: #S2JisOnCIServer ifAbsent: [ nil ])! !

!S2JTestCase class methodsFor: 'testing' stamp: 'ag 2/3/2011 13:08'!
setIsOnCIServer
	
	^ self setIsOnCIServer: true! !

!S2JTestCase class methodsFor: 'testing' stamp: 'ag 2/3/2011 13:08'!
setIsOnCIServer: aBoolean
	
	Smalltalk at: #S2JisOnCIServer put: aBoolean.! !

!S2JTestCase methodsFor: 'private' stamp: 'srv 1/5/2011 11:09'!
timeoutForTest
	"remove test-timeout on the CI-server"
	
	^ self class isOnCIServer
		ifTrue: [ 15 * 60 * 1000 ]
		ifFalse: [ ^ super timeoutForTest ]! !

!BlockClosure methodsFor: '*Squeak2JS' stamp: 'ag 1/19/2011 18:34'!
onServer
	"This is overridden when executing on the client. On the server, there is no change."
	
	^ self! !

!Collection methodsFor: '*Squeak2JS' stamp: 'ag 1/25/2011 16:03'!
separatedBy: aString
	
	| stream |
	stream := TextStream on: ''.
	self asStringOn: stream delimiter: aString.
	^ stream contents! !

TextStream subclass: #S2JTextStream
	instanceVariableNames: 'printer privateTextStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Translator'!

!S2JTextStream class methodsFor: 'instance creation' stamp: 'ag 1/9/2011 17:37'!
for: anS2JAbstractPrinter
	
	^ (self
		on: '')
		printer: anS2JAbstractPrinter;
		yourself! !

!S2JTextStream methodsFor: 'initialization' stamp: 'ag 1/9/2011 20:08'!
initialize
	
	super initialize.
	self privateTextStream: (TextStream on: '').! !

!S2JTextStream methodsFor: 'printing' stamp: 'ag 1/10/2011 10:14'!
printBlock: aConditionBlock prependedBy: prependBlock
	"aConditionBlock is evaluated with a 'mock'-stream. if true is returned, prependBlock is evaluated and aConditionBlock is evaluated with the actual self."
	
	(aConditionBlock value: self privateTextStream)
		ifTrue: [
			prependBlock value.
			aConditionBlock value: self].! !

!S2JTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 17:38'!
printer
	
	^ printer! !

!S2JTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 17:38'!
printer: anS2JAbstractPrinterClass
	
	printer := anS2JAbstractPrinterClass! !

!S2JTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream
	
	^ privateTextStream! !

!S2JTextStream methodsFor: 'accessing' stamp: 'ag 1/9/2011 20:08'!
privateTextStream: aStream
	
	privateTextStream := aStream! !

Object subclass: #JsDocument
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Stubs'!

Object subclass: #JsGlobal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Stubs'!

!JsGlobal commentStamp: 'lw 12/2/2010 18:43' prior: 0!
JsGlobal is a substitute for the JavaScript Global namespace. In order to call global JavaScript functions like alert, just send them to this Class.!

!Object methodsFor: '*Squeak2JS' stamp: 'LT 1/18/2011 20:24:23.541'!
serializeToJavaScript
	"example:
		{1. 2. 3} asSet serializeToJavaScript "
	^ S2JCodeFactory codeFor: self serializeToSmalltalk.! !

!Object methodsFor: '*Squeak2JS' stamp: 'LT 1/18/2011 20:23:31.669'!
serializeToSmalltalk

	"SmartRefStream might allow even object identity"
	^ self storeString.! !

Object subclass: #S2JAbstractCodeFactory
	instanceVariableNames: 'codeCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

S2JAbstractCodeFactory class
	instanceVariableNames: 'instance'!

!S2JAbstractCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

S2JAbstractCodeFactory class
	instanceVariableNames: 'instance'!

!S2JAbstractCodeFactory class methodsFor: 'converting' stamp: 'ag 1/5/2011 22:12'!
converterClass
	
	self subclassResponsibility! !

!S2JAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/16/2010 11:44'!
destroyInstance
	
	instance := nil.! !

!S2JAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/13/2010 15:43'!
doesNotUnderstand: aMessage
	"removes need to call instance every time the code-factory is accessed"
	
	^ self instance perform: aMessage selector withArguments: aMessage arguments! !

!S2JAbstractCodeFactory class methodsFor: 'singleton' stamp: 'ag 12/16/2010 11:09'!
instance
	
	instance ifNil: [ instance := self new ].
	^ instance! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/7/2011 23:29'!
cacheCodeFor: anObject producedBy: aBlock
	
	^ (self cacheObjectFor: anObject producedBy: aBlock) javascriptObject! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/7/2011 23:29'!
cacheObjectFor: anObject producedBy: aBlock
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| result |
	result := self codeCache at: anObject ifAbsent: [ nil ].
	result
		ifNil: [
			result := aBlock value.
			self codeCache at: anObject put: result.
			result update]
		ifNotNil: [
			result hasChanged ifTrue: [ result update ]].
	^ result! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/7/2011 23:28'!
cacheObjectForClass: aClassOrClassname
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	| cl |
	cl := self class converterClass classNamed: aClassOrClassname.
	^ self
		cacheObjectFor: cl
		producedBy: [ S2JCompiledClass from: cl inFactory: self ]! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/7/2011 23:28'!
cacheObjectForMethod: aCompiledMethod
	"cacheObject* returns the value in the dictionary, not the javascript-code (this is done by cacheCodeFor*)"
	
	^ self
		cacheObjectFor: aCompiledMethod
		producedBy: [
			S2JCompiledMethod from: aCompiledMethod inFactory: self ]! !

!S2JAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 12/15/2010 10:42'!
clearCache
	
	self codeCache removeAll.! !

!S2JAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache
	^ codeCache! !

!S2JAbstractCodeFactory methodsFor: 'accessing' stamp: 'ag 12/13/2010 14:30'!
codeCache: anObject
	codeCache := anObject! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/5/2011 22:19'!
codeFor: aString
	"A simple DoIt-code-snippet"
	
	^ self
		cacheCodeFor: aString
		producedBy: [ S2JCompiledDoIt from: aString inFactory: self ]! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/7/2011 23:27'!
codeForClass: aClassOrClassName
	"A JS-Code-String representing the whole class. Without a DoIt code-snippet etc."
	
	^ (self cacheObjectForClass: aClassOrClassName) javascriptObject! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/5/2011 20:07'!
codeForClasses: anArrayOfClasses
	"An JS-code-string representing all classes in anArrayOfClasses"
	
	^ self
		cacheCodeFor: anArrayOfClasses
		producedBy: [ S2JCompiledClasses from: anArrayOfClasses inFactory: self ]! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 12/14/2010 00:37'!
codeForCompiledMethod: aCompiledMethod
	"An association from the function-name to the function-declaration-code"
	
	^ (self cacheObjectForMethod: aCompiledMethod) javascriptObject! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/5/2011 20:13'!
codeForMethod: aMethodSelector of: aClassOrClassName
	
	^ self codeForCompiledMethod:
		(self class converterClass method: aMethodSelector of: aClassOrClassName)! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 12/16/2010 11:47'!
compareCacheKey: o1 to: o2
	"CompiledMethods are considered equal, if they represent the same selector in the same class.
	Even after recompilation of a method, the old instance of CompiledMethod will be kept in the dictionary."
	
	o1 class = o2 class ifFalse: [ ^ false ].
	^ o1 class = CompiledMethod
		ifTrue: [ o1 methodClass = o2 methodClass and: [ o1 selector = o2 selector ] ]
		ifFalse: [ o1 = o2 ]! !

!S2JAbstractCodeFactory methodsFor: 'private' stamp: 'ag 1/3/2011 22:19'!
hashCodeFor: anObject
	
	^ anObject class = CompiledMethod
		ifTrue: [ 7 * anObject methodClass hash + anObject selector hash]
		ifFalse: [ anObject hash ]! !

!S2JAbstractCodeFactory methodsFor: 'initialization' stamp: 'ag 1/3/2011 22:06'!
initialize
	
	super initialize.
	self codeCache: PluggableDictionary new.
	self codeCache
		equalBlock: [ :o1 :o2 | self compareCacheKey: o1 to: o2 ];
		hashBlock: [ :object | self hashCodeFor: object ].! !

!S2JAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 14:04'!
invalidateMethod: aSelector of: anObject
	
	^ self invalidateObject: (self class converterClass method: aSelector of: anObject)! !

!S2JAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 13:51'!
invalidateObject: anObject
	
	^ self codeCache removeKey: anObject! !

!S2JAbstractCodeFactory methodsFor: 'cache' stamp: 'ag 2/1/2011 13:51'!
isObjectCached: anObject
	
	^ self codeCache includesKey: anObject! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/9/2011 13:58'!
printCodeFor: aCachedObject to: aFilename
	"Print the code for an object, that is already in the cache, to a new (or overridden) file aFilename.
	Return nil, if object already in cache, return the printed code if successfull."
	
	^ self class converterClass
		print: (self cacheCodeFor: aCachedObject producedBy: [ ^ nil ])
		to: aFilename! !

!S2JAbstractCodeFactory methodsFor: 'javascript code' stamp: 'ag 1/6/2011 21:03'!
readAllCodeInImage
	"Read each method in the image. This will not throw an error, just skip methods, that cannot be read."
	
	self class converterClass allMethodsDoAndReport: [ :eachMethod |
		self codeForCompiledMethod: eachMethod ].! !

S2JAbstractCodeFactory subclass: #S2JCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JCodeFactory commentStamp: 'ag 12/16/2010 12:46' prior: 0!
S2JCodeFactory readAllCodeInImage.

S2JCodeFactory clearCache.

S2JCodeFactory codeCache size. 84964. 64386. " 62492 !!!! ".

S2JCodeFactory codeFor: 'a doit-snippet squeak code'.

S2JCodeFactory  codeForClass: Object.

S2JCodeFactory codeForMethod: #asString of: Object.
!

!S2JCodeFactory class methodsFor: 'converting' stamp: 'ag 1/5/2011 22:12'!
converterClass
	
	^ S2JPrinter! !

S2JAbstractCodeFactory subclass: #S2JPrettyCodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JPrettyCodeFactory class methodsFor: 'converting' stamp: 'ag 1/6/2011 20:49'!
converterClass
	
	^ S2JPrettyPrinter! !

Object subclass: #S2JClient
	instanceVariableNames: 'revision application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JClient class methodsFor: 'configuration' stamp: 'SW 1/25/2011 12:00'!
defaultRevision

	^ 1! !

!S2JClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 12:25'!
adaptRevision

	self revision: self application repository revision.! !

!S2JClient methodsFor: 'accessing' stamp: 'LT 1/17/2011 12:16:26.766'!
application
	^ application! !

!S2JClient methodsFor: 'accessing' stamp: 'LT 1/17/2011 12:16:26.778'!
application: anObject
	application := anObject! !

!S2JClient methodsFor: 'client handling' stamp: 'SW 1/17/2011 15:35'!
identifier

	self subclassResponsibility ! !

!S2JClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 12:25'!
increaseRevision

	self revision: (self revision + 1).
! !

!S2JClient methodsFor: 'initialization' stamp: 'HK 1/4/2011 14:20'!
initialize

	self revision: self class defaultRevision.! !

!S2JClient methodsFor: 'initialization' stamp: 'SW 1/17/2011 12:42'!
isAMethodInvokation: aMessage
	 	
	^ aMessage beginsWith: 'INVOKE: '! !

!S2JClient methodsFor: 'initialization' stamp: 'HK 1/5/2011 16:52'!
isAlive 

	self subclassResponsibility.! !

!S2JClient methodsFor: 'convenience' stamp: 'SW 1/18/2011 15:35'!
isBehind

	^ self revision < self application repository revision ! !

!S2JClient methodsFor: 'convenience' stamp: 'SW 1/18/2011 15:36'!
isBehindFor: aRepository

	^ self revision < aRepository revision.! !

!S2JClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:17'!
isCometClient

	^ self subclassResponsibility.! !

!S2JClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:17'!
isWebSocketClient

	^ self subclassResponsibility.! !

!S2JClient methodsFor: 'initialization' stamp: 'SW 1/18/2011 14:56'!
receive: aMessage
	"Send aMessage from server to client. The client should receive that message."
	
	 ^ self subclassResponsibility ! !

!S2JClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 22:35'!
revision
	^ revision! !

!S2JClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 22:35'!
revision: anObject
	revision := anObject! !

!S2JClient methodsFor: 'initialization' stamp: 'SW 1/18/2011 11:23'!
sent: aMessage

	"Send aMessage from client to server. The server should receive that message."
	
	 ^ self subclassResponsibility ! !

!S2JClient methodsFor: 'client handling' stamp: 'LT 1/17/2011 12:13:49.516'!
update

	self subclassResponsibility! !

S2JClient subclass: #S2JCometClient
	instanceVariableNames: 'identifier timeStampOfLatestRequest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JCometClient class methodsFor: 'configuration' stamp: 'SW 1/25/2011 12:16'!
HTTPStatusCodeTimeOut
	
	^ 408! !

!S2JCometClient class methodsFor: 'configuration' stamp: 'lw 1/25/2011 17:24'!
defaultMaximumAge

	^ Duration minutes: 10 ! !

!S2JCometClient class methodsFor: 'configuration' stamp: 'SW 1/18/2011 15:40'!
defaultMillisecondsToWait

	^ 500! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 16:47'!
checkLivelinessDelayed

	[ (Delay forDuration: self class defaultMaximumAge) wait. 
	self isAlive ifFalse: [
		self application removeClient: self ]] fork.! !

!S2JCometClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 14:34'!
identifier
	^ identifier! !

!S2JCometClient methodsFor: 'accessing' stamp: 'HK 12/15/2010 14:34'!
identifier: anObject
	identifier := anObject! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/25/2011 11:14'!
initialize 
	
	super initialize.
	self timeStampOfLatestRequest: TimeStamp now! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 16:11'!
isAlive

	^ (TimeStamp now - self timeStampOfLatestRequest < self class defaultMaximumAge )! !

!S2JCometClient methodsFor: 'convenience' stamp: 'SW 1/25/2011 11:46'!
isCometClient

	^ true! !

!S2JCometClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:18'!
isWebSocketClient

	^ false.! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:57'!
receive: aMessage


! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 16:03'!
sent: aMessage

	self timeStampOfLatestRequest: TimeStamp now.
	self application processClientData: aMessage! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest
	^ timeStampOfLatestRequest! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 15:59'!
timeStampOfLatestRequest: anObject
	timeStampOfLatestRequest := anObject! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/25/2011 12:16'!
update

	self timeStampOfLatestRequest: TimeStamp now.
	[ self isBehind ] whileFalse:
		[ self isAlive ifFalse:
			[ self application removeClient: self.
			^ self application answerWithStatusCode: self class HTTPStatusCodeTimeOut ]. 
		self wait ].
	self checkLivelinessDelayed.	
	^ self application answerNewContentTo: self! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 14:57'!
wait

	self waitForMilliseconds: self class defaultMillisecondsToWait.! !

!S2JCometClient methodsFor: 'accessing' stamp: 'SW 1/18/2011 14:57'!
waitForMilliseconds: milliseconds

	(Delay forMilliseconds: milliseconds) wait.! !

S2JClient subclass: #S2JWebSocketClient
	instanceVariableNames: 'socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JWebSocketClient class methodsFor: 'instance creation' stamp: 'HK 12/15/2010 21:46'!
newWithSocket: aWebSocket

	^ self basicNew socket: aWebSocket; initialize.! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 15:34'!
close

	self isAlive ifTrue: [ self socket close ].
	self application  removeClient: self.! !

!S2JWebSocketClient methodsFor: 'configuration' stamp: 'SW 1/17/2011 12:36'!
defaultReceiveBlock

	
	^ [ :message |
		self receivePost: message.
		"S2JDispatcher updateContentWith: message.
		S2JWebSocketHandler updateAllClientsExcept: self "		
		].! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/17/2011 15:35'!
identifier
	
	 ^ 'WebSocket'! !

!S2JWebSocketClient methodsFor: 'initialization' stamp: 'SW 1/18/2011 11:23'!
initialize

	super initialize.
	self socket receiveBlock: [:aMessage | self sent: aMessage ]! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'HK 1/5/2011 16:53'!
isAlive
	
	^ self socket isOpen and: [ self socket connection stream notNil ].! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:18'!
isCometClient

	^ false.! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'HK 12/15/2010 16:18'!
isWebSocketClient

	^ true.! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/31/2011 10:12'!
receive: aMessage

	[ self socket send: aMessage asString ] on: Error do: [ self application removeClient: self ].! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/18/2011 15:01'!
sent: aMessage

	self application processClientData: aMessage! !

!S2JWebSocketClient methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket
	^ socket! !

!S2JWebSocketClient methodsFor: 'accessing' stamp: 'HK 12/14/2010 10:20'!
socket: anObject
	socket := anObject! !

!S2JWebSocketClient methodsFor: 'convenience' stamp: 'SW 1/18/2011 11:03'!
update

	 [ self isBehind ] whileTrue: [self increaseRevision. 
		self receive: (self application repository contentAtRevision: self revision) ]
		! !

Object subclass: #S2JCompiledObject
	instanceVariableNames: 'squeakObject javascriptObject codeFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

S2JCompiledObject subclass: #S2JCompiledClass
	instanceVariableNames: 'instMethods classMethods instVars superclass classInstVars classVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JCompiledClass methodsFor: 'converting' stamp: 'ag 1/5/2011 22:01'!
allMethodsOf: aSourceClass
	
	^ aSourceClass methodDict values collect: [ :method | 
		self codeFactory cacheObjectForMethod: method ]! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars
	
	^ classInstVars! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classInstVars: anObject
	
	classInstVars := anObject! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods
	
	^ classMethods! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classMethods: anObject
	
	classMethods := anObject! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
classVars
	
	^ classVars! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:59'!
classVars: anObject
	
	classVars := anObject! !

!S2JCompiledClass methodsFor: 'converting' stamp: 'ag 1/10/2011 19:36'!
doUpdate
	"First memorize the fields, then do the actual conversion. Errors during the conversion will happen only once,
	the old version of the javascriptObject will be kept if that happens.
	TODO Bad: this is kind of the same code like in S2JAbstractPrinter class >> convertClass:to:retrievingFrom: and #hasChanged in this class"
	
	self
		superclass: self squeakObject superclass name;
		instMethods: (self allMethodsOf: self squeakObject);
		classMethods: (self allMethodsOf: self squeakObject class);
		instVars: self squeakObject instVarNames;
		classInstVars: self squeakObject class instVarNames;
		classVars: self squeakObject classVarNames;
		javascriptObject: (
			self converterClass
				convertClass: self squeakObject
				retrievingFrom: [ :method | "Here the CodeFactory is plugged to the Printer"
					self codeFactory codeForCompiledMethod: method ]).! !

!S2JCompiledClass methodsFor: 'converting' stamp: 'ag 1/10/2011 19:36'!
hasChanged
	
	^ 
	((self haveMethodsChanged: self instMethods from: self squeakObject)
	or: [ self haveMethodsChanged: self classMethods from: self squeakObject class ]
	or: [ self superclass ~= self squeakObject superclass name ]
	or: [ self instVars ~= self squeakObject instVarNames ]
	or: [ self classInstVars ~= self squeakObject class instVarNames ])
	or: [ self classVars ~= self squeakObject classVarNames ]! !

!S2JCompiledClass methodsFor: 'converting' stamp: 'ag 2/1/2011 14:05'!
haveMethodsChanged: originalMethods from: originalClass
	"Check #hasChanged on each S2JCompiledMethod and check, whether there are more methods in the class now."
	
	^ originalMethods isNil or: 
		[ (originalMethods anySatisfy: [ :method | 
			(self codeFactory isObjectCached: method squeakObject) not or: 
			[ method hasChanged ] ]) or:
		[ originalClass methodDict size ~= originalMethods size ]]! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods
	
	^ instMethods! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:57'!
instMethods: anObject
	
	instMethods := anObject! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars
	
	^ instVars! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
instVars: anObject
	
	instVars := anObject! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass
	
	^ superclass! !

!S2JCompiledClass methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:58'!
superclass: aString
	
	superclass := aString! !

!S2JCompiledClass methodsFor: 'converting' stamp: 'ag 1/6/2011 16:24'!
updateMessage
	
	^ 'class ', self squeakObject name! !

S2JCompiledObject subclass: #S2JCompiledClasses
	instanceVariableNames: 'compiledClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JCompiledClasses methodsFor: 'accessing' stamp: 'ag 1/7/2011 23:23'!
compiledClasses
	^ compiledClasses! !

!S2JCompiledClasses methodsFor: 'accessing' stamp: 'ag 1/7/2011 23:23'!
compiledClasses: aSet
	compiledClasses := aSet! !

!S2JCompiledClasses methodsFor: 'converting' stamp: 'ag 1/7/2011 23:40'!
doUpdate
	"Call convertClasses: of the converterClass and memorize the result. Plug the codeFactory to the converterClass (via the block).
	Actually, the compiledClasses should always stay the same objects, but just in case we refill the Set.
	Might happen, that one single object is removed from the cache."
	
	self compiledClasses removeAll.
	self javascriptObject: (
		self converterClass
			convertClasses: self squeakObject
			retrievingFrom: [ :class | "Here the CodeFactory is plugged to the Printer"
				self compiledClasses add: (self codeFactory cacheObjectForClass: class).
				self codeFactory codeForClass: class ]).! !

!S2JCompiledClasses methodsFor: 'converting' stamp: 'ag 2/1/2011 14:05'!
hasChanged
	
	^ self compiledClasses isNil or:
		[ self compiledClasses anySatisfy: [ :each | 
			(self codeFactory isObjectCached: each squeakObject) not
			or: [ each hasChanged ] ]]! !

!S2JCompiledClasses methodsFor: 'initialization' stamp: 'ag 1/6/2011 16:36'!
initialize
	
	super initialize.
	self compiledClasses: Set new.! !

!S2JCompiledClasses methodsFor: 'converting' stamp: 'ag 1/6/2011 16:25'!
updateMessage
	
	^ 'classes ', self squeakObject asString.! !

S2JCompiledObject subclass: #S2JCompiledDoIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JCompiledDoIt methodsFor: 'converting' stamp: 'ag 1/6/2011 21:00'!
doUpdate
	
	self javascriptObject: 
		(self converterClass convertDoIt: self squeakObject).! !

!S2JCompiledDoIt methodsFor: 'converting' stamp: 'ag 12/14/2010 00:33'!
hasChanged
	"DoIt-code does not change"
	
	^ false! !

S2JCompiledObject subclass: #S2JCompiledMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Cache'!

!S2JCompiledMethod methodsFor: 'converting' stamp: 'ag 1/19/2011 15:58'!
currentSqueakObject
	
	^ self converterClass
		method: self squeakObject selector
		of: self squeakObject methodClass
		ifAbsent: [ nil ]! !

!S2JCompiledMethod methodsFor: 'converting' stamp: 'ag 1/6/2011 20:59'!
doUpdate
	"Additionally to converting the current code, store the current instance of CompiledMethod"
	
	self
		squeakObject: self currentSqueakObject;
		javascriptObject: (self converterClass convertCompiledMethod: self squeakObject).! !

!S2JCompiledMethod methodsFor: 'converting' stamp: 'ag 12/17/2010 00:15'!
hasChanged
	"This is the main point of cache-validation. The instance-equality of the CompiledMethod is used.
	If the method-code changed, the method is recompiled. If the method is recompiled, there will be a new instance of CompiledMethod."
	
	^ (self squeakObject == self currentSqueakObject) not! !

!S2JCompiledMethod methodsFor: 'converting' stamp: 'ag 1/6/2011 16:27'!
updateMessage
	
	^ 'method ', self squeakObject methodClass name, ' >> ', self squeakObject selector! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:30'!
disableAllPrintUpdates
	
	S2JCompiledObject withAllSubclassesDo: [ :eachClass |
		eachClass enablePrintUpdates ].! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:31'!
disablePrintUpdates
	
	Smalltalk
		at: self updatesKey
		put: false.! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:30'!
enableAllPrintUpdates
	
	S2JCompiledObject withAllSubclassesDo: [ :eachClass |
		eachClass enablePrintUpdates ].! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:32'!
enablePrintUpdates
	
	Smalltalk
		at: self updatesKey
		put: true.! !

!S2JCompiledObject class methodsFor: 'instance creation' stamp: 'ag 1/5/2011 21:45'!
from: aSqueakObject inFactory: aCodeFactory
	
	^ self new
		squeakObject: aSqueakObject;
		codeFactory: aCodeFactory;
		yourself! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/6/2011 16:32'!
printUpdates
	
	^ (Smalltalk
		at: self updatesKey
		ifAbsent: [ nil ])
			== true! !

!S2JCompiledObject class methodsFor: 'printing updates' stamp: 'ag 1/10/2011 19:34'!
updatesKey
	"The key used in the global Smalltalk dictionary to store the per-class setting to print updates."
	
	^ ('S2JPrintCacheUpdates-', self name) asSymbol! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory
	
	^ codeFactory! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 1/5/2011 21:44'!
codeFactory: aCodeFactory
	
	codeFactory := aCodeFactory! !

!S2JCompiledObject methodsFor: 'converting' stamp: 'ag 1/6/2011 21:02'!
converterClass
	"Convenience for subclasses"
	
	^ self codeFactory class converterClass! !

!S2JCompiledObject methodsFor: 'converting' stamp: 'ag 1/6/2011 11:21'!
doUpdate
	"Update converted contents from the current state of the squeak-object."
	
	self subclassResponsibility! !

!S2JCompiledObject methodsFor: 'converting' stamp: 'ag 12/13/2010 22:41'!
hasChanged
	"Return true, if the source squeak-object has changed, since it has last been updated."
	
	self subclassResponsibility! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
javascriptObject
	
	^ javascriptObject! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
javascriptObject: anObject
	
	javascriptObject := anObject! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject
	
	^ squeakObject! !

!S2JCompiledObject methodsFor: 'accessing' stamp: 'ag 12/14/2010 00:34'!
squeakObject: anObject
	
	squeakObject := anObject! !

!S2JCompiledObject methodsFor: 'converting' stamp: 'ag 1/6/2011 16:24'!
update
	"To track the updates easily, they are printed to Transript, if activated"
	
	| message |
	message := self updateMessage.
	(message notNil and: [self class printUpdates]) ifTrue: [
		Transcript cr; show: 'Updating ', message].
	^ self doUpdate! !

!S2JCompiledObject methodsFor: 'converting' stamp: 'ag 1/6/2011 16:23'!
updateMessage
	"Return something else than nil to print it to Transcript on updates."
	
	^ nil! !

Object subclass: #S2JContentRepository
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Server'!

!S2JContentRepository methodsFor: 'management' stamp: 'ag 1/12/2011 19:54'!
addContent: javascriptCode
	
	^ self repository add: javascriptCode! !

!S2JContentRepository methodsFor: 'management' stamp: 'ag 1/12/2011 19:53'!
contentAtRevision: aRevision

	^ self repository
		at: aRevision
		ifAbsent: [ nil ]! !

!S2JContentRepository methodsFor: 'management' stamp: 'ag 1/12/2011 19:54'!
initialize
	
	self
		repository: OrderedCollection new;
		addContent: ''.! !

!S2JContentRepository methodsFor: 'management' stamp: 'SW 1/18/2011 15:36'!
latestContent
	
	^ self contentAtRevision: self revision.! !

!S2JContentRepository methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:44'!
repository
	^ repository! !

!S2JContentRepository methodsFor: 'accessing' stamp: 'HK 1/11/2011 16:44'!
repository: anObject
	repository := anObject! !

!S2JContentRepository methodsFor: 'management' stamp: 'SW 1/18/2011 15:36'!
revision
	
	^ self repository size! !

Object subclass: #S2JServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Stubs'!

Object subclass: #S2JShape
	instanceVariableNames: 'canvas color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JShape subclass: #S2JCircle
	instanceVariableNames: 'position radius'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JCircle subclass: #S2JBall
	instanceVariableNames: 'speed direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-BallGame'!

!S2JBall methodsFor: 'logic' stamp: 'ag 1/17/2011 20:34'!
bounceIfNeeded 
	|width height|
	width := JsGlobal js number: (JsDocument js getElementById: #ballCanvas) js width.
	height := JsGlobal js number: (JsDocument js getElementById: #ballCanvas) js height.
	 (self position x > width or: [self position x < 0]) ifTrue: [ self direction: (self direction x * -1) @ self direction y ].
	 (self position y > height or: [self position y < 0]) ifTrue: [ self direction: self direction x @ (self direction y * -1) ].! !

!S2JBall methodsFor: 'accessing' stamp: 'sec 1/5/2011 20:48'!
direction
	^ direction! !

!S2JBall methodsFor: 'accessing' stamp: 'sec 1/5/2011 20:48'!
direction: anObject
	direction := anObject! !

!S2JBall methodsFor: 'initialize-release' stamp: 'ag 1/24/2011 17:12'!
initialize
	
	super initialize.
	self speed: 2.
	self direction: 1@1.! !

!S2JBall methodsFor: 'accessing' stamp: 'sec 1/5/2011 20:48'!
speed
	^ speed! !

!S2JBall methodsFor: 'accessing' stamp: 'lw 2/2/2011 23:31'!
speed: anObject

	speed := anObject
! !

!S2JBall methodsFor: 'logic' stamp: 'ag 1/12/2011 15:41'!
step
	
	super step.
	self position: self position + (self direction * self speed).
	self bounceIfNeeded.! !

!S2JCircle methodsFor: 'graphics' stamp: 'LT 1/10/2011 18:51:39.505'!
drawOn: aCanvas
	
	"CTK library: http://canvastoolkit.codeplex.com/"
	aCanvas js setColor: self color.
	aCanvas js fillCircle: { self position x. self position y. self radius }.! !

!S2JCircle methodsFor: 'initialization' stamp: 'ag 1/24/2011 20:34'!
initialize
	
	super initialize.
	self
		color: '#ff0000';
		radius: 4;
		position: 1@1.! !

!S2JCircle methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:14:18.683'!
position
	^ position! !

!S2JCircle methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:14:18.695'!
position: anObject
	position := anObject! !

!S2JCircle methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:14:18.704'!
radius
	^ radius! !

!S2JCircle methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:14:18.712'!
radius: anObject
	radius := anObject! !

!S2JCircle methodsFor: 'graphics' stamp: 'ag 1/12/2011 15:41'!
step
	
	self drawOn: self canvas.! !

!S2JShape methodsFor: 'accessing' stamp: 'sec 12/22/2010 12:03'!
canvas
	^ canvas! !

!S2JShape methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:33:58.38'!
canvas: anObject
	canvas := anObject! !

!S2JShape methodsFor: 'accessing' stamp: 'ag 1/12/2011 10:51'!
color
	^ color! !

!S2JShape methodsFor: 'accessing' stamp: 'ag 1/12/2011 10:51'!
color: anObject
	color := anObject! !

S2JShape subclass: #S2JWorld
	instanceVariableNames: 'bounds elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JWorld subclass: #S2JBallGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-BallGame'!

!S2JBallGame commentStamp: 'sec 1/4/2011 10:22' prior: 0!
usage:
S2JCodeFactory clearCache.
S2JBouncingBallsApp startWebApp.!

!S2JBallGame class methodsFor: 'start' stamp: 'ag 1/24/2011 20:42'!
start
	
	^ self new run! !

!S2JBallGame methodsFor: 'graphics' stamp: 'ag 1/24/2011 20:32'!
addBall
	
	^ self elements add: (S2JBall new canvas: self canvas; yourself)! !

!S2JBallGame methodsFor: 'initialization' stamp: 'ag 1/13/2011 20:02'!
initialize
	
	super initialize.
	self addBall.! !

!S2JWorld methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:22:07.324'!
bounds
	^ bounds! !

!S2JWorld methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:22:07.336'!
bounds: anObject
	bounds := anObject! !

!S2JWorld methodsFor: 'graphics' stamp: 'LT 1/10/2011 18:51:48.444'!
clear
	
	self canvas js setColor: self color.
	self canvas js
		fillRect: {
			self bounds origin x.
			self bounds origin y.
			self bounds corner x.
			self bounds corner y }.! !

!S2JWorld methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:22:07.346'!
elements
	^ elements! !

!S2JWorld methodsFor: 'accessing' stamp: 'LT 12/21/2010 18:22:07.352'!
elements: anObject
	elements := anObject! !

!S2JWorld methodsFor: 'initialization' stamp: 'ag 1/24/2011 20:30'!
initialize
	
	| width height |
	super initialize.
	self color: '#0000ff'.
	width := JsGlobal js number: (JsDocument js getElementById: #ballCanvas) js width.
	height := JsGlobal js number: (JsDocument js getElementById: #ballCanvas) js height.
	
	self
		elements: OrderedCollection new;
		bounds: (Rectangle origin: (0@0) corner: (width@height));
		canvas: (JsGlobal js ctk js Graphics jsNew: (JsGlobal js document js getElementById: #ballCanvas));
		clear.! !

!S2JWorld methodsFor: 'graphics' stamp: 'ag 1/24/2011 20:35'!
run
	
	JsGlobal js repeatWithPause: { [ self step ]. 3 }.! !

!S2JWorld methodsFor: 'graphics' stamp: 'ag 1/12/2011 10:55'!
step
	
	self clear.
	self elements do: [ :each | each step].! !

Object subclass: #S2JWidget
	instanceVariableNames: 'node children extent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JWidget subclass: #S2JButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JButton class methodsFor: 'instance creation' stamp: 'ag 1/26/2011 15:53'!
doing: aBlock
	
	^ self new onClickDo: aBlock; yourself! !

!S2JButton class methodsFor: 'instance creation' stamp: 'ag 1/26/2011 16:13'!
doing: aBlock withText: aString
	
	^ self new
		onClickDo: aBlock;
		text: aString;
		yourself! !

!S2JButton class methodsFor: 'configuration' stamp: 'SW 1/24/2011 14:05'!
htmlElementType
	
	^ 'input'! !

!S2JButton methodsFor: 'rendering' stamp: 'LT 1/26/2011 17:14:26.561'!
disable
	
	self node js disabled js: true! !

!S2JButton methodsFor: 'rendering' stamp: 'LT 1/26/2011 17:14:22.009'!
enable
	
	self node js disabled js: false! !

!S2JButton methodsFor: 'initialization' stamp: 'lw 1/25/2011 18:38'!
initialize 
	
	super initialize.
	self node js type js: 'button'.

	! !

!S2JButton methodsFor: 'rendering' stamp: 'LT 1/26/2011 10:50:54.157'!
onClickDo: aBlockClosure

	self node js onclick js: aBlockClosure.! !

!S2JButton methodsFor: 'rendering' stamp: 'SW 1/24/2011 13:44'!
text: aString
	
	self node js value js: aString
	! !

S2JButton subclass: #S2JSaveButton
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JSaveButton class methodsFor: 'instance creation' stamp: 'LT 1/26/2011 10:40:51.535'!
newOn: aBrowser

	^self basicNew
		browser: aBrowser;
		initialize;
		yourself.! !

!S2JSaveButton methodsFor: 'accessing' stamp: 'LT 1/26/2011 10:41:10.614'!
browser
	^ browser! !

!S2JSaveButton methodsFor: 'accessing' stamp: 'LT 1/26/2011 10:41:10.624'!
browser: anObject
	browser := anObject! !

!S2JSaveButton methodsFor: 'save source' stamp: 'lw 2/2/2011 23:32'!
compileSource: source of: className 

	[ :cls :src | (Smalltalk classNamed: cls) compile: src ] onServer value: className value: source.! !

!S2JSaveButton methodsFor: 'initialization' stamp: 'LT 1/26/2011 17:11:26.978'!
initialize

	super initialize.
	self
		text: 'Save';
		onClickDo: [ self saveSourceCode];
		extent: 200@20! !

!S2JSaveButton methodsFor: 'save source' stamp: 'LT 1/26/2011 15:37:40.441'!
isInputValidWith: source and: className

	| errorText |
	errorText := ''.
	"does anyone know how to do newline in JS?"
	(source isEmpty) ifTrue: [errorText := 'No source input.', [ String cr ] onServer value].
	(className isEmpty) ifTrue: [errorText := errorText, 'No class selected.'].
	errorText isEmpty ifFalse: [JsGlobal js alert: errorText].
	^ errorText isEmpty! !

!S2JSaveButton methodsFor: 'save source' stamp: 'LT 1/26/2011 16:44:29.114'!
saveSourceCode

	| source className |
	source := self browser codeBox text.
	className := self browser classList selectedClassName.
	(self isInputValidWith: source and: className)
		ifTrue: [ self compileSource: source of: className].! !

S2JWidget subclass: #S2JClassBrowser
	instanceVariableNames: 'packageList protocolList classList methodList saveButton codeBox sideSelector spinner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JClassBrowser class methodsFor: 'configuration' stamp: 'robi 1/17/2011 19:05'!
htmlElementType
	
	^ 'div'! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'SW 1/31/2011 14:16'!
addElements

	| lists classContainer |
	classContainer := (S2JContainer vertical
		add: self classList;
		add: self sideSelector).

	lists := (S2JContainer horizontal
		add: self packageList;
		add: classContainer;
		add: self protocolList;
		add: self methodList;
		yourself).
	
	S2JContainer vertical
		add: lists;
		add: self codeBox;
		add: self saveButton;
		yourself;
		addToWorld.
		
	self spinner addToWorld.! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'ag 1/17/2011 18:18'!
classList
	^ classList! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'robi 1/17/2011 19:17'!
classList: anObject
	classList := anObject! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/26/2011 15:33:01.404'!
codeBox

	^ codeBox! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/26/2011 15:33:17.874'!
codeBox: aS2JCodeBox

	codeBox := aS2JCodeBox! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'hk 1/26/2011 16:51'!
createButtons

	self saveButton: (S2JSaveButton newOn: self).! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'SW 1/31/2011 11:05'!
createPanes
	
	self codeBox: (S2JCodeBox newOn: self).
	self methodList: (S2JMethodPane newOn: self).
	self protocolList: (S2JProtocolPane newOn: self).
	self classList: ((S2JClassPane newOn: self) extent: 200 @ 280).
	self packageList: (S2JPackagePane newOn: self).
	self sideSelector: (S2JSideSelector newOn: self).! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'SW 1/31/2011 13:59'!
createSpinner
	
	self spinner: S2JLoader new.				! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'SW 1/31/2011 14:17'!
initialize

	super initialize.
	self 
		createSpinner;
		createPanes; 
		createButtons;
		addElements.! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'ag 1/17/2011 18:18'!
methodList
	^ methodList! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'ag 1/17/2011 18:18'!
methodList: anObject
	methodList := anObject! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/24/2011 14:20:34.638'!
packageList
	^ packageList! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/24/2011 14:20:34.649'!
packageList: anObject
	packageList := anObject! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'sec 1/25/2011 14:12'!
protocolList
	^ protocolList! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'sec 1/25/2011 14:12'!
protocolList: anObject
	protocolList := anObject! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/26/2011 10:43:15.463'!
saveButton
	^ saveButton! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'LT 1/26/2011 10:43:15.474'!
saveButton: anObject
	saveButton := anObject! !

!S2JClassBrowser methodsFor: 'initialization' stamp: 'LT 1/26/2011 16:54:08.361'!
showSide 

	self protocolList update.
	self codeBox displayClassDefinition.! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'sec 1/25/2011 18:05'!
sideSelector
	^ sideSelector! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'sec 1/25/2011 18:05'!
sideSelector: anObject
	sideSelector := anObject! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'SW 1/31/2011 13:03'!
spinner
	^ spinner! !

!S2JClassBrowser methodsFor: 'accessing' stamp: 'SW 1/31/2011 13:03'!
spinner: anObject
	spinner := anObject! !

S2JWidget subclass: #S2JContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JContainer class methodsFor: 'instance creation' stamp: 'hk 1/25/2011 14:46'!
horizontal

	^ S2JHorizontalContainer new! !

!S2JContainer class methodsFor: 'configuration' stamp: 'hk 1/24/2011 10:43'!
htmlElementType

	^ 'div'! !

!S2JContainer class methodsFor: 'configuration' stamp: 'hk 2/1/2011 15:25'!
start

	| group |
	group := S2JRadioGroup newNamed: 'foo'.
	group add: (S2JRadioButton withCaption: 'First' andValue: 'first').
	group add: (S2JRadioButton withCaption: 'Second' andValue: 'second').
	group add: (S2JRadioButton withCaption: 'Third' andValue: 'third').
	
	^ S2JLoader new addToWorld! !

!S2JContainer class methodsFor: 'instance creation' stamp: 'hk 1/24/2011 16:56'!
vertical

	^ S2JVerticalContainer new! !

!S2JContainer methodsFor: 'rendering' stamp: 'hk 1/26/2011 11:39'!
add: element

	super add: element.
	element beRelative! !

S2JContainer subclass: #S2JHorizontalContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JHorizontalContainer methodsFor: 'initialization' stamp: 'hk 1/31/2011 15:09'!
initialize
	
	super initialize.
	self cssClass: 'horizontal'.
! !

S2JContainer subclass: #S2JVerticalContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JVerticalContainer methodsFor: 'initialization' stamp: 'hk 1/31/2011 15:09'!
initialize
	
	super initialize.
	self cssClass: 'vertical'.! !

S2JWidget subclass: #S2JImage
	instanceVariableNames: 'source description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JImage class methodsFor: 'configuration' stamp: 'SW 1/31/2011 12:40'!
htmlElementType

	^ 'img'! !

!S2JImage methodsFor: 'accessing' stamp: 'SW 1/31/2011 12:41'!
description
	^ description! !

!S2JImage methodsFor: 'accessing' stamp: 'SW 1/31/2011 12:44'!
description: anObject

	description := anObject.
	self node js alt js: anObject.
	! !

!S2JImage methodsFor: 'initialization' stamp: 'hk 2/1/2011 12:29'!
initialize 
	
	super initialize.
	self
		source: '';
		description: ''.! !

!S2JImage methodsFor: 'accessing' stamp: 'SW 1/31/2011 12:41'!
source
	^ source! !

!S2JImage methodsFor: 'accessing' stamp: 'SW 1/31/2011 12:43'!
source: anObject

	source := anObject.
	self node js src js: anObject.! !

S2JWidget subclass: #S2JList
	instanceVariableNames: 'options'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JList subclass: #S2JBrowserPane
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JBrowserPane class methodsFor: 'instance creation' stamp: 'sec 1/25/2011 16:07'!
newOn: aBrowser

	^self basicNew
		browser: aBrowser;
		initialize;
		yourself.! !

!S2JBrowserPane methodsFor: 'accessing' stamp: 'sec 1/25/2011 15:29'!
browser
	^ browser! !

!S2JBrowserPane methodsFor: 'accessing' stamp: 'sec 1/25/2011 15:29'!
browser: anObject
	browser := anObject! !

!S2JBrowserPane methodsFor: 'initialization' stamp: 'sec 1/25/2011 15:53'!
initialize
	
	super initialize.
	self
		position: 0 @ 0;
		extent: 200 @ 300.! !

!S2JBrowserPane methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:40'!
update
	
	self clear.! !

S2JBrowserPane subclass: #S2JClassPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JClassPane methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:29'!
clear

	super clear.
	self browser protocolList clear.! !

!S2JClassPane methodsFor: 'rendering' stamp: 'ag 1/26/2011 17:26'!
selectedClassName

	| className |
	className := self browser classList currentSelection.
	(self browser sideSelector currentSelection = 'class') ifTrue: [className := className, ' class'].
	^ className! !

!S2JClassPane methodsFor: 'rendering' stamp: 'SW 1/31/2011 13:49'!
updateForPackage: aPackageName
	
	| packageClassNames |
	self browser spinner show.
	self update.
	packageClassNames := [ :name | Smalltalk allClasses 
			select: [ :each | each category asString = name ]
			thenCollect: [ :eachClass | eachClass name asString ] ]
		onServer value: aPackageName.
	packageClassNames do: 
		[ :each | self
			addString: each 
			onClickDo: 
				[self browser protocolList update.
				self browser codeBox displayClassDefinition ]].
	self browser spinner hide.! !

S2JBrowserPane subclass: #S2JMethodPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JMethodPane methodsFor: 'rendering' stamp: 'LT 1/26/2011 10:54:45.028'!
clear

	super clear.
	self browser codeBox clear.! !

!S2JMethodPane methodsFor: 'rendering' stamp: 'SW 1/31/2011 13:49'!
updateForClass: aClassName in: aProtocolName

	|selectors|
	self browser spinner show.
	self update.
	selectors := [ :className :category || selectorNames class |
			class := Smalltalk classNamed: className.
			selectorNames := class organization
				listAtCategoryNamed: category.
			selectorNames collect: [ :each | each asString ]
		] onServer value: aClassName value: aProtocolName.
	selectors do: [ :selector |
			self
				addString: selector asString
				onClickDo: [ self browser codeBox updateFor: selector of: aClassName]].
	self browser spinner hide.! !

S2JBrowserPane subclass: #S2JPackagePane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JPackagePane methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:53'!
clear

	super clear.
	self browser classList clear.! !

!S2JPackagePane methodsFor: 'initialization' stamp: 'sec 1/25/2011 15:27'!
initialize

	super initialize.
	self update.! !

!S2JPackagePane methodsFor: 'rendering' stamp: 'SW 1/31/2011 13:48'!
update
	
	| allCategoryNames |
	self browser spinner show.
	super update.
	allCategoryNames := [ Smalltalk organization categories
			collect: [ :each | each asString ] ]
		onServer value.
	allCategoryNames do: 
		[ :each | 
			self
				addString: each
				onClickDo: [ self browser classList updateForPackage: each ]].
	self browser spinner hide.			! !

S2JBrowserPane subclass: #S2JProtocolPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JProtocolPane methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:29'!
clear

	super clear.
	self browser methodList clear.! !

!S2JProtocolPane methodsFor: 'rendering' stamp: 'SW 1/31/2011 13:53'!
update
	
	| allCategoryNames className |
	self browser spinner show.
	super update.	
	className := (self browser classList selectedClassName).
	allCategoryNames := [ :class  |
			(Smalltalk classNamed: class) organization categories 
				collect: [ :each | each asString ] ]
		onServer value: className.
	allCategoryNames do: 
		[ :eachCategory | 
			self
				addString: eachCategory
				onClickDo: [ self browser methodList updateForClass: className in: eachCategory  ]].
	self browser spinner hide.! !

!S2JList class methodsFor: 'configuration' stamp: 'lw 1/13/2011 10:38'!
htmlElementType
	^ 'select'! !

!S2JList methodsFor: 'rendering' stamp: 'ag 1/18/2011 14:36'!
addOption: aString
	
	^ self addOption: aString value: aString onClickDo: []! !

!S2JList methodsFor: 'rendering' stamp: 'robi 1/17/2011 19:39'!
addOption: aString value: aValueString
	
	^ self addOption: aString value: aValueString defaultSelected: false selected: false! !

!S2JList methodsFor: 'rendering' stamp: 'LT 1/18/2011 22:15:50.369'!
addOption: anOption value: aValue defaultSelected: isDefaultSelected selected: isSelected
	"Just to have the full HTML-Option-API reflected here."
	
	^self add: (S2JListOption new text: anOption; yourself).! !

!S2JList methodsFor: 'rendering' stamp: 'sec 1/26/2011 12:03'!
addOption: aString value: anotherString onClickDo: aBlock
	
	| anOption |
	anOption := self addOption: aString value: anotherString defaultSelected: false selected: false.
	anOption node js onclick js: [aBlock value: aString].
	^anOption.! !

!S2JList methodsFor: 'rendering' stamp: 'LT 1/18/2011 21:13:49.638'!
addString: aString

	^ self addString: aString onClickDo: []! !

!S2JList methodsFor: 'rendering' stamp: 'sec 1/26/2011 12:23'!
addString: aString onClickDo: aBlock

	^self addOption: aString asString value: aString onClickDo: aBlock! !

!S2JList methodsFor: 'rendering' stamp: 'SW 1/31/2011 11:21'!
collapse

	self node js size js: '1'.
	self node js height js: '20px'.! !

!S2JList methodsFor: 'rendering' stamp: 'sec 1/26/2011 12:21'!
currentSelection

	^JsGlobal js string: self node js value! !

!S2JList methodsFor: 'rendering' stamp: 'LT 1/18/2011 21:12:22.393'!
fillWith: anOrderedCollection
	
	
	
	
"	| text |
	text := anArrayString.
	S2JText new 
		addToWorld;
		text: text;
		position: 10 @ 200."! !

!S2JList methodsFor: 'initialization' stamp: 'LT 1/18/2011 21:50:38.38'!
initialize
	
	super initialize.
	self options: OrderedCollection new.
	self node js size js: '2'.! !

!S2JList methodsFor: 'initialization' stamp: 'sec 1/26/2011 12:10'!
onChange: aBlock

	self node js onchange js: aBlock.! !

!S2JList methodsFor: 'accessing' stamp: 'LT 1/18/2011 21:50:25.125'!
options
	^ options! !

!S2JList methodsFor: 'accessing' stamp: 'LT 1/18/2011 21:50:25.137'!
options: anObject
	options := anObject! !

S2JList subclass: #S2JSideSelector
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JSideSelector class methodsFor: 'instance creation' stamp: 'sec 1/26/2011 14:33'!
newOn: aBrowser

	^self basicNew
		browser: aBrowser;
		initialize;
		yourself.! !

!S2JSideSelector methodsFor: 'accessing' stamp: 'sec 1/25/2011 17:52'!
browser
	^ browser! !

!S2JSideSelector methodsFor: 'accessing' stamp: 'sec 1/25/2011 17:52'!
browser: anObject
	browser := anObject! !

!S2JSideSelector methodsFor: 'initialization' stamp: 'SW 1/31/2011 11:19'!
initialize

	super initialize.
	self collapse.
	self onChange: [ self browser showSide ].
	self addString: 'class'.
	self addString: 'instance'.! !

S2JWidget subclass: #S2JLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JLoader class methodsFor: 'configuration' stamp: 'hk 2/1/2011 15:27'!
defaultImage

	^ 'http://localhost:8887/play/file/images/loader.gif'! !

!S2JLoader class methodsFor: 'configuration' stamp: 'hk 2/1/2011 12:31'!
htmlElementType

	^ 'div'! !

!S2JLoader methodsFor: 'initialization' stamp: 'hk 2/1/2011 15:53'!
initialize

	| image |
	super initialize.
	
	image := S2JImage new
		source: self class defaultImage;
	 	description: 'Loading'.
	self cssClass: 'loader'.
	self add: (S2JParagraph new add: image; yourself).
	self hide.! !

!S2JLoader methodsFor: 'initialization' stamp: 'hk 2/1/2011 15:56'!
show 

	self node js style js display js: 'table'.! !

S2JWidget subclass: #S2JNamedWidget
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JNamedWidget methodsFor: 'accessing' stamp: 'hk 2/1/2011 14:18'!
name
	^ name! !

!S2JNamedWidget methodsFor: 'accessing' stamp: 'hk 2/1/2011 14:18'!
name: anObject
	name := anObject! !

S2JNamedWidget subclass: #S2JRadioButton
	instanceVariableNames: 'caption value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JRadioButton class methodsFor: 'configuration' stamp: 'hk 2/1/2011 10:51'!
htmlElementType

	^ 'input'! !

!S2JRadioButton class methodsFor: 'instance creation' stamp: 'hk 2/1/2011 14:34'!
withCaption: aCaption andValue: aValue

	^ self new caption: aCaption; value: aValue; refreshMarkup! !

!S2JRadioButton methodsFor: 'accessing' stamp: 'hk 2/1/2011 10:54'!
caption
	^ caption! !

!S2JRadioButton methodsFor: 'accessing' stamp: 'hk 2/1/2011 10:54'!
caption: anObject
	caption := anObject! !

!S2JRadioButton methodsFor: 'convenience' stamp: 'hk 2/1/2011 13:56'!
check

	self node js checked js: 'checked'.! !

!S2JRadioButton methodsFor: 'initialization' stamp: 'hk 2/1/2011 14:34'!
initialize 

	super initialize.
	self node js type js: 'radio'.! !

!S2JRadioButton methodsFor: 'convenience' stamp: 'hk 2/1/2011 11:09'!
isChecked

	^ self node js checked! !

!S2JRadioButton methodsFor: 'initialization' stamp: 'hk 2/1/2011 14:35'!
refreshMarkup

	self node js value js: self value.
	self node js name js: self name.! !

!S2JRadioButton methodsFor: 'accessing' stamp: 'hk 2/1/2011 10:56'!
value
	^ value! !

!S2JRadioButton methodsFor: 'accessing' stamp: 'hk 2/1/2011 10:56'!
value: anObject
	value := anObject! !

S2JNamedWidget subclass: #S2JRadioGroup
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JRadioGroup class methodsFor: 'configuration' stamp: 'hk 2/1/2011 10:49'!
htmlElementType

	^ 'div'! !

!S2JRadioGroup class methodsFor: 'instance creation' stamp: 'hk 2/1/2011 11:10'!
newNamed: aName

	^ self new name: aName! !

!S2JRadioGroup methodsFor: 'rendering' stamp: 'hk 2/1/2011 15:23'!
add: aRadioButton

	self children ifEmpty: [ aRadioButton check ].
	self addLabelFor: aRadioButton.
	aRadioButton name: self name; refreshMarkup.
	super add: aRadioButton.! !

!S2JRadioGroup methodsFor: 'rendering' stamp: 'hk 2/1/2011 15:23'!
addLabelFor: aRadioButton

	| label |
	label := S2JLabel new text: aRadioButton caption.
	super add: label.! !

!S2JRadioGroup methodsFor: 'initialization' stamp: 'hk 2/1/2011 14:17'!
initialize

	super initialize.
	self cssClass: 'radio'! !

S2JWidget subclass: #S2JText
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

S2JText subclass: #S2JCodeBox
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-ClassBrowser'!

!S2JCodeBox class methodsFor: 'instance creation' stamp: 'LT 1/26/2011 17:04:37.953'!
newOn: aBrowser

	^self basicNew
		browser: aBrowser;
		initialize;
		yourself.! !

!S2JCodeBox methodsFor: 'accessing' stamp: 'LT 1/26/2011 17:04:49.802'!
browser
	^ browser! !

!S2JCodeBox methodsFor: 'accessing' stamp: 'LT 1/26/2011 17:04:49.812'!
browser: anObject
	browser := anObject! !

!S2JCodeBox methodsFor: 'rendering' stamp: 'ag 1/26/2011 17:26'!
displayClassDefinition

	| definition className |
	self browser saveButton disable.
	className := self browser classList selectedClassName.
	className isEmpty
		ifFalse: 
			[definition := [ :cls | (Smalltalk classNamed: cls) definition ] onServer value: className.
			self text: definition]! !

!S2JCodeBox methodsFor: 'initialization' stamp: 'hk 1/26/2011 17:02'!
initialize 

	super initialize.
	self
		extent: 800 @ 300;
		yourself.! !

!S2JCodeBox methodsFor: 'rendering' stamp: 'SW 1/31/2011 14:20'!
updateFor: aMethodname of: aClassname

	self browser spinner show.
	self clear.
	self browser saveButton enable.
	self text: (
		[ :methodName :classname |
		((Smalltalk classNamed: classname) methodDict at: methodName asSymbol) getSource asString ]
			onServer
				value: aMethodname
				value: aClassname).
	self browser spinner hide.! !

S2JText subclass: #S2JLabel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JLabel class methodsFor: 'configuration' stamp: 'hk 2/1/2011 14:31'!
htmlElementType

	^ 'label'! !

S2JText subclass: #S2JListOption
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JListOption class methodsFor: 'configuration' stamp: 'LT 1/18/2011 22:11:51.562'!
htmlElementType
	^'option'! !

!S2JListOption methodsFor: 'initialization' stamp: 'LT 1/18/2011 22:14:55.606'!
initialize

	self node: (#new Option: { '' js. '' js. false js. false js }).! !

!S2JListOption methodsFor: 'rendering' stamp: 'LT 1/18/2011 22:19:00.748'!
text: aString
	
	self node js innerHTML js: aString.
	self node js value js: aString.
	^self.! !

S2JText subclass: #S2JParagraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Widgets'!

!S2JParagraph class methodsFor: 'configuration' stamp: 'hk 2/1/2011 15:41'!
htmlElementType
	
	^ 'p'! !

!S2JText class methodsFor: 'configuration' stamp: 'robi 1/17/2011 19:41'!
htmlElementType
	
	^ 'textarea'! !

!S2JText class methodsFor: 'instance creation' stamp: 'hk 1/26/2011 11:26'!
with: aString

	^ self new text: aString; yourself! !

!S2JText methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:30'!
clear

	super clear.
	self text: ''! !

!S2JText methodsFor: 'rendering' stamp: 'sec 1/26/2011 14:08'!
text
	
	^ JsGlobal js string: self node js value
	! !

!S2JText methodsFor: 'rendering' stamp: 'LT 1/26/2011 10:53:11.371'!
text: aString
	
	self node js innerHTML js: aString
	! !

!S2JWidget class methodsFor: 'configuration' stamp: 'ag 1/18/2011 09:43'!
htmlElementType
	
	self subclassResponsibility! !

!S2JWidget class methodsFor: 'instance creation' stamp: 'hk 2/1/2011 11:27'!
newFor: anApp

	^ self new app: anApp! !

!S2JWidget class methodsFor: 'configuration' stamp: 'robi 1/26/2011 12:00'!
start
	" this method will only be invoked in case you use your widget as a start app"
	^ self new addToWorld! !

!S2JWidget methodsFor: 'rendering' stamp: 'LT 1/19/2011 09:26:09.951'!
add: anObject
	
	self children add: anObject.
	self node js appendChild: anObject.
	^ anObject! !

!S2JWidget methodsFor: 'rendering' stamp: 'sec 1/25/2011 15:18'!
addAll: aCollection

	aCollection do: [ :each | self add: each ].! !

!S2JWidget methodsFor: 'rendering' stamp: 'ag 1/18/2011 09:46'!
addToWorld
	"Add this widget to a root-div-element."
	
	(JsDocument js getElementById: 'container')
		js appendChild: self.! !

!S2JWidget methodsFor: 'rendering' stamp: 'hk 1/26/2011 11:39'!
beRelative

	self node js style js position js: 'relative'.
	self node js style js top js: '0'.
	self node js style js left js: '0'.! !

!S2JWidget methodsFor: 'accessing' stamp: 'LT 1/18/2011 21:53:58.821'!
children
	^ children! !

!S2JWidget methodsFor: 'accessing' stamp: 'LT 1/18/2011 21:53:58.832'!
children: anObject
	children := anObject! !

!S2JWidget methodsFor: 'rendering' stamp: 'LT 1/18/2011 22:00:29.834'!
clear
	
	self children do: [ :each | self node js removeChild: each node.].
	self children removeAll.! !

!S2JWidget methodsFor: 'rendering' stamp: 'hk 1/31/2011 15:08'!
cssClass: aString

	self node js className js: aString.! !

!S2JWidget methodsFor: 'rendering' stamp: 'hk 1/31/2011 15:10'!
cssId: aString

	self node js id js: aString.! !

!S2JWidget methodsFor: 'accessing' stamp: 'SW 1/31/2011 11:18'!
extent
	
	^ Point x: (self node js width) y: (self node js height)! !

!S2JWidget methodsFor: 'accessing' stamp: 'sec 1/25/2011 18:03'!
extent: anObject
	
	extent := anObject.
	self node js style js height js: anObject y asString , 'px'. 
	self node js style js width js: anObject x asString , 'px'.! !

!S2JWidget methodsFor: 'accessing' stamp: 'hk 2/1/2011 12:38'!
hide 

	self node js style js display js: 'none'.! !

!S2JWidget methodsFor: 'initialization' stamp: 'ag 1/26/2011 16:16'!
initialize
	
	self
		node: (JsDocument js createElement: self class htmlElementType);
		children: OrderedCollection new.! !

!S2JWidget methodsFor: 'initialization' stamp: 'robi 1/17/2011 19:58'!
js
	"The html-node of a widget is unpacked, when the widget is passed as a parameter."
	
	^ self node js! !

!S2JWidget methodsFor: 'accessing' stamp: 'lw 1/13/2011 10:35'!
node
	^ node! !

!S2JWidget methodsFor: 'accessing' stamp: 'ag 1/18/2011 09:45'!
node: anObject
	"Must not be set after initialization"
	
	node := anObject! !

!S2JWidget methodsFor: 'accessing' stamp: 'hk 2/1/2011 11:31'!
pathFor: aFile

	^ self app uniquePath, '/', aFile! !

!S2JWidget methodsFor: 'accessing' stamp: 'hk 1/26/2011 10:55'!
position: anObject
	
	self node js style js position js: 'absolute'.
	self node js style js top js: anObject y asString, 'px'.
	self node js style js left js: anObject x asString , 'px'.! !

!S2JWidget methodsFor: 'accessing' stamp: 'hk 2/1/2011 12:38'!
show 

	self node js style js display js: 'block'.! !

S2JWidget subclass: #S2JWorkspace
	instanceVariableNames: 'textInput'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Squeak2JS-Examples-Workspace'!

!S2JWorkspace class methodsFor: 'configuration' stamp: 'robi 1/26/2011 13:12'!
htmlElementType
	
	^ 'div'! !

!S2JWorkspace methodsFor: 'initialization' stamp: 'ag 1/26/2011 16:41'!
createButtons
	
	^ S2JContainer horizontal addAll: {
		S2JButton doing: [ self printHere ] withText: 'Print here'.
		S2JButton doing: [ self printThere ] withText: 'Print there'.
		"S2JButton doing: [ self printJavaScript ] withText: 'Print JavaScript code'."
		S2JButton doing: [ self doIt ] withText: 'Do it' }! !

!S2JWorkspace methodsFor: 'initialization' stamp: 'ag 1/26/2011 16:33'!
createInput
	
	self textInput:
		(S2JText new
			position: 100@100;
			extent: 810 @ 200).
	^ self textInput! !

!S2JWorkspace methodsFor: 'logic' stamp: 'ag 1/26/2011 17:18'!
doIt
	
	^ S2JServer js performOnServer: self textInput text! !

!S2JWorkspace methodsFor: 'initialization' stamp: 'ag 1/26/2011 16:33'!
initialize
	
	super initialize.
	self add: 
		(S2JContainer vertical
			add: self createInput;
			add: self createButtons;
			yourself).! !

!S2JWorkspace methodsFor: 'logic' stamp: 'ag 1/26/2011 17:18'!
printHere
	
	self textInput text: self input text, ' ', self doIt.! !

!S2JWorkspace methodsFor: 'logic' stamp: 'ag 1/26/2011 16:40'!
printJavaScript
	
	self error: 'not implemented'! !

!S2JWorkspace methodsFor: 'logic' stamp: 'ag 1/26/2011 17:17'!
printThere
	
	^ [ :object | Transcript cr; show: object. nil ]
		onServer
			value: self doIt! !

!S2JWorkspace methodsFor: 'accessing' stamp: 'ag 1/26/2011 16:06'!
textInput
	^ textInput! !

!S2JWorkspace methodsFor: 'accessing' stamp: 'ag 1/26/2011 16:06'!
textInput: anObject
	textInput := anObject! !
